/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QMap>
#include <QStringList>
#include "actionexecutionuserfeedback.h"


QString sh::actions::ActionExecutionUserFeedback::simpleInputBox(QString text, QString deflt)
{
    QString sres = deflt;
    QString proceedstr = QObject::tr("Proceed");
    QString cancelstr = QObject::tr("Cancel");
    int ires = inputBox(text, QStringList({proceedstr, cancelstr}), &sres, "dialog-question", 0, 1);
    return (ires == 0) ? sres : QString();
}

sh::actions::ActionExecutionUserFeedback::MessageBoxButton sh::actions::ActionExecutionUserFeedback::simpleMessageBox(QString text, int buttons, QString icon, sh::actions::ActionExecutionUserFeedback::MessageBoxButton defaultbutton, sh::actions::ActionExecutionUserFeedback::MessageBoxButton cancelbutton)
{
    QStringList buttonlst;
    QList<QString> buttontexts;
    QList<sh::actions::ActionExecutionUserFeedback::MessageBoxButton> buttonroles;
    int defaultanswer = -1;
    int cancelanswer = -1;
    QMap<int, sh::actions::ActionExecutionUserFeedback::MessageBoxButton> results;
    buttontexts.append(QObject::tr("OK")); buttonroles.append(sh::actions::ActionExecutionUserFeedback::MessageBoxButton::OK);
    buttontexts.append(QObject::tr("Continue")); buttonroles.append(sh::actions::ActionExecutionUserFeedback::MessageBoxButton::Continue);
    buttontexts.append(QObject::tr("Yes")); buttonroles.append(sh::actions::ActionExecutionUserFeedback::MessageBoxButton::Yes);
    buttontexts.append(QObject::tr("No")); buttonroles.append(sh::actions::ActionExecutionUserFeedback::MessageBoxButton::No);
    buttontexts.append(QObject::tr("Retry")); buttonroles.append(sh::actions::ActionExecutionUserFeedback::MessageBoxButton::Retry);
    buttontexts.append(QObject::tr("Cancel")); buttonroles.append(sh::actions::ActionExecutionUserFeedback::MessageBoxButton::Cancel);
    for(int i=0; i<buttontexts.length(); i++)
    {
        if (buttons & (int) buttonroles[i])
        {
            results[buttonlst.length()] = buttonroles[i];
            if (buttonroles[i] == defaultbutton)
                defaultanswer = buttonlst.length();
            if (buttonroles[i] == cancelbutton)
                cancelanswer = buttonlst.length();
            buttonlst.append(buttontexts[i]);
        }
    }
    int ires = messageBox(text, buttonlst, icon, defaultanswer, cancelanswer);
    return results[ires];
}


sh::actions::ActionExecutionUserFeedback::GridformEntry::GridformEntry(QString label, sh::ui::feedbackpanels::helpers::Choices choices)
{
    this->label = label;
    this->choices = choices;
}

sh::actions::ActionExecutionUserFeedback::GridformEntry::~GridformEntry()
{
    foreach (sh::ui::feedbackpanels::helpers::Choice* choice, choices.list)
        delete choice;
}

int sh::actions::ActionExecutionUserFeedback::GridformEntry::addChoice(QString label, QString text)
{
    choices.list.append(new sh::ui::feedbackpanels::helpers::Choice(label, text));
    return choices.list.length()-1;
}


sh::actions::ActionExecutionUserFeedback::GridformEntries::GridformEntries()
{
}

sh::actions::ActionExecutionUserFeedback::GridformEntries::~GridformEntries()
{
    foreach (sh::actions::ActionExecutionUserFeedback::GridformEntry* entry, list)
        delete entry;
}

sh::actions::ActionExecutionUserFeedback::GridformEntry* sh::actions::ActionExecutionUserFeedback::GridformEntries::newEntry(QString label)
{
    sh::actions::ActionExecutionUserFeedback::GridformEntry* r = new sh::actions::ActionExecutionUserFeedback::GridformEntry(label, sh::ui::feedbackpanels::helpers::Choices());
    list.append(r);
    return r;
}
