/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ACTIONABSTRACTTRANSFERTREE_H
#define ACTIONABSTRACTTRANSFERTREE_H

#include <QList>
#include "actions/actionactionitem.h"
#include "base/filesystemoperationtransfers.h"


namespace sh{
    namespace base{class Operation;}
    namespace actions{
        namespace common {

            /** \brief
             * Abstract action for transferring (copying, moving, et al) a tree of items.
             */
            class ActionAbstractTransferTree : public sh::actions::ActionActionItem
            {
                Q_OBJECT
            public:
                ActionAbstractTransferTree(QList<std::shared_ptr<const sh::base::Eurl>> sources, std::shared_ptr<const sh::base::Eurl> destination);
                ~ActionAbstractTransferTree();
                static void makereadableunits(double* value, QString* vunit);

            protected:

                class MyFilesystemOperationProgressMonitor : public sh::base::FilesystemOperationProgressMonitor
                {
                public:
                    MyFilesystemOperationProgressMonitor(ActionExecutionInfo* info);
                protected:
                    virtual void changed();
                    virtual void resolveConflicts(QList<sh::base::FilesystemOperationTransfers::OperationStep*> conflictsteps);
                    virtual void respectCancel();
                };

                QList<std::shared_ptr<const sh::base::Eurl> > _sources;
                std::shared_ptr<const sh::base::Eurl> _destination;
            };

        }
    }
}

#endif // ACTIONABSTRACTTRANSFERTREE_H
