/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ACTIONBOOKMARKS_H
#define ACTIONBOOKMARKS_H

#include "actions/actionactionitem.h"
#include "actions/submenuactionitem.h"


namespace sh{
    namespace base { class FilesystemNode; class Eurl; }
    namespace actions{
        namespace common {

            /** \brief
             * Action for navigating to a certain bookmark
             */
            class ActionBookmark : public sh::actions::ActionActionItem
            {
                Q_OBJECT
            public:
                explicit ActionBookmark(QString label, std::shared_ptr<const sh::base::Eurl> eurl);
            protected:
                virtual void action(sh::actions::ActionExecutionInfo* info);
                std::shared_ptr<const sh::base::Eurl> eurl;
            };

            /** \brief
             * Submenu action of bookmarks
             */
            class ActionBookmarkFolder : public sh::actions::SubmenuActionItem
            {
                Q_OBJECT
            public:
                explicit ActionBookmarkFolder(QString label);
                void addSubitem(std::shared_ptr<sh::actions::AbstractActionItem> item);
                void clearSubitems();
            protected:
                QList<std::shared_ptr<sh::actions::AbstractActionItem>> _subs;
                void initialize();
            };

            /** \brief
             * Bookmarking action (the main one you see in the toolbar)
             */
            class ActionBookmarks : public ActionBookmarkFolder
            {
                Q_OBJECT

            public:
                explicit ActionBookmarks();
                void initialize();
            };

            /** \brief
             * Action for managing bookmarks.
             */
            class ActionManageBookmarks : public sh::actions::ActionActionItem
            {
                Q_OBJECT
            public:
                explicit ActionManageBookmarks();
            protected:
                virtual void action(sh::actions::ActionExecutionInfo* info);
            };

            /** \brief
             * Action for bookmarking a directory.
             */
            class ActionAddToBookmarks : public sh::actions::ActionActionItem
            {
                Q_OBJECT
            public:
                explicit ActionAddToBookmarks(QList<std::shared_ptr<sh::base::FilesystemNode> > nodes);
                /** \brief
                 * Only called by the Shallot infrastructure for initialization.
                 */
                static void _initialize();
                /** \brief
                 * Only called by the Shallot infrastructure for initialization.
                 */
                static void _shutdown();
            protected:
                virtual void action(sh::actions::ActionExecutionInfo* info);
                QList<std::shared_ptr<sh::base::FilesystemNode>> nodes;
            };

        }
    }
}

#endif // ACTIONBOOKMARKS_H
