/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ACTIONCLIPBOARD_H
#define ACTIONCLIPBOARD_H

#include <QList>
#include "actions/actionactionitem.h"


namespace sh{
    namespace base{ class Eurl; class FilesystemNode; }
    namespace actions {
        namespace common {

            /** \brief
             * Action for copying to clipboard.
             */
            class ActionClipboardCopy : public sh::actions::ActionActionItem
            {
                Q_OBJECT
            public:
                explicit ActionClipboardCopy(QList<std::shared_ptr<sh::base::FilesystemNode>> nodes);
                /** \brief
                 * Only called by the Shallot infrastructure for initialization.
                 */
                static void _initialize();
                /** \brief
                 * Only called by the Shallot infrastructure for initialization.
                 */
                static void _shutdown();
            protected:
                virtual void action(sh::actions::ActionExecutionInfo* info);
                virtual void initialize();
                QList<std::shared_ptr<sh::base::FilesystemNode>> _nodes;
            };

            /** \brief
             * Action for cutting to clipboard.
             */
            class ActionClipboardCut : public sh::actions::ActionActionItem
            {
                Q_OBJECT
            public:
                explicit ActionClipboardCut(QList<std::shared_ptr<sh::base::FilesystemNode>> nodes);
                /** \brief
                 * Only called by the Shallot infrastructure for initialization.
                 */
                static void _initialize();
                /** \brief
                 * Only called by the Shallot infrastructure for initialization.
                 */
                static void _shutdown();
            protected:
                virtual void action(sh::actions::ActionExecutionInfo* info);
                virtual void initialize();
                QList<std::shared_ptr<sh::base::FilesystemNode>> _nodes;
            };

            /** \brief
             * Action for pasting from clipboard.
             */
            class ActionClipboardPaste : public sh::actions::ActionActionItem
            {
                Q_OBJECT
            public:
                explicit ActionClipboardPaste(QList<std::shared_ptr<sh::base::FilesystemNode>> nodes);
                bool shortcutTriggersOnFolder() { return true; }
                /** \brief
                 * Only called by the Shallot infrastructure for initialization.
                 */
                static void _initialize();
                /** \brief
                 * Only called by the Shallot infrastructure for initialization.
                 */
                static void _shutdown();
            protected:
                virtual void action(sh::actions::ActionExecutionInfo* info);
                virtual void initialize();
                QList<std::shared_ptr<sh::base::FilesystemNode>> _nodes;
            };

            /** \brief
             * Action for pasting from clipboard as link.
             */
            class ActionClipboardPasteAsLink : public sh::actions::ActionActionItem
            {
                Q_OBJECT
            public:
                explicit ActionClipboardPasteAsLink(QList<std::shared_ptr<sh::base::FilesystemNode>> nodes);
                /** \brief
                 * Only called by the Shallot infrastructure for initialization.
                 */
                static void _initialize();
                /** \brief
                 * Only called by the Shallot infrastructure for initialization.
                 */
                static void _shutdown();
            protected:
                virtual void action(sh::actions::ActionExecutionInfo* info);
                virtual void initialize();
                QString _linktarget(std::shared_ptr<const sh::base::Eurl> link, std::shared_ptr<const sh::base::Eurl> destination, bool absolute);
                QList<std::shared_ptr<sh::base::FilesystemNode>> _nodes;
            };

        }
    }
}

#endif // ACTIONCLIPBOARD_H
