/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "actions/actionactionitem.h"
#include "actions/actioncategory.h"
#include "actions/actionfactory.h"
#include "actions/headeractionitem.h"
#include "actions/actionsmanager.h"
#include "actiongroups.h"


sh::actions::common::ActionGroups::ActionGroups(std::shared_ptr<sh::actions::ActionCategory> category)
        : sh::actions::SubmenuActionItem(category->label(), QList<std::shared_ptr<sh::actions::AbstractActionItem>>(), true, category->icon()), category(category)
{
}

void sh::actions::common::ActionGroups::initialize()
{
}

void sh::actions::common::ActionGroups::setGroupActions(std::shared_ptr<sh::actions::ActionInstantiation> rootsituation, QList<std::shared_ptr<sh::actions::ActionInstantiation>> selacts, QList<std::shared_ptr<sh::actions::ActionInstantiation>> diracts)
{
    QList<std::shared_ptr<sh::actions::AbstractActionItem> > actions;
    if (!rootsituation->selectedNodes.isEmpty()) {
        actions.append(std::make_shared<sh::actions::HeaderActionItem>(sh::actions::ActionsManager::selectionLabel(rootsituation->selectedNodes)));
        QList<std::shared_ptr<sh::actions::ActionInstantiation> > _actions;
        for(auto acsit : selacts) {
            if (acsit->category == category)
                _actions.append(acsit);
        }
        sh::actions::ActionsManager::sortAndSeparateActions(&_actions);
        for(auto acs : _actions) {
            if (!acs->shortcut.isEmpty()) {
                auto aact = std::dynamic_pointer_cast<sh::actions::ActionActionItem>(acs->createdAction);
                if (aact)
                    aact->setShortcuthint(acs->shortcut);
            }
            actions.append(acs->createdAction);
        }
    }
    actions.append(std::make_shared<sh::actions::HeaderActionItem>(sh::actions::ActionsManager::directoryLabel(rootsituation->currentDirectory)));
    QList<std::shared_ptr<sh::actions::ActionInstantiation> > _actions;
    for(auto acsit : diracts) {
        if (acsit->category == category)
            _actions.append(acsit);
    }
    sh::actions::ActionsManager::sortAndSeparateActions(&_actions);
    for(auto acs : _actions) {
        if (!acs->shortcut.isEmpty()) {
            auto aact = std::dynamic_pointer_cast<sh::actions::ActionActionItem>(acs->createdAction);
            if (aact)
                aact->setShortcuthint(acs->shortcut);
        }
        actions.append(acs->createdAction);
    }
    setSubitems(actions);
}
