/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "actions/actionfactory.h"
#include "base/filesystemmodel.h"
#include "base/filesystemnode.h"
#include "ui/fileview/fileviewcontrol.h"
#include "ui/filesystempanel.h"
#include "ui/fileview/fileview.h"
#include "ui/mainwindow.h"
#include "tools/iconmanager.h"
#include "tools/mainthread.h"
#include "actionnavigation.h"


sh::actions::common::ActionNavigateInHistory::ActionNavigateInHistory(sh::ui::fileview::FileViewControl *filelist, int idx, std::shared_ptr<const sh::base::Eurl> eurl, bool isdefault) :
    sh::actions::ActionActionItem(eurl->asString(), true, QString(), isdefault?1:0), _filelist(filelist), _idx(idx), _eurl(eurl)
{
}

void sh::actions::common::ActionNavigateInHistory::action(sh::actions::ActionExecutionInfo*)
{
    sh::tools::MainThread::dispatcher()->invokeSync([&](){
        _filelist->historyTracker()->revisitValue(_idx);
        sh::ui::MainWindow::mainWindow()->jumpToEurl(_eurl);
    });
}

sh::actions::common::ActionHistoryNavigate::ActionHistoryNavigate(QString text, std::function<QList<sh::tools::HistoryTracker<std::shared_ptr<const sh::base::Eurl>>::HistoryEntry>()> _getentriesfct,
                                             QString icon) :
    sh::actions::SubmenuActionItem(text, QList<std::shared_ptr<sh::actions::AbstractActionItem>>(), true, icon),
    getentriesfct(_getentriesfct)
{
    connect(sh::ui::MainWindow::mainWindow(), &sh::ui::MainWindow::currentDirectoryChanged, this, [=](){
        reinitialize();
    });
}

void sh::actions::common::ActionHistoryNavigate::initialize()
{
    sh::tools::MainThread::dispatcher()->invokeSync([&](){
        bool isempty = true;
        QList<std::shared_ptr<sh::actions::AbstractActionItem> > subitems;
        if (!sh::ui::MainWindow::mainWindow()->filesystemPanel())
            return;  // otherwise we crash by some ways of closing Shallot
        foreach(sh::tools::HistoryTracker<std::shared_ptr<const sh::base::Eurl>>::HistoryEntry entry, getentriesfct())
        {
            std::shared_ptr<sh::actions::AbstractActionItem> suba(new sh::actions::common::ActionNavigateInHistory(sh::ui::MainWindow::mainWindow()->filesystemPanel()->currentView(),
                                                                                 entry.index(), entry.value(), isempty));
            subitems.append(suba);
            isempty = false;
        }
        if (isempty)
            setEnabled(false);
        setSubitems(subitems);
    });
}

sh::actions::common::ActionHistoryNavigateForward::ActionHistoryNavigateForward() :
    sh::actions::common::ActionHistoryNavigate("&F", [=](){ return sh::ui::MainWindow::mainWindow()->filesystemPanel()->currentView()->historyTracker()->forwardList(); }, "go-next")
{

}

sh::actions::common::ActionHistoryNavigateBackward::ActionHistoryNavigateBackward() :
    sh::actions::common::ActionHistoryNavigate("&B", [=](){ return sh::ui::MainWindow::mainWindow()->filesystemPanel()->currentView()->historyTracker()->backwardList(); }, "go-previous")
{

}
