/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "actions/actionfactory.h"
#include "base/eurl.h"
#include "base/filesystemnode.h"
#include "tools/mainthread.h"
#include "ui/mainwindow.h"
#include "actionopenarchive.h"


sh::actions::common::ActionOpenArchive::ActionOpenArchive(QList<std::shared_ptr<sh::base::FilesystemNode> > nodes) :
        sh::actions::ActionActionItem("", true, QString(), sh::actions::ActionDefaultPrecedenceValues::PRECEDENCE_BUILTIN_SPECIAL_OPEN)
{
    _nodes = nodes;
}

void sh::actions::common::ActionOpenArchive::initialize()
{
    setText(label());
}

void sh::actions::common::ActionOpenArchive::action(sh::actions::ActionExecutionInfo*)
{
    foreach(std::shared_ptr<sh::base::FilesystemNode> node, _nodes)
    {
        sh::tools::MainThread::dispatcher()->invokeSync([&](){
            std::shared_ptr<const sh::base::Eurl> neurl = node->eurl()->enwrapWithOuterUrl(scheme(),"","/");
            sh::ui::MainWindow::mainWindow()->jumpToEurl(neurl);
        });
    }
}
