/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QApplication>
#include "actions/actionexecutioninfo.h"
#include "actions/actionfactory.h"
#include "actions/actionsmanager.h"
#include "base/eurl.h"
#include "base/filesystemnode.h"
#include "base/operation.h"
#include "tools/singletoninitializer.h"
#include "actionopendirectoryinnewwindow.h"


namespace sh{ namespace actions{ namespace common{INITIALIZE_STATIC_IN_MAINTHREAD(10000, ActionOpenDirectoryInNewWindow)}}}

void sh::actions::common::ActionOpenDirectoryInNewWindow::_initialize()
{
    sh::actions::ActionsManager::instance()->registerActionFactory(
                std::make_shared<sh::actions::ActionFactory<sh::actions::common::ActionOpenDirectoryInNewWindow>>(
                    sh::actions::ActionCategory::getByName("open"), QList<std::shared_ptr<Predicate>>({
                    std::make_shared<sh::actions::HideOnSelectionLevelPredicate>(),
                    std::make_shared<sh::actions::PositionIndexPredicate>(-300000),
                    std::make_shared<sh::actions::KeyShortcutPredicate>(QKeySequence(Qt::CTRL | Qt::Key_N), true),
    })));
}

void sh::actions::common::ActionOpenDirectoryInNewWindow::_shutdown()
{
}

sh::actions::common::ActionOpenDirectoryInNewWindow::ActionOpenDirectoryInNewWindow(QList<std::shared_ptr<sh::base::FilesystemNode>> nodes)
    :sh::actions::ActionActionItem(tr("Open in new window..."))
{
    _nodes = nodes;
}

void sh::actions::common::ActionOpenDirectoryInNewWindow::action(sh::actions::ActionExecutionInfo*)
{
    foreach(std::shared_ptr<sh::base::FilesystemNode> node, _nodes)
        QProcess::startDetached(QApplication::instance()->applicationFilePath(), {node->eurl()->asString()});
}

