/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "actions/actionfactory.h"
#include "actions/actionsmanager.h"
#include "base/eurl.h"
#include "base/filesystemnode.h"
#include "tools/mainthread.h"
#include "tools/singletoninitializer.h"
#include "ui/mainwindow.h"
#include "actionopensharcarchive.h"


namespace sh{ namespace actions{ namespace common{INITIALIZE_STATIC_IN_MAINTHREAD(10000, ActionOpenSharcArchive)}}}

void sh::actions::common::ActionOpenSharcArchive::_initialize()
{
    sh::actions::ActionsManager::instance()->registerActionFactory(
                std::make_shared<sh::actions::ActionFactory<sh::actions::common::ActionOpenSharcArchive>>(
                    sh::actions::ActionCategory::getByName("open"), QList<std::shared_ptr<Predicate>>({
                    std::make_shared<sh::actions::OnFilesPredicate>(),
                    std::make_shared<sh::actions::ByRegExpPredicate>(".*\\.sharc")
    })));
}

void sh::actions::common::ActionOpenSharcArchive::_shutdown()
{
}

sh::actions::common::ActionOpenSharcArchive::ActionOpenSharcArchive(QList<std::shared_ptr<base::FilesystemNode> > nodes) :
        sh::actions::ActionActionItem( tr("Open Sharc archive"), true, QString(), 0
                          , sh::actions::ActionDefaultPrecedenceValues::PRECEDENCE_BUILTIN_SPECIAL_OPEN)
{
    _nodes = nodes;
}

void sh::actions::common::ActionOpenSharcArchive::action(sh::actions::ActionExecutionInfo*)
{
    foreach(std::shared_ptr<sh::base::FilesystemNode> node, _nodes)
    {
        sh::tools::MainThread::dispatcher()->invokeSync([&](){
            std::shared_ptr<const sh::base::Eurl> neurl = node->eurl()->enwrapWithOuterUrl("sharc", "", "/");
            sh::ui::MainWindow::mainWindow()->jumpToEurl(neurl);
        });
    }
}
