/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QProcess>
#include "tools/iconmanager.h"
#include "ui/mainwindow.h"
#include "actionopenterminal.h"


sh::actions::common::ActionOpenTerminal::ActionOpenTerminal(QString dir) :
    sh::actions::SubmenuActionItem(tr("&Terminal"), QList<std::shared_ptr<sh::actions::AbstractActionItem>>(), true, "utilities-terminal")
{
    workdir = dir;
    setEnabled( !dir.isNull() );
}

void sh::actions::common::ActionOpenTerminal::initialize()
{
    QList<std::shared_ptr<sh::actions::AbstractActionItem>> subitems;
    subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::common::ActionOpenTerminalAsUser(workdir)));
    subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::common::ActionOpenTerminalAsRoot(workdir)));
    setSubitems(subitems);
}

sh::actions::common::ActionOpenTerminalAsUser::ActionOpenTerminalAsUser(QString dir)
    : sh::actions::ActionActionItem(tr("Open Terminal as &user"), true, "utilities-terminal", 1)
{
    workdir = dir;
    setEnabled( !dir.isNull() );
    setShortcuthint(QKeySequence(Qt::CTRL + Qt::Key_T), true);
}


void sh::actions::common::ActionOpenTerminalAsUser::action(sh::actions::ActionExecutionInfo*)
{
    bool running = false;
    QStringList terminals;
#ifdef Q_OS_WIN
    terminals << "cmd.exe";
#else
    terminals << "xdg-terminal" << "x-terminal-emulator" << "konsole" << "gnome-terminal" << "xterm";
#endif
    while(!running && !terminals.isEmpty()) {
        QString term = terminals.takeFirst();
        QStringList args;
        if (term=="konsole")
            args << "--workdir" << workdir;
        foreach(QString a, additionalArgs)
            args << a;
        running = QProcess::startDetached(term, args, workdir);
    }
}

sh::actions::common::ActionOpenTerminalAsRoot::ActionOpenTerminalAsRoot(QString dir)
    : sh::actions::common::ActionOpenTerminalAsUser(dir)
{
    setText(tr("Open Terminal as &root"));
    setShortcuthint( QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_T), true);
    additionalArgs << "-e" << "su";
#ifdef Q_OS_WIN
    setEnabled(false);
#endif
}

