/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ACTIONOPENTERMINAL_H
#define ACTIONOPENTERMINAL_H

#include "actions/actionactionitem.h"
#include "actions/submenuactionitem.h"


namespace sh{
    namespace actions{
        namespace common {

            /** \brief
             * Action submenu with actions for opening commandline terminals.
             */
            class ActionOpenTerminal : public sh::actions::SubmenuActionItem
            {
                Q_OBJECT

            public:
                explicit ActionOpenTerminal(QString dir);
                void initialize();

            protected:
                QString workdir;
            };

            /** \brief
             * Action for opening a commandline terminal (with current user).
             */
            class ActionOpenTerminalAsUser : public sh::actions::ActionActionItem
            {
                Q_OBJECT
            public:
                explicit ActionOpenTerminalAsUser(QString dir);

            protected:

                QStringList additionalArgs;
                virtual void action(sh::actions::ActionExecutionInfo* info);
                bool shortcutTriggersOnFolder() { return true; }
                QString workdir;
            };

            /** \brief
             * Action for opening a commandline terminal as root.
             */
            class ActionOpenTerminalAsRoot : public ActionOpenTerminalAsUser
            {
                Q_OBJECT
            public:
                explicit ActionOpenTerminalAsRoot(QString dir);

            protected:
                QString workdir;
            };

        }
    }
}

#endif // ACTIONOPENTERMINAL_H
