/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "actions/actionexecutioninfo.h"
#include "actions/actionfactory.h"
#include "actions/actionsmanager.h"
#include "base/eurl.h"
#include "base/operation.h"
#include "base/filesystemoperation.h"
#include "base/filesystemnode.h"
#include "tools/mainthread.h"
#include "tools/singletoninitializer.h"
#include "ui/mainwindow.h"
#include "ui/filepropertydialog/filepropertydialog.h"
#include "actionshowproperties.h"


namespace sh{ namespace actions{ namespace common{INITIALIZE_STATIC_IN_MAINTHREAD(10000, ActionShowProperties)}}}

void sh::actions::common::ActionShowProperties::_initialize()
{
    sh::actions::ActionsManager::instance()->registerActionFactory(
                std::make_shared<sh::actions::ActionFactory<sh::actions::common::ActionShowProperties>>(
                    sh::actions::ActionCategory::getByName("manage"), QList<std::shared_ptr<Predicate>>({
                    std::make_shared<sh::actions::DontResolveLinksPredicate>(),
                    std::make_shared<sh::actions::OnSingleEntrySelectionPredicate>(),
                    std::make_shared<sh::actions::PositionIndexPredicate>(-500000),
                    std::make_shared<sh::actions::KeyShortcutPredicate>(QKeySequence(Qt::CTRL|Qt::SHIFT|Qt::Key_P), false)
    })));
}

void sh::actions::common::ActionShowProperties::_shutdown()
{
}

sh::actions::common::ActionShowProperties::ActionShowProperties(QList<std::shared_ptr<sh::base::FilesystemNode>> nodes)
    :sh::actions::ActionActionItem( tr("Properties..."), true, QString())
{
    _nodes = nodes;
}

void sh::actions::common::ActionShowProperties::initialize()
{
}

void sh::actions::common::ActionShowProperties::action(sh::actions::ActionExecutionInfo*)
{
    sh::tools::MainThread::dispatcher()->invokeSync([&](){
        sh::ui::filepropertydialog::FilePropertyDialog::showForNodes(_nodes);
    });
}
