/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "actions/actionexecutioninfo.h"
#include "actions/separatoractionitem.h"
#include "actions/actionexecutionuserfeedback.h"
#include "actions/mainmenu/actionabout.h"
#include "actions/mainmenu/actionapplyprofile.h"
#include "actions/mainmenu/actionhelp.h"
#include "actions/mainmenu/actionmanagesavedsettings.h"
#include "actions/mainmenu/actionquit.h"
#include "actions/mainmenu/actionrefresh.h"
#include "actions/mainmenu/actionsavesettings.h"
#include "actions/mainmenu/actionsetwindowtitlepattern.h"
#include "actions/mainmenu/actionshowlog.h"
#include "actions/mainmenu/actiontuning.h"
#include "actions/mainmenu/actionsearch.h"
#include "base/eurl.h"
#include "base/operation.h"
#include "base/filesystemhandler.h"
#include "base/filesystemnode.h"
#include "exceptions/permissiondeniedexception.h"
#include "tools/singletoninitializer.h"
#include "tools/mainthread.h"
#include "tools/threadpool.h"
#include "ui/mainwindow.h"
#include "ui/filesystempanel.h"
#include "actionmain.h"


namespace sh{namespace actions{namespace mainmenu{INITIALIZE_SINGLETON_IN_MAINTHREAD(20210, ActionMain)}}}

std::shared_ptr<sh::actions::mainmenu::ActionMain> sh::actions::mainmenu::ActionMain::instance()
{
    if (!_instance)
        _instance = std::shared_ptr<sh::actions::mainmenu::ActionMain>(new sh::actions::mainmenu::ActionMain());
    return _instance;
}

std::shared_ptr<sh::actions::mainmenu::ActionMain> sh::actions::mainmenu::ActionMain::_instance = 0;

sh::actions::mainmenu::ActionMain::ActionMain()
    : sh::actions::SubmenuActionItem("&S", QList<std::shared_ptr<sh::actions::AbstractActionItem>>(), true, "configure")
{
}

void sh::actions::mainmenu::ActionMain::initialize()
{
    QList<std::shared_ptr<sh::actions::AbstractActionItem>> subitems;
    sh::tools::MainThread::dispatcher()->invokeSync([&](){
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::mainmenu::ActionSaveSettings()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::mainmenu::ActionManageSavedSettings()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::mainmenu::ActionApplyProfile()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::mainmenu::ActionTuning()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::SeparatorActionItem()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::mainmenu::ActionSetWindowTitlePattern()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::SeparatorActionItem()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::mainmenu::ActionRefresh()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::mainmenu::ActionSearch()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::SeparatorActionItem()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::mainmenu::ActionShowLog()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::mainmenu::ActionAbout()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::mainmenu::ActionHelp()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::SeparatorActionItem()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::mainmenu::ActionQuit()));
    });
    setSubitems(subitems);
}

void sh::actions::mainmenu::ActionMain::_initialize()
{
}

void sh::actions::mainmenu::ActionMain::_shutdown()
{
}
