/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "actions/actionexecutioninfo.h"
#include "base/filesystemnode.h"
#include "base/eurl.h"
#include "settings/settingsmanager.h"
#include "tools/mainthread.h"
#include "ui/mainwindow.h"
#include "ui/filesystempanel.h"
#include "ui/fileview/fileview.h"
#include "ui/fileview/fileviewcontrol.h"
#include "ui/storeprofiledialog.h"
#include "actionsavesettings.h"


sh::actions::mainmenu::ActionSaveSettings::ActionSaveSettings()
    : sh::actions::ActionActionItem(tr("&Save settings..."), true, "format-insert-node")
{
}

void sh::actions::mainmenu::ActionSaveSettings::action(sh::actions::ActionExecutionInfo* info)
{
    info->setVisualProcessFeedbackActive(false);
    sh::tools::MainThread::dispatcher()->invokeSync([&](){
        sh::exceptions::Exception::executeGuarded([&](){
            sh::ui::fileview::FileViewControl* currentfilelist = sh::ui::MainWindow::mainWindow()->filesystemPanel()->currentView();
            std::shared_ptr<const sh::base::Eurl> currenteurl = currentfilelist->node()->eurl();
            sh::ui::StoreProfileDialog dialog( currenteurl->asString() );
            int dialogresult = dialog.exec();
            if (dialogresult == QDialog::Accepted)
            {
                QMap<QString, QString> vals;
                QMap<QString, int> onlyinfileviewindex;
                QList<sh::ui::StoreProfileDialog::CheckedSetting> checked = dialog.getCheckedSettings();
                QString profilename = dialog.profilename();
                foreach(sh::ui::StoreProfileDialog::CheckedSetting setting, checked)
                {
                    QString sname = setting.setting->name();
                    QString sval = setting.setting->getValue(currentfilelist);
                    if (!sval.isNull()){
                        vals[sname] = sval;
                        onlyinfileviewindex[sname] = setting.onlyInFileview;
                    }
                }
                if (dialog.hasGlobal())
                    currenteurl = 0;
                bool subfolders = dialog.withSubdirectories();
                QStringList inheritsFrom = dialog.inheritsFrom();
                sh::settings::SettingsManager::instance()->storeProfile(currenteurl, profilename, vals, onlyinfileviewindex, subfolders, inheritsFrom );
            }
        }, sh::exceptions::ExecuteGuardFlag::RESUMEABLE_PROGRAM_ERROR_END_WITH_USER_FEEDBACK_HERE);
    });
}
