/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "actions/actionexecutioninfo.h"
#include "base/filesystemnode.h"
#include "base/eurl.h"
#include "tools/mainthread.h"
#include "ui/mainwindow.h"
#include "ui/filesystempanel.h"
#include "ui/fileview/fileview.h"
#include "search/searchmanager.h"
#include "search/search.h"
#include "actionsearch.h"


sh::actions::mainmenu::ActionSearch::ActionSearch()
    : sh::actions::ActionActionItem(tr("&Search"), true, "edit-find")
{
    setShortcuthint(QKeySequence(Qt::CTRL|Qt::Key_F));
}

void sh::actions::mainmenu::ActionSearch::action(sh::actions::ActionExecutionInfo* info)
{
    info->setVisualProcessFeedbackActive(false);
    sh::tools::MainThread::dispatcher()->invokeSync([&](){
        std::shared_ptr<sh::base::FilesystemNode> searchrootnode = sh::ui::MainWindow::mainWindow()->filesystemPanel()->viewwidget(sh::ui::MainWindow::mainWindow()->filesystemPanel()->currentViewIndex())->node();
        if (searchrootnode->eurl()->scheme() == "search")
        {
            std::shared_ptr<sh::base::FilesystemNode> n = searchrootnode;
            while (n && n->eurl()->scheme() == "search")
                n = n->parentnode();
            if (n)
                sh::ui::MainWindow::mainWindow()->jumpToNode(n);
            sh::search::SearchManager::instance()->removeSearch(sh::search::SearchManager::instance()->findSearch(searchrootnode->eurl()->root()));
        }
        else
        {
            std::shared_ptr<sh::search::Search> search = sh::search::SearchManager::instance()->requestSearch(searchrootnode->eurl(), "", searchrootnode);
            sh::ui::MainWindow::mainWindow()->jumpToNode(search->searchnode());
        }
    });
}
