/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "actions/actionexecutioninfo.h"
#include "actions/actionexecutionuserfeedback.h"
#include "tools/mainthread.h"
#include "ui/mainwindow.h"
#include "ui/filesystempanel.h"
#include "ui/fileview/fileview.h"
#include "actionsetwindowtitlepattern.h"


sh::actions::mainmenu::ActionSetWindowTitlePattern::ActionSetWindowTitlePattern()
    : sh::actions::ActionActionItem(tr("Set &window title pattern..."), true)
{
}

void sh::actions::mainmenu::ActionSetWindowTitlePattern::action(sh::actions::ActionExecutionInfo* info)
{
    info->setHead(tr("Setting window title pattern"));
    QString oldval;
    sh::tools::MainThread::dispatcher()->invokeSync([&](){
        oldval = sh::ui::MainWindow::mainWindow()->getTitlePattern();
    });
    QString res = info->userfeedback()->simpleInputBox(tr("Specify the pattern for controlling the title of the main window.\n{dir}: full path of current dir.\n{dirname}: name of current dir."), oldval);
    if (!res.isNull())
        sh::tools::MainThread::dispatcher()->invokeSync([&](){
            sh::ui::MainWindow::mainWindow()->setTitlePattern(res);
        });
}
