/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "submenuactionitem.h"


sh::actions::SubmenuActionItem::SubmenuActionItem(QString text, const QList< std::shared_ptr<sh::actions::AbstractActionItem> > subitems, bool enabled
            , QString icon, int defaultActionPrecedence, bool checkable, bool ischecked) :
    sh::actions::AbstractActionItem(text, enabled, icon, defaultActionPrecedence, checkable, ischecked)
{
    _subitems = subitems;
}

const QList<std::shared_ptr<sh::actions::AbstractActionItem> > sh::actions::SubmenuActionItem::subitems()
{
    QMutexLocker _l(&mutex);
    return _subitems;
}

void sh::actions::SubmenuActionItem::setSubitems(const QList<std::shared_ptr<sh::actions::AbstractActionItem> > subitems)
{
    {
        QMutexLocker _l(&mutex);
        _subitems = subitems;
        foreach(std::shared_ptr<sh::actions::AbstractActionItem> subitem, subitems)
            subitem->_setParentAction(shared_from_this());
    }
    emit subitemsChanged();
    if (isInitialized() || isInitializing()) // initialize the subitems if we are initialized (or initializing)
        postinitialize();
}

void sh::actions::SubmenuActionItem::postinitialize()
{
    mutex.lock();
    QList<std::shared_ptr<sh::actions::AbstractActionItem> > __subitems = _subitems;
    mutex.unlock();
    foreach(std::shared_ptr<sh::actions::AbstractActionItem> subitem, __subitems)
    {
        subitem->initializeAsync();
    }
}

