/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SUBMENUACTIONITEM_H
#define SUBMENUACTIONITEM_H

#include <memory>
#include "abstractactionitem.h"


namespace sh{
    namespace actions{

        /** \brief
         * Abstract base class for a submenu.
         *
         * It can be made visible in menus or the toolbar or executed directly.
         *
         * Subclasses fill the submenu with other actions in order to be useful.
         */
        class SubmenuActionItem : public sh::actions::AbstractActionItem
        {
            Q_OBJECT
        public:
            /** \brief Is (for subclasses) intended to be directly constructed from everywhere or by
             * registering a factory somewhere.
             */
            SubmenuActionItem(QString text, const QList<std::shared_ptr<sh::actions::AbstractActionItem> > subitems, bool enabled = true
                    , QString icon = QString(), int defaultActionPrecedence = 0, bool checkable = false, bool ischecked = false);
            /** \brief
             * Returns the list of subitems from this submenu.
             */
            const QList< std::shared_ptr<sh::actions::AbstractActionItem> > subitems();
            /** \brief
             * Sets the subitems.
             */
            void setSubitems(const QList< std::shared_ptr<sh::actions::AbstractActionItem> > subitems);

        signals:
            /** \brief
             * Emits when the list of subitems changed.
             */
            void subitemsChanged();

        public slots:

        protected:
            virtual void postinitialize();

        private:
            QList<std::shared_ptr<sh::actions::AbstractActionItem> > _subitems;
        };

    }
}

#endif // SUBMENUACTIONITEM_H
