/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "tools/mainthread.h"
#include "ui/mainwindow.h"
#include "ui/filesystempanel.h"
#include "ui/fileview/fileview.h"
#include "ui/fileview/fileviewcontrol.h"
#include "actionfileviewchangeicondimension.h"


sh::actions::viewmenu::ActionFileViewChangeIconDimension::ActionFileViewChangeIconDimension(QString name, int direction, QString icon, QKeySequence shortcut)
    : sh::actions::ActionActionItem(name, true, icon), _direction(direction)
{
    setShortcuthint(shortcut);
    connect(sh::ui::MainWindow::mainWindow(), &sh::ui::MainWindow::viewOptionChanged, this, [=](){
        configure();
    });
    connect(sh::ui::MainWindow::mainWindow()->filesystemPanel(), &sh::ui::FilesystemPanel::currentViewChanged, this, [=](){
        configure();
    });
    configure();
}

void sh::actions::viewmenu::ActionFileViewChangeIconDimension::configure()
{
    setVisible(sh::ui::MainWindow::mainWindow()->filesystemPanel()->currentView()->viewmode() == sh::ui::fileview::FileViewMode::IconView);
}

void sh::actions::viewmenu::ActionFileViewChangeIconDimension::action(sh::actions::ActionExecutionInfo*)
{
    sh::tools::MainThread::dispatcher()->invokeSync([&](){
        auto v = sh::ui::MainWindow::mainWindow()->filesystemPanel()->currentView();
        v->setIconDimension(v->iconDimension() + _direction);
    });
}

sh::actions::viewmenu::ActionFileViewIncreaseIconDimension::ActionFileViewIncreaseIconDimension()
    : sh::actions::viewmenu::ActionFileViewChangeIconDimension(tr("&Zoom in"), 1, "zoom-in", QKeySequence(Qt::CTRL | Qt::Key_Plus))
{
    connect(sh::ui::MainWindow::mainWindow()->filesystemPanel(), &sh::ui::FilesystemPanel::viewOptionChanged, this, [=](int){
        configure();
    });
    configure();
}


sh::actions::viewmenu::ActionFileViewDecreaseIconDimension::ActionFileViewDecreaseIconDimension()
    : sh::actions::viewmenu::ActionFileViewChangeIconDimension(tr("Zoo&m out"), -1, "zoom-out", QKeySequence(Qt::CTRL | Qt::Key_Minus))
{
    connect(sh::ui::MainWindow::mainWindow()->filesystemPanel(), &sh::ui::FilesystemPanel::viewOptionChanged, this, [=](int){
        configure();
    });
    configure();
}
