/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "tools/mainthread.h"
#include "ui/mainwindow.h"
#include "ui/filesystempanel.h"
#include "ui/fileview/fileviewcontrol.h"
#include "actionfileviewview.h"


sh::actions::viewmenu::ActionFileviewView::ActionFileviewView()
    : sh::actions::ActionActionItem(tr("Show as &icons"), true, QString(), 0, true)
{
    setShortcuthint(QKeySequence(Qt::CTRL|Qt::SHIFT|Qt::Key_L));
    connect(sh::ui::MainWindow::mainWindow()->filesystemPanel(), &sh::ui::FilesystemPanel::viewmodeChanged, this, [=](int){
        configure();
    });
    configure();
}

void sh::actions::viewmenu::ActionFileviewView::configure()
{
    _setAsIcons = sh::ui::MainWindow::mainWindow()->filesystemPanel()->viewMode(sh::ui::MainWindow::mainWindow()->filesystemPanel()->currentViewIndex()) == sh::ui::fileview::FileViewMode::ListView;
    setChecked(!_setAsIcons);
}

void sh::actions::viewmenu::ActionFileviewView::action(sh::actions::ActionExecutionInfo*)
{
    sh::tools::MainThread::dispatcher()->invokeSync([&](){
        sh::ui::MainWindow::mainWindow()->filesystemPanel()->setViewMode(sh::ui::MainWindow::mainWindow()->filesystemPanel()->currentViewIndex(),
                                                                 _setAsIcons ? sh::ui::fileview::FileViewMode::IconView : sh::ui::fileview::FileViewMode::ListView);
    });
}
