/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "actions/separatoractionitem.h"
#include "tools/mainthread.h"
#include "ui/mainwindow.h"
#include "ui/filesystempanel.h"
#include "actionnumberfileviews.h"


sh::actions::viewmenu::ActionNumberFileviews::ActionNumberFileviews()
    : sh::actions::SubmenuActionItem(tr("File &lists"), QList<std::shared_ptr<sh::actions::AbstractActionItem>>())
{
}

void sh::actions::viewmenu::ActionNumberFileviews::initialize()
{
    sh::tools::MainThread::dispatcher()->invokeSync([&](){
        connect(sh::ui::MainWindow::mainWindow()->filesystemPanel(), &sh::ui::FilesystemPanel::viewCountChanged, this, [=](){
            configure();
        });
        configure();
    });
}

void sh::actions::viewmenu::ActionNumberFileviews::configure()
{
    QList<std::shared_ptr<sh::actions::AbstractActionItem>> subitems;
    subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new ActionNumberFileviews_Add()));
    if (sh::ui::MainWindow::mainWindow()->filesystemPanel()->viewsCount() > 1)
    {
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::SeparatorActionItem()));
        for(int i=0; i<sh::ui::MainWindow::mainWindow()->filesystemPanel()->viewsCount(); i++)
            subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new ActionNumberFileviews_Remove(i)));
    }
    setSubitems(subitems);
}

sh::actions::viewmenu::ActionNumberFileviews::ActionNumberFileviews_Add::ActionNumberFileviews_Add()
    : sh::actions::ActionActionItem(QObject::tr("Add new file list"))
{
}

void sh::actions::viewmenu::ActionNumberFileviews::ActionNumberFileviews_Add::action(sh::actions::ActionExecutionInfo*)
{
    sh::tools::MainThread::dispatcher()->invokeSync([&](){
        sh::ui::MainWindow::mainWindow()->filesystemPanel()->addView(true);
    });
}

sh::actions::viewmenu::ActionNumberFileviews::ActionNumberFileviews_Remove::ActionNumberFileviews_Remove(int i)
    : sh::actions::ActionActionItem(QObject::tr("Remove list #%1").arg(i+1))
{
    _i = i;
}

void sh::actions::viewmenu::ActionNumberFileviews::ActionNumberFileviews_Remove::action(sh::actions::ActionExecutionInfo*)
{
    sh::tools::MainThread::dispatcher()->invokeSync([&](){
        sh::ui::MainWindow::mainWindow()->filesystemPanel()->removeView(_i);
    });
}

