/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "actions/actionexecutioninfo.h"
#include "actions/separatoractionitem.h"
#include "actions/actionexecutionuserfeedback.h"
#include "actions/viewmenu/actionfileviewchangeicondimension.h"
#include "actions/viewmenu/actionfileviewfilesizeformatting.h"
#include "actions/viewmenu/actionfileviewshowhiddenfiles.h"
#include "actions/viewmenu/actionfileviewview.h"
#include "actions/viewmenu/actionnumberfileviews.h"
#include "actions/viewmenu/actionshowfoldertree.h"
#include "actions/viewmenu/actiontreestickyness.h"
#include "actions/viewmenu/actionshowdetailpanel.h"
#include "actions/viewmenu/actiongotodirectory.h"
#include "base/eurl.h"
#include "base/operation.h"
#include "base/filesystemhandler.h"
#include "base/filesystemnode.h"
#include "tools/singletoninitializer.h"
#include "tools/mainthread.h"
#include "tools/threadpool.h"
#include "ui/mainwindow.h"
#include "ui/filesystempanel.h"
#include "actionview.h"


namespace sh{namespace actions{namespace viewmenu{INITIALIZE_SINGLETON_IN_MAINTHREAD(20200, ActionView)}}}

std::shared_ptr<sh::actions::viewmenu::ActionView> sh::actions::viewmenu::ActionView::instance()
{
    if (!_instance)
        _instance = std::shared_ptr<sh::actions::viewmenu::ActionView>(new sh::actions::viewmenu::ActionView());
    return _instance;
}

std::shared_ptr<sh::actions::viewmenu::ActionView> sh::actions::viewmenu::ActionView::_instance = 0;

sh::actions::viewmenu::ActionView::ActionView()
    : sh::actions::SubmenuActionItem( tr("&View"), QList<std::shared_ptr<sh::actions::AbstractActionItem>>(), true, "view")
{
}

void sh::actions::viewmenu::ActionView::initialize()
{
    QList<std::shared_ptr<sh::actions::AbstractActionItem>> subitems;
    sh::tools::MainThread::dispatcher()->invokeSync([&](){
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::viewmenu::ActionNumberFileviews()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::viewmenu::ActionFileviewView()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::viewmenu::ActionFileviewShowHiddenFiles()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::viewmenu::ActionFileviewFilesizeFormatting()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::viewmenu::ActionFileViewIncreaseIconDimension()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::viewmenu::ActionFileViewDecreaseIconDimension()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::SeparatorActionItem()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::viewmenu::ActionShowFolderTree()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::viewmenu::ActionTreeStickyness()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::viewmenu::ActionShowDetailPanel()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::SeparatorActionItem()));
        subitems.append(std::shared_ptr<sh::actions::AbstractActionItem>(new sh::actions::viewmenu::ActionGotoDirectory()));
    });
    setSubitems(subitems);
}

void sh::actions::viewmenu::ActionView::_initialize()
{
}

void sh::actions::viewmenu::ActionView::_shutdown()
{
}
