/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QMutex>
#include "exceptions/exception.h"
#include "filesystemhandlers/localfilesystemhandler.h"
#include "filesystemhandlers/sharcfilesystemhandler.h"
#include "tools/singletoninitializer.h"
#include "filesystemhandler.h"
#include "filesystemhandlerregister.h"


namespace sh{namespace base{INITIALIZE_SINGLETON_IN_MAINTHREAD(30060, FilesystemHandlerRegister)}}

sh::base::FilesystemHandlerRegister::FilesystemHandlerRegister(QObject *parent) :
    QObject(parent)
{
}

sh::base::FilesystemHandlerRegister* sh::base::FilesystemHandlerRegister::instance()
{
    static sh::base::FilesystemHandlerRegister* _instance = new sh::base::FilesystemHandlerRegister();
    return _instance;
}

void sh::base::FilesystemHandlerRegister::_initialize()
{
}

void sh::base::FilesystemHandlerRegister::_shutdown()
{
}

void sh::base::FilesystemHandlerRegister::addHandler(QString scheme, std::shared_ptr<sh::base::FilesystemHandler> handler)
{
    QMutexLocker _l(&mutex);
    if (scheme.length() < 2)
        throw sh::exceptions::ArgumentException(sh::exceptions::ExceptionData().details("Scheme names must have a length of at least 2."));
    handlers.insert(scheme, handler);
}

std::shared_ptr<sh::base::FilesystemHandler> sh::base::FilesystemHandlerRegister::findHandler(QString scheme)
{
    QMutexLocker _l(&mutex);
    return handlers.value(scheme, 0);
}
