/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef FILESYSTEMMODELDIRECTORYTREEPROXYVISIBILITYENFORCEMENTS_H
#define FILESYSTEMMODELDIRECTORYTREEPROXYVISIBILITYENFORCEMENTS_H

#include "filesystemnode.h"
#include <memory>
#include <QMultiMap>


namespace sh {
    namespace base{

        /** \brief
         * Maintains a list of currently visible directories (i.e. the current one in each view) and controls
         * enforced visibility of them in the directory tree.
         */
        class FilesystemModelDirectoryTreeProxyVisibilityEnforcements : public QObject
        {
        public:
            /** \brief
             * Returns the instance of this singleton.
             */
            static std::shared_ptr<FilesystemModelDirectoryTreeProxyVisibilityEnforcements> instance();
            /** \brief
             * Only called by the Shallot infrastructure for initialization.
             */
            void _initialize();
            /** \brief
             * Only called by the Shallot infrastructure for shutdown.
             */
            void _shutdown();
            /** \brief Constructed only indirectly.
             */
            FilesystemModelDirectoryTreeProxyVisibilityEnforcements();
            /** \brief Called when a view enters the given directory node.
             */
            void nodeEnteredView(std::shared_ptr<FilesystemNode> node);
            /** \brief Called when a view leaves the given directory node.
             */
            void nodeLeftView(std::shared_ptr<FilesystemNode> node);
            /** \brief Called when a directory node is collapsed in the directory tree.
             */
            void nodeCollapsedInTree(std::shared_ptr<FilesystemNode> node);
        private:
            QMultiMap<std::shared_ptr<FilesystemNode>, QObject*> _enterednodesrepresentatives;
            bool _addenforcement(std::shared_ptr<FilesystemNode> node, std::shared_ptr<FilesystemNode> hnode);
            QList<std::tuple<std::weak_ptr<FilesystemNode>, QList<std::shared_ptr<FilesystemNode>>>> hiddenForcedVisibles;
        };

    }
}

#endif // FILESYSTEMMODELDIRECTORYTREEPROXYVISIBILITYENFORCEMENTS_H
