/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "tools/threadpool.h"
#include "configurationvalue.h"
#include "configurationvaluetype.h"


sh::configuration::ConfigurationValueType* sh::configuration::ConfigurationValue::ValueTypeString()
{
    QMutexLocker l1(_gettypeinitmutex());
    static sh::configuration::ConfigurationValueType* _valueTypeString = new sh::configuration::ConfigurationValueTypeString();
    return _valueTypeString;
}

sh::configuration::ConfigurationValueType* sh::configuration::ConfigurationValue::ValueTypeInteger()
{
    QMutexLocker l1(_gettypeinitmutex());
    static sh::configuration::ConfigurationValueType* _valueTypeInteger = new sh::configuration::ConfigurationValueTypeInteger();
    return _valueTypeInteger;
}

sh::configuration::ConfigurationValueType* sh::configuration::ConfigurationValue::ValueTypeBoolean()
{
    QMutexLocker l1(_gettypeinitmutex());
    static sh::configuration::ConfigurationValueType* _valueTypeBoolean = new sh::configuration::ConfigurationValueTypeBoolean();
    return _valueTypeBoolean;
}

sh::configuration::ConfigurationValueType* sh::configuration::ConfigurationValue::ValueTypeLocalFilePath()
{
    QMutexLocker l1(_gettypeinitmutex());
    static sh::configuration::ConfigurationValueType* _valueTypeLocalFilePath = new sh::configuration::ConfigurationValueTypeLocalFilePath();
    return _valueTypeLocalFilePath;
}

sh::configuration::ConfigurationValue::ConfigurationValue(QString name, QVariant deflt, QString desc, QString longdesc, sh::configuration::ConfigurationCategory cat, sh::configuration::ConfigurationValueType* valuetype, QString changehint)
    :name(name), desc(desc), longdesc(longdesc), changehint(changehint), cat(cat), valuetype(valuetype), deflt(deflt), _callbacksmutex(QMutex::Recursive)
{
}

int sh::configuration::ConfigurationValue::getConfigValueInt()
{
    return _getConfigValueVariant().toInt();
}

bool sh::configuration::ConfigurationValue::getConfigValueBool()
{
    return _getConfigValueVariant().toBool();
}

QString sh::configuration::ConfigurationValue::getConfigValueString()
{
    return _getConfigValueVariant().toString();
}

void sh::configuration::ConfigurationValue::consumeValue(QObject* o, std::function<void()> f)
{
    QMutexLocker _l1(&_callbacksmutex);
    static int i = 0;
    int _i = i++;
    _callbacks.insert(_i, f);
    QObject::connect(o, &QObject::destroyed, [=](){
        _callbacks.remove(_i);
    });
    f();
}

QList<std::function<void()>> sh::configuration::ConfigurationValue::callbacks()
{
    QMutexLocker _l1(&_callbacksmutex);
    return _callbacks.values();
}

QString sh::configuration::ConfigurationValue::getDescription()
{
    return (desc.isEmpty() ? name : desc);
}

QString sh::configuration::ConfigurationValue::getLongDescription()
{
    return longdesc;
}

sh::configuration::ConfigurationCategory sh::configuration::ConfigurationValue::getCategory()
{
    return cat;
}

QString sh::configuration::ConfigurationValue::changeHint()
{
    return changehint;
}


sh::configuration::ConfigurationValueType* sh::configuration::ConfigurationValue::getValueType()
{
    return valuetype;
}

QMutex* sh::configuration::ConfigurationValue::_typeinitmutex = 0;

QMutex* sh::configuration::ConfigurationValue::_gettypeinitmutex()
{ // i admit, my lazy mutex initialization looks weird :) but it works since the initialization is guaranteed to happen in main thread without interruption
    if (!_typeinitmutex)
        _typeinitmutex = new QMutex();
    return _typeinitmutex;
}
