/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QBoxLayout>
#include <QFileDialog>
#include <QInputDialog>
#include <QLabel>
#include "configurationmanager.h"
#include "configurationvalue.h"
#include "configurationvaluetype.h"


QString sh::configuration::ConfigurationValueType::valueDescription(QVariant v)
{
    return v.toString();
}

QVariant sh::configuration::ConfigurationValueType::parse(QString input)
{
    return QVariant(input);
}

void sh::configuration::ConfigurationValueType::changegui(sh::configuration::ConfigurationValue* cval, QWidget *parent)
{
    QString text = valueDescription(cval->_getConfigValueVariant());
    QString label = cval->getDescription();
    if (!cval->changeHint().isEmpty())
        label += "\n\n"+cval->changeHint();
    else if (!cval->getLongDescription().isEmpty())
        label += "\n\n"+cval->getLongDescription();
    QString n = QInputDialog::getText(parent, QObject::tr("Change"), label, QLineEdit::Normal, text);
    if (!n.isNull()){
        QVariant val = parse(n);
        if (!val.isNull())
            cval->setConfigValue(val);
    }
}

bool sh::configuration::ConfigurationValueType::showChangeHintInOverview()
{
    return false;
}

QWidget* sh::configuration::ConfigurationValueType::gui(sh::configuration::ConfigurationValue* cval, QWidget* parent)
{
    QWidget* w = new QWidget(parent);
    QHBoxLayout* l = new QHBoxLayout();
    QWidget* iw = new QWidget(parent);
    QVBoxLayout* il = new QVBoxLayout();
    QLabel* lbl = new QLabel(iw);
    lbl->setWordWrap(true);
    cval->consumeValue(parent, [=](){
        lbl->setText( cval->getValueType()->valueDescription(cval->_getConfigValueVariant()) );
    });
    lbl->setStyleSheet(QString("color: %2;")
                        .arg(sh::configuration::ConfigurationManager::instance()->style()->foregroundColorLighter1().name())
                             );
    sh::ui::LinkButton* btnch = new sh::ui::LinkButton(w);
    btnch->setText(QObject::tr("change"));
    QObject::connect(btnch, &QToolButton::clicked, [=](){
        changegui(cval, parent->window());
    });
    if (!cval->getLongDescription().isEmpty()){
        QLabel* lbl2 = new QLabel(iw);
        lbl2->setText(cval->getLongDescription());
        lbl2->setStyleSheet(QString("font-size:%1pt;")
                            .arg(sh::configuration::ConfigurationManager::instance()->style()->fontSizeInPt(90)));
        lbl2->setWordWrap(true);
        il->addWidget(lbl2);
    }
    if ( showChangeHintInOverview() && !cval->changeHint().isEmpty()){
        QLabel* lbl2 = new QLabel(iw);
        lbl2->setText(cval->changeHint());
        lbl2->setStyleSheet(QString("font-size:%1pt;")
                            .arg(sh::configuration::ConfigurationManager::instance()->style()->fontSizeInPt(90)));
        lbl2->setWordWrap(true);
        il->addWidget(lbl2);
    }
    il->addWidget(lbl);
    l->addWidget(iw);
    l->addWidget(btnch);
    l->setStretch(0,1);
    l->setStretch(1,0);
    iw->setLayout(il);
    w->setLayout(l);
    l->setMargin(2);
    il->setMargin(0);
    il->setSpacing(5);
    return w;
}


void sh::configuration::ConfigurationValueTypeLocalFilePath::changegui(sh::configuration::ConfigurationValue *cval, QWidget *parent)
{
    QString text = cval->getConfigValueString();
    QString label = cval->getDescription();
    if (!cval->changeHint().isEmpty())
        label += "\n\n"+cval->changeHint();
    else if (!cval->getLongDescription().isEmpty())
        label += "\n\n"+cval->getLongDescription();
    QString n = QFileDialog::getOpenFileName(parent, QObject::tr("Change"), text, QString());
    if (!n.isNull())
        cval->setConfigValue(n);
}

bool sh::configuration::ConfigurationValueTypeLocalFilePath::showChangeHintInOverview()
{
    return true;
}


void sh::configuration::ConfigurationValueTypeInteger::changegui(sh::configuration::ConfigurationValue *cval, QWidget *parent)
{
    int val = cval->getConfigValueInt();
    QString label = cval->getDescription();
    if (!cval->changeHint().isEmpty())
        label += "\n\n"+cval->changeHint();
    else if (!cval->getLongDescription().isEmpty())
        label += "\n\n"+cval->getLongDescription();
    bool _ok;
    val = QInputDialog::getInt(parent, QObject::tr("Change"), label, val, INT_MIN, INT_MAX, 1, &_ok);
    if (_ok)
        cval->setConfigValue(val);
}

void sh::configuration::ConfigurationValueTypeBoolean::changegui(sh::configuration::ConfigurationValue *cval, QWidget*)
{
    cval->setConfigValue( !cval->getConfigValueBool() );
}

bool sh::configuration::ConfigurationValueTypeBoolean::showChangeHintInOverview()
{
    return true;
}
