/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CONFIGURATIONVALUETYPE_H
#define CONFIGURATIONVALUETYPE_H

#include <QVariant>
#include <QWidget>
#include "ui/linkbutton.h"


namespace sh{
    namespace configuration {

        class ConfigurationValue;

        /** \brief
         * Abstract base class for a configuration value type.
         *
         * Each subclass implements support for a certain type of configuration values
         * (e.g. string, integer, ...).
         */
        class ConfigurationValueType
        {
        public:
            virtual QString valueDescription(QVariant v);
            virtual QVariant parse(QString input);
            virtual void changegui(ConfigurationValue *cval, QWidget *parent);
            virtual QWidget* gui(ConfigurationValue *cval, QWidget *parent);
        protected:
            virtual bool showChangeHintInOverview();
        };

        /** \brief Configuration value type 'file path'.
         */
        class ConfigurationValueTypeLocalFilePath : public ConfigurationValueType
        {
        public:
            virtual void changegui(ConfigurationValue *cval, QWidget *parent);
        protected:
            virtual bool showChangeHintInOverview();
        };

        /** \brief Configuration value type 'string'.
         */
        class ConfigurationValueTypeString : public ConfigurationValueType
        {
        };

        /** \brief Configuration value type 'integer'.
         */
        class ConfigurationValueTypeInteger : public ConfigurationValueType
        {
        public:
            virtual void changegui(ConfigurationValue *cval, QWidget *parent);
        };

        /** \brief Configuration value type 'boolean'.
         */
        class ConfigurationValueTypeBoolean : public ConfigurationValueType
        {
        public:
            virtual void changegui(ConfigurationValue *cval, QWidget *parent);
        protected:
            virtual bool showChangeHintInOverview();
        };

    }
}

#endif // CONFIGURATIONVALUETYPE_H
