/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef UISTYLE_H
#define UISTYLE_H

#include <memory>
#include <QColor>
#include <QObject>


namespace sh{
    namespace ui {class MainWindow;}

    namespace configuration {

        class ConfigurationValue;

        /** \brief
         * Methods for applying the Shallot visible style.
         *
         * See sh::configuration::ConfigurationManager about how to get an instance.
         */
        class UIStyle : public QObject
        {
            Q_OBJECT
        public:
            /** \brief Constructed only by the infrastructure and made available otherwise.
             */
            explicit UIStyle(sh::ui::MainWindow *parent = 0);

            QColor windowColor();
            QColor backgroundColor();
            QColor inactiveBackgroundColor();
            QColor windowColorHighlighted();
            QColor windowColorHalfHighlighted();
            QColor windowColorDarkened();
            QColor windowColorLikeTitlebar();

            QColor linkColor();

            QColor titlebarColor();

            QColor brandingColor();

            QColor foregroundColor();
            QColor foregroundColorLighter1();
            QColor foregroundColorLighter2();

            int fontSizeInPt(int r=100);

            QColor mix(float n1, QColor c1, QColor c2);

            class Sheet;
            Sheet createSheet();

            class Sheet
            {
                friend class UIStyle;
            public:
                QString sheet();
                Sheet customcss(QString css);
                Sheet fontsize_byFactor(int f);
                Sheet fontsize_header();
                Sheet fontsize_smaller();
                Sheet textcolor(QColor color);
                Sheet textcolor_lighter1();
                Sheet textcolor_lighter2();
                Sheet background(QColor color, bool restrictQWidget = true);
                Sheet background_highlighted(bool restrictQWidget = true);
                Sheet background_halfhighlighted(bool restrictQWidget = true);
                Sheet background_darkened(bool restrictQWidget = true);
                Sheet background_liketitlebar(bool restrictQWidget = true);
                Sheet applyTo(QWidget* widget);
                Sheet bold();
            private:
                Sheet(UIStyle* style, QString sheet);
                QString _sheet;
                UIStyle* _style;
            };

        private:
            sh::ui::MainWindow* _mainWindow;
            QColor _brandingcolor;
            std::shared_ptr<sh::configuration::ConfigurationValue> cfgvalBrandingColor;

        };

    }
}

#endif // UISTYLE_H
