/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/filesystemnode.h"
#include "base/filesystemhandler.h"
#include "base/filesystemoperation.h"
#include "base/filesystemmodelfileviewproxy.h"
#include "tools/singletoninitializer.h"
#include "detailcolumncustomattributes.h"
#include "detailcolumnresolvedsymlink.h"


namespace sh{namespace detailcolumns{INITIALIZE_SINGLETON_IN_MAINTHREAD(100000, DetailColumnCustomAttributes)}}

sh::detailcolumns::DetailColumnCustomAttributes::DetailColumnCustomAttributes()
    : sh::base::DetailColumn("CUSTOMATTR", QObject::tr("Custom attributes"))
{
}

std::shared_ptr<sh::detailcolumns::DetailColumnCustomAttributes> sh::detailcolumns::DetailColumnCustomAttributes::instance()
{
    static std::shared_ptr<sh::detailcolumns::DetailColumnCustomAttributes> _inst = std::make_shared<sh::detailcolumns::DetailColumnCustomAttributes>();
    return _inst;
}

QVariant sh::detailcolumns::DetailColumnCustomAttributes::determineValue(std::shared_ptr<sh::base::FilesystemNode> node, sh::base::Operation* op)
{
    QStringList result;
    QMap<QString, QString> customvalues = node->handler()->getCustomAttributes(op, node->eurl());
    foreach(QString attname, customvalues.keys())
        result << attname << customvalues[attname];
    return result;
}

QMap<QString, QString> sh::detailcolumns::DetailColumnCustomAttributes::getMapByValue(QVariant val)
{
    QMap<QString, QString> result;
    QString key;
    foreach(QString s, val.toStringList())
    {
        if (key.isNull())
            key = s;
        else
        {
            result[key] = s;
            key = QString();
        }
    }
    return result;
}

bool sh::detailcolumns::DetailColumnCustomAttributes::isVisible()
{
    return false;
}

void sh::detailcolumns::DetailColumnCustomAttributes::applyValueByEurl(std::shared_ptr<const sh::base::Eurl> eurl, sh::base::Operation* op, QVariant value)
{
    QStringList lst = value.toStringList();
    while(!lst.isEmpty())
    {
        QString attname = lst.takeFirst();
        QString attval = lst.takeFirst();
        sh::base::FilesystemHandlerRegister::instance()->findHandler(eurl->scheme())->setCustomAttribute(op, eurl, attname, attval);
    }
}

void sh::detailcolumns::DetailColumnCustomAttributes::_initialize()
{
    sh::base::FilesystemOperation::addTransferrableDetailColumn(500001, instance());
    sh::base::DetailColumn::registerKnownDetailColumn("customattributes", instance());
}

void sh::detailcolumns::DetailColumnCustomAttributes::_shutdown()
{
}
