/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/filesystemnode.h"
#include "base/filesystemhandler.h"
#include "tools/singletoninitializer.h"
#include "detailcolumndirectsymlinktarget.h"


namespace sh{namespace detailcolumns{INITIALIZE_SINGLETON_IN_MAINTHREAD(100000, DetailColumnDirectSymlinkTarget)}}

sh::detailcolumns::DetailColumnDirectSymlinkTarget::DetailColumnDirectSymlinkTarget()
    : sh::base::DetailColumn("DIRECT_SYMLINK_TARGET", QObject::tr("Direct symlink target"))
{
}

std::shared_ptr<sh::detailcolumns::DetailColumnDirectSymlinkTarget> sh::detailcolumns::DetailColumnDirectSymlinkTarget::instance()
{
    static std::shared_ptr<sh::detailcolumns::DetailColumnDirectSymlinkTarget> _inst = std::make_shared<sh::detailcolumns::DetailColumnDirectSymlinkTarget>();
    return _inst;
}

QVariant sh::detailcolumns::DetailColumnDirectSymlinkTarget::determineValue(std::shared_ptr<sh::base::FilesystemNode> node, sh::base::Operation* op)
{
    if (node->nodetype()==sh::base::FilesystemNodeType::Link)
        return node->handler()->getLinkTarget(op, node->eurl());
    else
        return QVariant();
}

bool sh::detailcolumns::DetailColumnDirectSymlinkTarget::isVisible()
{
    return false;
}

void sh::detailcolumns::DetailColumnDirectSymlinkTarget::_initialize()
{
    sh::base::DetailColumn::registerKnownDetailColumn("directsymlinktarget", instance());
}

void sh::detailcolumns::DetailColumnDirectSymlinkTarget::_shutdown()
{
}
