/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/filesystemnode.h"
#include "base/filesystemhandler.h"
#include "base/filesystemoperation.h"
#include "base/filesystemmodelfileviewproxy.h"
#include "tools/singletoninitializer.h"
#include "detailcolumnextendedattributes.h"
#include "detailcolumnresolvedsymlink.h"


namespace sh{namespace detailcolumns{INITIALIZE_SINGLETON_IN_MAINTHREAD(100000, DetailColumnExtendedAttributes)}}

sh::detailcolumns::DetailColumnExtendedAttributes::DetailColumnExtendedAttributes()
    : sh::base::DetailColumn("XATTR", QObject::tr("Extended attributes"))
{
}

std::shared_ptr<sh::detailcolumns::DetailColumnExtendedAttributes> sh::detailcolumns::DetailColumnExtendedAttributes::instance()
{
    static std::shared_ptr<sh::detailcolumns::DetailColumnExtendedAttributes> _inst = std::make_shared<sh::detailcolumns::DetailColumnExtendedAttributes>();
    return _inst;
}

QVariant sh::detailcolumns::DetailColumnExtendedAttributes::determineValue(std::shared_ptr<sh::base::FilesystemNode> node, sh::base::Operation* op)
{
    QStringList result;
    foreach(QString attname, node->handler()->listExtendedAttributes(op, node->eurl()))
    {
        QByteArray ba = node->handler()->getExtendedAttribute(op, node->eurl(), attname);
        result << attname << QString::fromUtf8(ba.toBase64());
    }
    return result;
}

QMap<QString, QByteArray> sh::detailcolumns::DetailColumnExtendedAttributes::getMapByValue(QVariant val)
{
    QMap<QString, QByteArray> result;
    QString key;
    foreach(QString s, val.toStringList())
    {
        if (key.isNull())
            key = s;
        else
        {
            result[key] = QByteArray::fromBase64(s.toUtf8());
            key = QString();
        }
    }
    return result;
}

bool sh::detailcolumns::DetailColumnExtendedAttributes::isVisible()
{
    return false;
}

void sh::detailcolumns::DetailColumnExtendedAttributes::applyValueByEurl(std::shared_ptr<const sh::base::Eurl> eurl, sh::base::Operation* op, QVariant value)
{
    QStringList lst = value.toStringList();
    while(!lst.isEmpty())
    {
        QString attname = lst.takeFirst();
        QString attvalbase64 = lst.takeFirst();
        QByteArray attval = QByteArray::fromBase64(attvalbase64.toUtf8());
        sh::base::FilesystemHandlerRegister::instance()->findHandler(eurl->scheme())->setExtendedAttribute(op, eurl, attname, attval);
    }
}

void sh::detailcolumns::DetailColumnExtendedAttributes::_initialize()
{
    sh::base::FilesystemOperation::addTransferrableDetailColumn(500000, instance());
    sh::base::DetailColumn::registerKnownDetailColumn("extendedattributes", instance());

}

void sh::detailcolumns::DetailColumnExtendedAttributes::_shutdown()
{
}
