/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/filesystemnode.h"
#include "base/filesystemhandler.h"
#include "base/filesystemmodelfileviewproxy.h"
#include "tools/singletoninitializer.h"
#include "detailcolumnfilesize.h"
#include "detailcolumnresolvedsymlink.h"


namespace sh{namespace detailcolumns{INITIALIZE_SINGLETON_IN_MAINTHREAD(100000, DetailColumnFilesize)}}

sh::detailcolumns::DetailColumnFilesize::DetailColumnFilesize()
    : sh::base::DetailColumn("FILESIZE", QObject::tr("Size"), DISPLAYINDEX_CORE+100, true, 100, true)
{
}

std::shared_ptr<sh::detailcolumns::DetailColumnFilesize> sh::detailcolumns::DetailColumnFilesize::instance()
{
    static std::shared_ptr<sh::detailcolumns::DetailColumnFilesize> _inst = std::make_shared<sh::detailcolumns::DetailColumnFilesize>();
    return _inst;
}

QVariant sh::detailcolumns::DetailColumnFilesize::determineValue(std::shared_ptr<sh::base::FilesystemNode> node, sh::base::Operation* op)
{
    std::shared_ptr<sh::base::DetailColumn> ressymlink = sh::detailcolumns::DetailColumnResolvedSymlink::instance();
    QList<std::shared_ptr<sh::base::FilesystemNode>> resnodes = ressymlink->requestValue(node, op).value<QList<std::shared_ptr<sh::base::FilesystemNode>>>();
    std::shared_ptr<sh::base::FilesystemNode> resnode = (resnodes.isEmpty()) ? 0 : resnodes.first();
    if (resnode && resnode->nodetype()==sh::base::FilesystemNodeType::File)
        return resnode->handler()->getSize(op, resnode->eurl());
    else
        return QVariant();
}

QString sh::detailcolumns::DetailColumnFilesize::displayValue(std::shared_ptr<sh::base::FilesystemNode> node, const sh::base::FilesystemModelFileviewProxy *viewmodel)
{
    static QLocale locale;
    static const QString ups = " kMGTPEZY"; // unit prefixes
    if (node->nodetype()!=sh::base::FilesystemNodeType::Directory)
    {
        bool _unneeded;
        QVariant val = value(node);
        if (val == sh::base::DetailColumn::VALUE_UNAVAILABLE() || val.isNull())
            return "";
        else {
            qint64 s = val.toLongLong(&_unneeded);
            if (viewmodel->getSizeFormattingMode() == sh::base::SizeFormatting::SizeFormattingModePrefixes){
                int pi = 0;
                while ( (s/ pow(1024,pi))> 1024 && pi<(ups.length()-1) )
                {
                    pi++;
                }
                QString up(ups.at(pi));
                up = up.trimmed();
                double ds = ((double)(s * 10 / (qint64)pow(1024,pi))) /10.0;
                if (pi==0)
                    return QString("%1 %2B").arg(s).arg(up);
                else
                    return QString("%1 %2B").arg(ds, 0, 'f', 1).arg(up);
            }
            return locale.toString(s);
        }
    }
    else
        return "";
}

void sh::detailcolumns::DetailColumnFilesize::_initialize()
{
    sh::base::DetailColumn::registerKnownDetailColumn("filesize", instance());
}

void sh::detailcolumns::DetailColumnFilesize::_shutdown()
{
}
