/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/filesystemnode.h"
#include "base/filesystemhandler.h"
#include "tools/singletoninitializer.h"
#include "detailcolumnmimetype.h"
#include "detailcolumnresolvedsymlink.h"


namespace sh{namespace detailcolumns{INITIALIZE_SINGLETON_IN_MAINTHREAD(100000, DetailColumnMimetype)}}

sh::detailcolumns::DetailColumnMimetype::DetailColumnMimetype() :
    sh::base::DetailColumn("MIMETYPE", QObject::tr("Type"), DISPLAYINDEX_CORE+300, true, 180)
{
}

std::shared_ptr<sh::detailcolumns::DetailColumnMimetype> sh::detailcolumns::DetailColumnMimetype::instance()
{
    static std::shared_ptr<sh::detailcolumns::DetailColumnMimetype> _inst = std::make_shared<sh::detailcolumns::DetailColumnMimetype>();
    return _inst;
}

QVariant sh::detailcolumns::DetailColumnMimetype::determineValue(std::shared_ptr<sh::base::FilesystemNode> node, sh::base::Operation* op)
{
    std::shared_ptr<sh::base::DetailColumn> ressymlink = sh::detailcolumns::DetailColumnResolvedSymlink::instance();
    QList<std::shared_ptr<sh::base::FilesystemNode>> resnodes = ressymlink->requestValue(node, op).value<QList<std::shared_ptr<sh::base::FilesystemNode>>>();
    std::shared_ptr<sh::base::FilesystemNode> resnode = (resnodes.isEmpty()) ? 0 : resnodes.first();
    if (resnode && resnode->nodetype() == sh::base::FilesystemNodeType::File)
        return resnode->handler()->getMimetype(op, resnode->eurl());
    else
        return "";
}

QString sh::detailcolumns::DetailColumnMimetype::displayValue(std::shared_ptr<sh::base::FilesystemNode> node, const sh::base::FilesystemModelFileviewProxy*)
{
    if (node->nodetype() != sh::base::FilesystemNodeType::Directory)
        return value(node).toString();
    else
        return "";
}

void sh::detailcolumns::DetailColumnMimetype::_initialize()
{
    sh::base::DetailColumn::registerKnownDetailColumn("mimetype", instance());
}

void sh::detailcolumns::DetailColumnMimetype::_shutdown()
{
}
