/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/filesystemnode.h"
#include "base/filesystemhandler.h"
#include "base/filesystemhandlerregister.h"
#include "base/filesystemoperation.h"
#include "tools/singletoninitializer.h"
#include "detailcolumnmtime.h"
#include "detailcolumnresolvedsymlink.h"


namespace sh{namespace detailcolumns{INITIALIZE_SINGLETON_IN_MAINTHREAD(100000, DetailColumnMtime)}}

sh::detailcolumns::DetailColumnMtime::DetailColumnMtime() :
    sh::base::DetailColumn("MTIME", QObject::tr("Last modified"), DISPLAYINDEX_CORE+200, false, 180)
{
}

std::shared_ptr<sh::detailcolumns::DetailColumnMtime> sh::detailcolumns::DetailColumnMtime::instance()
{
    static std::shared_ptr<sh::detailcolumns::DetailColumnMtime> _inst = std::make_shared<sh::detailcolumns::DetailColumnMtime>();
    return _inst;
}

QVariant sh::detailcolumns::DetailColumnMtime::determineValue(std::shared_ptr<sh::base::FilesystemNode> node, sh::base::Operation* op)
{
    return node->handler()->getMtime(op, node->eurl());
}

QString sh::detailcolumns::DetailColumnMtime::displayValue(std::shared_ptr<sh::base::FilesystemNode> node, const sh::base::FilesystemModelFileviewProxy*)
{
    return value(node).toDateTime().toString(Qt::DefaultLocaleShortDate);
}

void sh::detailcolumns::DetailColumnMtime::applyValueByEurl(std::shared_ptr<const sh::base::Eurl> eurl, sh::base::Operation* op, QVariant value)
{
    sh::base::FilesystemHandlerRegister::instance()->findHandler(eurl->scheme())->setMtime(op, eurl, value.toDateTime());
}

void sh::detailcolumns::DetailColumnMtime::_initialize()
{
    sh::base::FilesystemOperation::addTransferrableDetailColumn(999999, instance());
    sh::base::DetailColumn::registerKnownDetailColumn("mtime", instance());
}

void sh::detailcolumns::DetailColumnMtime::_shutdown()
{
}
