/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef DETAILCOLUMNMTIME_H
#define DETAILCOLUMNMTIME_H

#include "base/detailcolumn.h"


namespace sh{
    namespace detailcolumns {

        /** \brief
         * Detail column which determines the file's modification time.
         */
        class DetailColumnMtime : public sh::base::DetailColumn
        {
        public:
            /** \brief
             * Returns the instance of this singleton.
             */
            static std::shared_ptr<DetailColumnMtime> instance();
            /** \brief
             * Only called by the Shallot infrastructure for initialization.
             */
            void _initialize();
            /** \brief
             * Only called by the Shallot infrastructure for shutdown.
             */
            void _shutdown();
            /** \brief Constructed only by the infrastructure and made available otherwise.
             */
            DetailColumnMtime();

            virtual void applyValueByEurl(std::shared_ptr<const sh::base::Eurl> eurl, sh::base::Operation* op, QVariant value);

        protected:
            virtual QVariant determineValue(std::shared_ptr<sh::base::FilesystemNode> node, sh::base::Operation* op);
            virtual QString displayValue(std::shared_ptr<sh::base::FilesystemNode> node, const sh::base::FilesystemModelFileviewProxy *viewmodel);
        };

    }
}

#endif // DETAILCOLUMNMTIME_H
