/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/filesystemnode.h"
#include "base/filesystemhandler.h"
#include "base/filesystemoperation.h"
#include "base/operation.h"
#include "tools/singletoninitializer.h"
#include "detailcolumnresolvedsymlink.h"


namespace sh{namespace detailcolumns{INITIALIZE_SINGLETON_IN_MAINTHREAD(100000, DetailColumnResolvedSymlink)}}

sh::detailcolumns::DetailColumnResolvedSymlink::DetailColumnResolvedSymlink()
    : sh::base::DetailColumn("RESOLVED_SYMLINK_TARGET", QObject::tr("Resolved symlink target"))
{
}

std::shared_ptr<sh::detailcolumns::DetailColumnResolvedSymlink> sh::detailcolumns::DetailColumnResolvedSymlink::instance()
{
    static std::shared_ptr<sh::detailcolumns::DetailColumnResolvedSymlink> _inst = std::make_shared<sh::detailcolumns::DetailColumnResolvedSymlink>();
    return _inst;
}

QVariant sh::detailcolumns::DetailColumnResolvedSymlink::determineValue(std::shared_ptr<sh::base::FilesystemNode> node, sh::base::Operation* operation)
{
    if (node->nodetype() == sh::base::FilesystemNodeType::Link)
    {
        QList<std::shared_ptr<sh::base::FilesystemNode>> hnodes = operation->filesystem()->resolveNodeLink(node, true, true);
        node->setLinkDestinationNodes(hnodes);
        return QVariant::fromValue(hnodes);
    }
    else
    {
        node->setLinkDestinationNodes(QList<std::shared_ptr<sh::base::FilesystemNode>>());
        return QVariant::fromValue(QList<std::shared_ptr<sh::base::FilesystemNode>>({node}));
    }
}

QString sh::detailcolumns::DetailColumnResolvedSymlink::displayValue(std::shared_ptr<sh::base::FilesystemNode> node, const sh::base::FilesystemModelFileviewProxy*)
{
    QList<std::shared_ptr<sh::base::FilesystemNode>> v = value(node).value<QList<std::shared_ptr<sh::base::FilesystemNode>>>();
    if (v.isEmpty())
        return "";
    else
        return v.first()->eurl()->asString();
}

bool sh::detailcolumns::DetailColumnResolvedSymlink::isVisible()
{
    return false;
}

void sh::detailcolumns::DetailColumnResolvedSymlink::_initialize()
{
    sh::base::DetailColumn::registerKnownDetailColumn("resolvedsymlink", instance());
}

void sh::detailcolumns::DetailColumnResolvedSymlink::_shutdown()
{
}
