/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef GNOMEIODEVICESFILESYSTEMHANDLER_H
#define GNOMEIODEVICESFILESYSTEMHANDLER_H

#include "gnomeiofilesystemhandler.h"


#ifdef GIO

namespace sh{
    namespace filesystemhandlers {

        /** \brief
         * A filesystem handler for the GIO filesystem 'Devices' subtree.
         */
        class GnomeIODevicesFilesystemHandler : public sh::filesystemhandlers::GnomeIOFilesystemHandler
        {
            friend class ActionMount;
            Q_OBJECT
        public:
            /** \brief
             * Returns the instance of this singleton.
             */
            static std::shared_ptr<GnomeIODevicesFilesystemHandler> instance();
            /** \brief
             * Only called by the Shallot infrastructure for initialization.
             */
            void _initialize();
            /** \brief
             * Only called by the Shallot infrastructure for shutdown.
             */
            void _shutdown();
            explicit GnomeIODevicesFilesystemHandler(sh::base::FilesystemModel* model);
            ~GnomeIODevicesFilesystemHandler();

            virtual void itemlist(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl, sh::base::FilesystemNodeType type, sh::base::FilesystemNodeListEditor *list);
            virtual sh::base::FilesystemNodeType getType(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl);

            virtual QList<std::shared_ptr<sh::actions::AbstractActionItem> > getActions(const QList<std::shared_ptr<const sh::base::Eurl>> eurls);
            void refresh();

            void requestNewNode(std::shared_ptr<const sh::base::Eurl> eurl, std::shared_ptr<sh::base::FilesystemNode>* nnode);

        protected:
            sh::base::FilesystemNodeList* childlist;
            std::shared_ptr<sh::base::FilesystemNode> rootnode;
            std::shared_ptr<sh::base::FilesystemNode> _getMountedNode(QString gvolumeuuid, QString grooturl, std::shared_ptr<sh::base::FilesystemNode> parentnode);
            std::shared_ptr<sh::base::FilesystemNode> _getUnmountedNode(QString gvolumeuuid, std::shared_ptr<sh::base::FilesystemNode> parentnode);
            QMap<QString, QString> devicenames;
            QMutex masternodeslock;
            void* volumemonitor;

            struct GnomeIODevice
            {
                GnomeIODevice(QString name, QString displayname, QString url, bool isMounted);
                QString name;
                QString displayname;
                QString url;
                bool isMounted;
            };

        };

    }
}

#endif

#endif // GNOMEIODEVICESFILESYSTEMHANDLER_H
