/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef GNOMEIONETWORKFILESYSTEMHANDLER_H
#define GNOMEIONETWORKFILESYSTEMHANDLER_H

#include "gnomeiofilesystemhandler.h"


#ifdef GIO

namespace sh{
    namespace filesystemhandlers {

        /** \brief
         * A filesystem handler for the GIO filesystem 'Network' subtree.
         */
        class GnomeIONetworkFilesystemHandler : public sh::filesystemhandlers::GnomeIOFilesystemHandler
        {
            Q_OBJECT
        public:

            virtual QList<std::shared_ptr<sh::actions::AbstractActionItem> > getActions(const QList<std::shared_ptr<const sh::base::Eurl>> eurls);

            /** \brief
             * Returns the instance of this singleton.
             */
            static std::shared_ptr<GnomeIONetworkFilesystemHandler> instance();
            /** \brief
             * Only called by the Shallot infrastructure for initialization.
             */
            void _initialize();
            /** \brief
             * Only called by the Shallot infrastructure for shutdown.
             */
            void _shutdown();
            explicit GnomeIONetworkFilesystemHandler(sh::base::FilesystemModel *model);

            void requestNewNode(std::shared_ptr<const sh::base::Eurl> eurl, std::shared_ptr<sh::base::FilesystemNode>* nnode);

        protected:
            static std::shared_ptr<sh::base::FilesystemNode> rootnode;
        };

        /** \brief Action for mounting a GIO network location.
         */
        class ActionMountNetworkGnomeIOLocation : public sh::actions::ActionActionItem
        {
            Q_OBJECT
        public:
            /** \brief Constructed only internally.
             */
            explicit ActionMountNetworkGnomeIOLocation();
        protected:
            virtual void action(sh::actions::ActionExecutionInfo* info);
        };

    }
}

#endif

#endif // GNOMEIONETWORKFILESYSTEMHANDLER_H
