/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef WINDOWSNETWORKFILESYSTEMHANDLER_H
#define WINDOWSNETWORKFILESYSTEMHANDLER_H

#include "localfilesystemhandler.h"


namespace sh{
    namespace filesystemhandlers {

        class WindowsNetworkFilesystemHandler : public sh::filesystemhandlers::LocalFilesystemHandler
        {
            Q_OBJECT
#ifdef Q_OS_WIN
        public:
            /** \brief
             * Returns the instance of this singleton.
             */
            static std::shared_ptr<WindowsNetworkFilesystemHandler> instance();
            /** \brief
             * Only called by the Shallot infrastructure for initialization.
             */
            void _initialize();
            /** \brief
             * Only called by the Shallot infrastructure for shutdown.
             */
            void _shutdown();
            explicit WindowsNetworkFilesystemHandler(sh::base::FilesystemModel *model);

            virtual void itemlist(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl, sh::base::FilesystemNodeType type, sh::base::FilesystemNodeListEditor *list);
            virtual sh::base::FilesystemNodeType getType(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl);

            virtual std::shared_ptr<const sh::base::Eurl> localToEurl(const QString local);
            virtual QString eurlToLocal(std::shared_ptr<const sh::base::Eurl> eurl);

        protected:
            void createRootNode();

        private:
            std::shared_ptr<sh::base::FilesystemNode> rootnode;
#endif
        };

    }
}

#endif // WINDOWSNETWORKFILESYSTEMHANDLER_H
