/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QApplication>
#include <QTranslator>
#include <QTime>
#include <functional>
#include "base/filesystemhandlerregister.h"
#include "configuration/configurationmanager.h"
#include "filesystemhandlers/archivefilesystemhandler.h"
#include "scripting/scriptingengine.h"
#include "tools/benchmarking.h"
#include "tools/iconmanager.h"
#include "tools/filetypes/filetypemanager.h"
#include "tools/mainthread.h"
#include "tools/logger.h"
#include "tools/singletoninitializer.h"
#include "ui/mainwindow.h"


int main(int argc, char *argv[])
{
    LOG_DEBUG("Starting Shallot.")

    QApplication a(argc, argv);
    QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps, true);
    a.setQuitOnLastWindowClosed(false);

    // load correct translation
    QString locale = QLocale::system().name();
    QTranslator translator;
    translator.load(QString("shallot_") + locale, sh::configuration::ConfigurationManager::shallotDataDir() + "/i18n");
    a.installTranslator(&translator);

    sh::tools::MainThread::dispatcher();
    sh::base::DetailColumn::VALUE_UNAVAILABLE();
    sh::exceptions::ExceptionHandlerSettingsManager::instance()->handlerSettings();

    sh::tools::SingletonInitializer singletonInitializer;

    sh::ui::MainWindow w(&singletonInitializer);
    w.setAttribute(Qt::WA_DeleteOnClose, false);
    w.setAttribute(Qt::WA_QuitOnClose, false);
    w.show();
    w.setEnabled(false);

    singletonInitializer.registerInitializer(999999999, [](){}, [&](){
        sh::tools::MainThread::dispatcher()->invokeSync([&](){
            w._initialize();
            w.setCurrentProfile("main");
            w.setEnabled(true);
        });
    }, [](){}, [](){});

    // initialize all singletons (most of them are stored in sh::tools::SingletonInitializer)
    singletonInitializer.callInitializers();

    // execute the main loop in an exception-aware way
    bool succ = false;
    int result = 100;
    while (!succ)
        sh::exceptions::Exception::executeGuarded([&](){
            result = a.exec();
            succ = true;
        },sh::exceptions::ExecuteGuardFlag::ALL_ERRORS_KILL_SHALLOT);

    return result;
}
