/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "tools/iconmanager.h"
#include "tools/threadpool.h"
#include "tools/mainthread.h"
#include "apiaction.h"


sh::scripting::api::ApiAbstractActionItem::ApiAbstractActionItem()
{
}

sh::scripting::api::ApiSubmenuActionItem::ApiSubmenuActionItem(QString text, bool enabled, QString icon, int defaultActionPrecedence) :
    sh::actions::SubmenuActionItem(text, QList<std::shared_ptr<sh::actions::AbstractActionItem>>(), enabled, icon, defaultActionPrecedence)
{
}

sh::scripting::api::ApiActionActionItem::ApiActionActionItem(QString text, bool enabled, QString icon, int defaultActionPrecedence) :
    ActionActionItem(text, enabled, icon, defaultActionPrecedence)
{
}

void sh::scripting::api::ApiActionActionItem::action(sh::actions::ActionExecutionInfo* info)
{
    sh::exceptions::Exception::executeGuarded_errorpanel([&](){
        _action(info);
    });
}

void sh::scripting::api::ApiSubmenuActionItem::initialize()
{
    bool succ = false;
    sh::exceptions::Exception::executeGuarded_errorpanel([&](){
        _initialize();
        succ = true;
    });
    if (!succ)
        setEnabled(false);
}

void sh::scripting::api::ApiActionActionItem::initialize()
{
    bool succ = false;
    sh::exceptions::Exception::executeGuarded_errorpanel([&](){
        _initialize();
        succ = true;
    });
    if (!succ)
        setEnabled(false);
}

void sh::scripting::api::ApiActionActionItem::_execute()
{
    std::shared_ptr<sh::actions::AbstractActionItem> _sharedptr = shared_from_this();
    sh::tools::MainThread::dispatcher()->invokeSync([=](){
        std::shared_ptr<sh::actions::AbstractActionItem> __sharedptr = _sharedptr;
        execute();
    });
}
