/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef APIFILEPROPERTYDIALOGTAB_H
#define APIFILEPROPERTYDIALOGTAB_H

#include <functional>
#include <memory>
#include <QPair>
#include "scripting/pythonscriptinterpreter.h"
#include "ui/filepropertydialog/filepropertydialog.h"


namespace sh{
    namespace scripting {
        namespace api {

            class ApiFilePropertyDialogTab : public sh::ui::filepropertydialog::FilePropertyDialogTab
            {
            public:

                class PropertyConfig {
                public:
                    typedef QPair<QString,QString> ButtonConfig;
                    PropertyConfig(QString title, int propertytype, QList<ButtonConfig> buttons);
                    QString title;
                    int propertytype;
                    QList<ButtonConfig> buttons;
                };

                static const int PROPERTY_TYPE_STRING;
                static const int PROPERTY_TYPE_STRINGMAP;
                static const int PROPERTY_TYPE_ICONTEXTBANNER;

                ApiFilePropertyDialogTab(QString title, QList<PropertyConfig> properties);
                virtual QString title();
                virtual QList<QString> properties();
                virtual QWidget* createWidget(int i);
                virtual void updateWidget(int i, QWidget *widget, sh::base::Operation *op);

                std::function<QList<QString> (int, sh::base::Operation*, qintptr)> _updateWidget;
                std::function<void (QString)> _buttonTriggered;

            private:
                typedef QPair<QString,QString> StringPair;
                QString _title;
                QList<PropertyConfig> _properties;
            };

            class ApiFilePropertyDialogTabFactory : public sh::ui::filepropertydialog::FilePropertyDialogTabFactory
            {
            public:
                ApiFilePropertyDialogTabFactory();
                virtual std::shared_ptr<sh::ui::filepropertydialog::FilePropertyDialogTab> construct();
                std::function<std::shared_ptr<sh::scripting::api::ApiFilePropertyDialogTab>()> _construct;
            };

        }
    }
}

#endif // APIFILEPROPERTYDIALOGTAB_H
