/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "apifilesystemhandler.h"


sh::scripting::api::ApiFilesystemHandler::ApiFilesystemHandler(sh::base::FilesystemModel* model)
    : sh::base::FilesystemHandler(model)
{
}

QString sh::scripting::api::ApiFilesystemHandler::getMimetype(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl)
{
    return _getMimetype(op, std::const_pointer_cast<sh::base::Eurl>(eurl));
}

sh::base::FilesystemNodeType sh::scripting::api::ApiFilesystemHandler::getType(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl)
{
    return (sh::base::FilesystemNodeType)_getType(op, std::const_pointer_cast<sh::base::Eurl>(eurl));
}

qint64 sh::scripting::api::ApiFilesystemHandler::getSize(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl)
{
    return _getSize(op, std::const_pointer_cast<sh::base::Eurl>(eurl));
}

QDateTime sh::scripting::api::ApiFilesystemHandler::getMtime(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl)
{
    qint64 res = _getMtime(op, std::const_pointer_cast<sh::base::Eurl>(eurl));
    if (res)
        return QDateTime::fromMSecsSinceEpoch(res);
    else
        return QDateTime();
}

void sh::scripting::api::ApiFilesystemHandler::setMtime(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl, QDateTime time)
{
    qint64 itime = time.toMSecsSinceEpoch();
    _setMtime(op, std::const_pointer_cast<sh::base::Eurl>(eurl), itime);
}

QString sh::scripting::api::ApiFilesystemHandler::getLinkTarget(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl)
{
    return _getLinkTarget(op, std::const_pointer_cast<sh::base::Eurl>(eurl));
}

bool sh::scripting::api::ApiFilesystemHandler::canCreateDirectory(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl)
{
    return _canCreateDirectory(op, std::const_pointer_cast<sh::base::Eurl>(eurl));
}

void sh::scripting::api::ApiFilesystemHandler::createDirectory(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl)
{
    return _createDirectory(op, std::const_pointer_cast<sh::base::Eurl>(eurl));
}

bool sh::scripting::api::ApiFilesystemHandler::canCreateLink(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl)
{
    return _canCreateLink(op, std::const_pointer_cast<sh::base::Eurl>(eurl));
}

void sh::scripting::api::ApiFilesystemHandler::createLink(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl, QString target)
{
    return _createLink(op, std::const_pointer_cast<sh::base::Eurl>(eurl), target);
}

bool sh::scripting::api::ApiFilesystemHandler::canCreateFile(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl)
{
    return _canCreateFile(op, std::const_pointer_cast<sh::base::Eurl>(eurl));
}

bool sh::scripting::api::ApiFilesystemHandler::canDeleteItem(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl)
{
    return _canDeleteItem(op, std::const_pointer_cast<sh::base::Eurl>(eurl));
}

void sh::scripting::api::ApiFilesystemHandler::deleteItem(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl)
{
    return _deleteItem(op, std::const_pointer_cast<sh::base::Eurl>(eurl));
}

bool sh::scripting::api::ApiFilesystemHandler::canRenameItem(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> src)
{
    return _canRenameItem(op, std::const_pointer_cast<sh::base::Eurl>(src));
}

void sh::scripting::api::ApiFilesystemHandler::renameItem(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> src, QString destpath)
{
    return _renameItem(op, std::const_pointer_cast<sh::base::Eurl>(src), destpath);
}

void sh::scripting::api::ApiFilesystemHandler::itemlist(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl, sh::base::FilesystemNodeType type, sh::base::FilesystemNodeListEditor *list)
{
    _itemlist(op, std::const_pointer_cast<sh::base::Eurl>(eurl), (int)type, list);
}

void sh::scripting::api::ApiFilesystemHandler::configureItems(sh::base::Operation* op, QList<std::shared_ptr<sh::base::FilesystemNode>> nodes)
{
    _configureItems(op, nodes);
}

bool sh::scripting::api::ApiFilesystemHandler::canGetFileContent(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl)
{
    return _canGetFileContent(op, std::const_pointer_cast<sh::base::Eurl>(eurl));
}

std::shared_ptr<QIODevice> sh::scripting::api::ApiFilesystemHandler::getFileContent(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl)
{
    return _getFileContent(op, std::const_pointer_cast<sh::base::Eurl>(eurl));
}

void sh::scripting::api::ApiFilesystemHandler::createFile(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl, QIODevice* content, HandlerTransfer *handlertransfer)
{
    _createFile(op, std::const_pointer_cast<sh::base::Eurl>(eurl), content, handlertransfer);
}

QList<QString> sh::scripting::api::ApiFilesystemHandler::listExtendedAttributes(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl)
{
    return _listExtendedAttributes(op, std::const_pointer_cast<sh::base::Eurl>(eurl));
}

quint64 sh::scripting::api::ApiFilesystemHandler::getExtendedAttributeSize(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl, QString attribute)
{
    return _getExtendedAttributeSize(op, std::const_pointer_cast<sh::base::Eurl>(eurl), attribute);
}

QByteArray sh::scripting::api::ApiFilesystemHandler::getExtendedAttribute(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl, QString attribute)
{
    return _getExtendedAttribute(op, std::const_pointer_cast<sh::base::Eurl>(eurl), attribute);
}

void sh::scripting::api::ApiFilesystemHandler::setExtendedAttribute(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl, QString attribute, QByteArray value)
{
    return _setExtendedAttribute(op, std::const_pointer_cast<sh::base::Eurl>(eurl), attribute, value);
}

void sh::scripting::api::ApiFilesystemHandler::removeExtendedAttribute(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl, QString attribute)
{
    return _removeExtendedAttribute(op, std::const_pointer_cast<sh::base::Eurl>(eurl), attribute);
}

QMap<QString, QString> sh::scripting::api::ApiFilesystemHandler::getCustomAttributes(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl)
{
    QMap<QString, QString> result;
    QStringList lst = _getCustomAttributes(op, std::const_pointer_cast<sh::base::Eurl>(eurl));
    while(!lst.isEmpty())
    {
        QString key = lst.takeFirst();
        QString val = lst.takeFirst();
        result.insert(key, val);
    }
    return result;
}

void sh::scripting::api::ApiFilesystemHandler::setCustomAttribute(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl, QString attribute, QString value)
{
    _setCustomAttribute(op, std::const_pointer_cast<sh::base::Eurl>(eurl), attribute, value);
}

QList<std::shared_ptr<sh::actions::AbstractActionItem>> sh::scripting::api::ApiFilesystemHandler::getActions(const QList<std::shared_ptr<const sh::base::Eurl>> eurls)
{
    QList<std::shared_ptr<sh::base::Eurl>> _eurls;
    foreach(std::shared_ptr<const sh::base::Eurl> eurl, eurls)
        _eurls.append(std::const_pointer_cast<sh::base::Eurl>(eurl));
    return _getActions(_eurls);
}
