/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef APIFILESYSTEMHANDLER_H
#define APIFILESYSTEMHANDLER_H

#include "actions/abstractactionitem.h"
#include "base/filesystemhandler.h"


namespace sh{
    namespace scripting {
        namespace api {

            class ApiFilesystemHandler : public sh::base::FilesystemHandler
            {
            Q_OBJECT
            public:
                explicit ApiFilesystemHandler(sh::base::FilesystemModel *model);

                virtual void itemlist(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl, sh::base::FilesystemNodeType type, sh::base::FilesystemNodeListEditor *list);
                virtual void configureItems(sh::base::Operation* op, QList<std::shared_ptr<sh::base::FilesystemNode>> nodes);
                virtual sh::base::FilesystemNodeType getType(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl);

                virtual qint64 getSize(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl);

                virtual QDateTime getMtime(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl);
                virtual void setMtime(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl, QDateTime time);

                virtual QString getMimetype(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl);

                virtual QString getLinkTarget(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl);

                virtual QList<QString> listExtendedAttributes(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl);
                virtual quint64 getExtendedAttributeSize(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl, QString attribute);
                virtual QByteArray getExtendedAttribute(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl, QString attribute);
                virtual void setExtendedAttribute(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl, QString attribute, QByteArray value);
                virtual void removeExtendedAttribute(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl, QString attribute);

                virtual QMap<QString, QString> getCustomAttributes(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl);
                virtual void setCustomAttribute(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl, QString attribute, QString value);

                virtual bool canGetFileContent(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl);
                virtual std::shared_ptr<QIODevice> getFileContent(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl);

                virtual bool canCreateDirectory(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl);
                virtual void createDirectory(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl);

                virtual bool canCreateLink(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl);
                virtual void createLink(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl, QString target);

                virtual bool canCreateFile(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl);
                virtual void createFile(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl, QIODevice* content, HandlerTransfer* handlertransfer);

                virtual bool canDeleteItem(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl);
                virtual void deleteItem(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl);

                virtual bool canRenameItem(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> src);
                virtual void renameItem(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> src, QString destpath);

                virtual QList<std::shared_ptr<sh::actions::AbstractActionItem>> getActions(const QList<std::shared_ptr<const sh::base::Eurl>> eurls);

                std::function<int (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl)> _getType;
                std::function<qint64 (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl)> _getSize;
                std::function<double (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl)> _getMtime;
                std::function<void (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl, double)> _setMtime;
                std::function<QString (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl)> _getLinkTarget;
                std::function<QList<QString> (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl)> _listExtendedAttributes;
                std::function<quint64 (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl, QString attribute)> _getExtendedAttributeSize;
                std::function<QByteArray (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl, QString attribute)> _getExtendedAttribute;
                std::function<void (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl, QString attribute, QByteArray value)> _setExtendedAttribute;
                std::function<void (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl, QString attribute)> _removeExtendedAttribute;
                std::function<QList<QString> (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl)> _getCustomAttributes;
                std::function<void (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl, QString attribute, QString value)> _setCustomAttribute;
                std::function<QString (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl)> _getMimetype;
                std::function<bool (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl)> _canCreateDirectory;
                std::function<void (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl)> _createDirectory;
                std::function<bool (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl)> _canCreateLink;
                std::function<void (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl, QString target)> _createLink;
                std::function<bool (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl)> _canCreateFile;
                std::function<bool (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl)> _canDeleteItem;
                std::function<void (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl)> _deleteItem;
                std::function<bool (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> src)> _canRenameItem;
                std::function<void (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> src, QString destpath)> _renameItem;
                std::function<void (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl, int type, sh::base::FilesystemNodeListEditor *list)> _itemlist;
                std::function<void (sh::base::Operation* op, QList<std::shared_ptr<sh::base::FilesystemNode>>)> _configureItems;
                std::function<bool (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl)> _canGetFileContent;
                std::function<std::shared_ptr<QIODevice> (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl)> _getFileContent;
                std::function<void (sh::base::Operation* op, std::shared_ptr<sh::base::Eurl> eurl, QIODevice* content, HandlerTransfer* handlertransfer)> _createFile;
                std::function<QList<std::shared_ptr<sh::actions::AbstractActionItem>> (QList<std::shared_ptr<sh::base::Eurl>> eurls)> _getActions;

            };

        }
    }
}

#endif // APIFILESYSTEMHANDLER_H
