/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef APIGLOBALOBJECT_H
#define APIGLOBALOBJECT_H

#include <QLocale>
#include <QMap>
#include "actions/abstractactionitem.h"
#include "actions/actionexecutionuserfeedback.h"
#include "actions/actionfactory.h"
#include "base/detailcolumn.h"
#include "base/eurl.h"
#include "base/filesystemnode.h"
#include "base/filesystemoperationtransfers.h"
#include "configuration/configurationvalue.h"
#include "paneldetails/paneldetailmanager.h"
#include "scripting/api.h"
#include "search/searchmanager.h"
#include "settings/setting.h"
#include "tools/bookmarkmanager.h"
#include "tools/thumbnailmanager.h"
#include "ui/filepropertydialog/filepropertydialog.h"
#include "apifilepropertydialogtab.h"


namespace sh{
    namespace ui{class MainWindow;}
    namespace scripting{
        namespace api{
            class ApiFilesystemHandler;
            class ApiActionActionItem;
            class ApiSubmenuActionItem;
            class ApiDetailColumn;
            class ApiPanelDetailFactorySingle;
            class ApiPanelDetailFactoryMulti;
            class ApiReadDataDevice;
            class ApiActionFactory;
            class ApiFilesystemOperationProgressMonitor;
            class ApiSetting;
            class ApiThread;
            class ApiTimer;
            class ApiSearchCriterion;
            class ApiSearchCriterionFactory;
            class ApiThumbnailProvider;

            class ApiGlobalObject
            {
            public:
                /** \brief
                 * Returns the instance of this singleton.
                 */
                static ApiGlobalObject* instance();
                /** \brief
                 * Only called by the Shallot infrastructure for initialization.
                 */
                void _initialize();
                /** \brief
                 * Only called by the Shallot infrastructure for shutdown.
                 */
                void _shutdown();

                std::shared_ptr<const sh::base::Eurl> getEurlFromString(QString eurl)
                {
                    return sh::base::Eurl::fromString(eurl);
                }

                void logDebug(QString msg);
                void logInfo(QString msg);
                void logWarning(QString msg);
                void logError(QString msg);

                int filesystemnodetype_file() { return (int)sh::base::FilesystemNodeType::File; }
                int filesystemnodetype_directory() { return (int)sh::base::FilesystemNodeType::Directory; }
                int filesystemnodetype_link() { return (int)sh::base::FilesystemNodeType::Link; }
                int filesystemnodetype_unknown() { return (int)sh::base::FilesystemNodeType::Unknown; }
                int filesystemnodetype_firsttype() { return (int)sh::base::FilesystemNodeType::FIRSTTYPE; }
                int filesystemnodetype_lastphysicaltype() { return (int)sh::base::FilesystemNodeType::LASTPHYSICALTYPE; }
                int filesystemnodetype_none() { return (int)sh::base::FilesystemNodeType::NONE; }
                int filesystemnodetype_specialtreeonlydirectory() { return (int)sh::base::FilesystemNodeType::SpecialTreeOnlyDirectory; }
                int filesystemnodetype_lasttype() { return (int)sh::base::FilesystemNodeType::LASTTYPE; }

                int actiondefaultprecedencevalues_openfile() { return sh::actions::ActionDefaultPrecedenceValues::PRECEDENCE_OPENFILE; }
                int actiondefaultprecedencevalues_builtinspecialopen() { return sh::actions::ActionDefaultPrecedenceValues::PRECEDENCE_BUILTIN_SPECIAL_OPEN; }

                int messageboxbutton_ok() { return (int)sh::actions::ActionExecutionUserFeedback::MessageBoxButton::OK; }
                int messageboxbutton_continue() { return (int)sh::actions::ActionExecutionUserFeedback::MessageBoxButton::Continue; }
                int messageboxbutton_cancel() { return (int)sh::actions::ActionExecutionUserFeedback::MessageBoxButton::Cancel; }
                int messageboxbutton_retry() { return (int)sh::actions::ActionExecutionUserFeedback::MessageBoxButton::Retry; }
                int messageboxbutton_yes() { return (int)sh::actions::ActionExecutionUserFeedback::MessageBoxButton::Yes; }
                int messageboxbutton_no() { return (int)sh::actions::ActionExecutionUserFeedback::MessageBoxButton::No; }

                int displayindex_core() { return sh::base::DetailColumn::DISPLAYINDEX_CORE; }
                int displayindex_interesting() { return sh::base::DetailColumn::DISPLAYINDEX_INTERESTING; }
                int displayindex_exotic() { return sh::base::DetailColumn::DISPLAYINDEX_EXOTIC; }

                int configurationcategory_gui() { return sh::configuration::ConfigurationCategory::CategoryGUI; }
                int configurationcategory_behavior() { return sh::configuration::ConfigurationCategory::CategoryBehavior; }
                int configurationcategory_externaltools() { return sh::configuration::ConfigurationCategory::CategoryExternalTools; }

                int settinggroup_global() { return sh::settings::SettingGroup::GLOBAL; }
                int settinggroup_gui() { return sh::settings::SettingGroup::GUI; }
                int settinggroup_filehandling() { return sh::settings::SettingGroup::FILEHANDLING; }
                int settinggroup_dataview() { return sh::settings::SettingGroup::DATAVIEW; }
                int settinggroup_behavior() { return sh::settings::SettingGroup::BEHAVIOR; }
                int settinggroup_special() { return sh::settings::SettingGroup::SPECIAL; }

                int paneldetail_positionindex_veryinteresting() { return sh::paneldetails::PanelDetailPositionIndex::VeryInteresting; }
                int paneldetail_positionindex_interesting() { return sh::paneldetails::PanelDetailPositionIndex::Interesting; }
                int paneldetail_positionindex_exotic() { return sh::paneldetails::PanelDetailPositionIndex::Exotic; }

                int operationstep_conflictresolution_mergedirectories() { return (int)sh::base::FilesystemOperationTransfers::OperationStep::ConflictResolution::MergeDirectories; }
                int operationstep_conflictresolution_overwritedestination() { return (int)sh::base::FilesystemOperationTransfers::OperationStep::ConflictResolution::OverwriteDestination; }
                int operationstep_conflictresolution_renamedestinationbefore() { return (int)sh::base::FilesystemOperationTransfers::OperationStep::ConflictResolution::RenameDestinationBefore; }
                int operationstep_conflictresolution_skip() { return (int)sh::base::FilesystemOperationTransfers::OperationStep::ConflictResolution::Skip; }
                int operationstep_conflictresolution_unresolved() { return (int)sh::base::FilesystemOperationTransfers::OperationStep::ConflictResolution::Unresolved; }
                int operationstep_conflictresolution_indirect() { return (int)sh::base::FilesystemOperationTransfers::OperationStep::ConflictResolution::Indirect; }
                int operationstep_conflictresolution_usedifferentdestinationname() { return (int)sh::base::FilesystemOperationTransfers::OperationStep::ConflictResolution::UseDifferentDestinationName; }

                int filepropertydialogtab_index_core() { return sh::ui::filepropertydialog::FilePropertyDialogTabFactory::REGISTER_FILEPROPERTYDIALOGTAB_INDEX_CORE; }
                int filepropertydialogtab_index_veryimportant() { return sh::ui::filepropertydialog::FilePropertyDialogTabFactory::REGISTER_FILEPROPERTYDIALOGTAB_INDEX_VERYIMPORTANT; }
                int filepropertydialogtab_index_important() { return sh::ui::filepropertydialog::FilePropertyDialogTabFactory::REGISTER_FILEPROPERTYDIALOGTAB_INDEX_IMPORTANT; }
                int filepropertydialogtab_index_normal() { return sh::ui::filepropertydialog::FilePropertyDialogTabFactory::REGISTER_FILEPROPERTYDIALOGTAB_INDEX_NORMAL; }
                int filepropertydialogtab_index_exotic() { return sh::ui::filepropertydialog::FilePropertyDialogTabFactory::REGISTER_FILEPROPERTYDIALOGTAB_INDEX_EXOTIC; }
                int filepropertydialogtab_propertytype_string() { return sh::scripting::api::ApiFilePropertyDialogTab::PROPERTY_TYPE_STRING; }
                int filepropertydialogtab_propertytype_stringmap() { return sh::scripting::api::ApiFilePropertyDialogTab::PROPERTY_TYPE_STRINGMAP; }
                int filepropertydialogtab_propertytype_icontextbanner() { return sh::scripting::api::ApiFilePropertyDialogTab::PROPERTY_TYPE_ICONTEXTBANNER; }

                int searchcriterionfactory_index_core() { return sh::search::SearchManager::REGISTER_SEARCHCRITERION_INDEX_CORE; }
                int searchcriterionfactory_index_normal() { return sh::search::SearchManager::REGISTER_SEARCHCRITERION_INDEX_NORMAL; }
                int searchcriterionfactory_index_exotic() { return sh::search::SearchManager::REGISTER_SEARCHCRITERION_INDEX_EXOTIC; }
                int thumbnailprovider_index_core() { return sh::tools::ThumbnailManager::REGISTER_THUMBNAILPROVIDER_INDEX_CORE; }
                int thumbnailprovider_index_normal() { return sh::tools::ThumbnailManager::REGISTER_THUMBNAILPROVIDER_INDEX_NORMAL; }
                int thumbnailprovider_index_exotic() { return sh::tools::ThumbnailManager::REGISTER_THUMBNAILPROVIDER_INDEX_EXOTIC; }
                int thumbnailprovider_index_fallback() { return sh::tools::ThumbnailManager::REGISTER_THUMBNAILPROVIDER_INDEX_FALLBACK; }

                void register_predicatedactionfactory(std::shared_ptr<sh::scripting::api::ApiActionFactory> f);

                bool isDebugBuild();

                std::shared_ptr<sh::scripting::api::ApiFilesystemHandler> create_FilesystemHandler();
                std::shared_ptr<sh::scripting::api::ApiActionActionItem> create_ActionActionItem(QString text, bool enabled, QString icon, int defaultActionPrecedence);
                std::shared_ptr<sh::scripting::api::ApiSubmenuActionItem> create_SubmenuActionItem(QString text, bool enabled, QString icon, int defaultActionPrecedence);
                std::shared_ptr<sh::scripting::api::ApiDetailColumn> create_DetailColumn(QString name, QString displayName, int displayIndex, bool sort_doTypediff, int defaultWidth, bool isRightAligned);
                std::shared_ptr<sh::scripting::api::ApiFilePropertyDialogTab> create_FilePropertyDialogTab(QString title, QList<QString> properties);
                std::shared_ptr<sh::scripting::api::ApiFilePropertyDialogTabFactory> create_FilePropertyDialogTabFactory();
                std::shared_ptr<sh::scripting::api::ApiActionFactory> create_ActionFactory(QString category, QList<std::shared_ptr<actions::Predicate> > predicates);
                std::shared_ptr<sh::scripting::api::ApiPanelDetailFactorySingle> create_DetailFactorySingle(int position, int valueWidthHint);
                std::shared_ptr<sh::scripting::api::ApiPanelDetailFactoryMulti> create_DetailFactoryMulti(int position, int valueWidthHint);
                std::shared_ptr<sh::scripting::api::ApiFilesystemOperationProgressMonitor> create_FilesystemOperationProgressMonitor(sh::actions::ActionExecutionInfo* actionExecution);
                std::shared_ptr<sh::base::FilesystemNode> _createFilesystemNode(std::shared_ptr<sh::base::Eurl> eurl, sh::scripting::api::ApiFilesystemHandler *handler, int nodetype, std::shared_ptr<sh::base::FilesystemNode> parent, bool doinsert, bool showInitialLoadingLabel, bool isHidden);
                std::shared_ptr<sh::scripting::api::ApiThumbnailProvider> create_ThumbnailProvider();
                std::shared_ptr<sh::base::FilesystemNode> _getOrCreateFilesystemNode(std::shared_ptr<sh::base::Eurl> eurl, sh::scripting::api::ApiFilesystemHandler* handler, int nodetype, std::shared_ptr<sh::base::FilesystemNode> parent, bool doinsert, bool showInitialLoadingLabel, bool isHidden);
                void _register_FilesystemHandler(std::shared_ptr<sh::scripting::api::ApiFilesystemHandler> handler, QString scheme);
                std::shared_ptr<sh::base::FilesystemNode> modelRootNode();
                QList<std::shared_ptr<sh::base::FilesystemNode>> _findFilesystemNodesForEurl(std::shared_ptr<const sh::base::Eurl> eurl);
                QList<std::shared_ptr<sh::base::FilesystemNode>> _tryGetFilesystemNodesForEurl(std::shared_ptr<const sh::base::Eurl> eurl);
                void refreshData(std::shared_ptr<const sh::base::Eurl> eurl, bool forceFindParent, bool withDetails);
                std::shared_ptr<base::Eurl> createEurl(QString scheme, QString hostname, QString path);
                std::shared_ptr<sh::base::Operation> createOperation();
                sh::scripting::StringMap<QString> createArgumentException();
                sh::scripting::StringMap<QString> createIOException();
                sh::scripting::StringMap<QString> createRuntimeException();
                sh::scripting::StringMap<QString> createProgramException();
                sh::scripting::StringMap<QString> createException();
                std::shared_ptr<sh::scripting::api::ApiReadDataDevice> _createReadDataDevice();
                std::shared_ptr<sh::settings::Setting> createSetting(QString name, QString description, int group, bool isAdvancedSetting, bool isGlobal, bool isPerFileview);
                std::shared_ptr<sh::scripting::api::ApiThread> create_Thread();
                std::shared_ptr<sh::scripting::api::ApiTimer> create_Timer();
                std::shared_ptr<sh::scripting::api::ApiSearchCriterion> create_SearchCriterion(std::shared_ptr<sh::scripting::api::ApiSearchCriterionFactory> factory);
                std::shared_ptr<sh::scripting::api::ApiSearchCriterionFactory> create_SearchCriterionFactory(QString key, QString description);
                sh::ui::MainWindow* mainwindow();
                void registerPanelDetailFactorySingle(std::shared_ptr<sh::scripting::api::ApiPanelDetailFactorySingle>);
                void registerPanelDetailFactoryMulti(std::shared_ptr<sh::scripting::api::ApiPanelDetailFactoryMulti>);
                void registerSetting(std::shared_ptr<sh::settings::Setting> setting);
                void openItems(QList<std::shared_ptr<const sh::base::Eurl>> eurls);

                void registerFilePropertyDialogTabFactory(int index, std::shared_ptr<sh::scripting::api::ApiFilePropertyDialogTabFactory>);
                void registerThumbnailProvider(int idx, std::shared_ptr<sh::scripting::api::ApiThumbnailProvider> thumbnailprovider);
                void registerSearchCriterionFactory(int idx, std::shared_ptr<ApiSearchCriterionFactory> apicrit);

                std::shared_ptr<sh::configuration::ConfigurationValue> register_ConfigurationValueInt(QString name, int deflt, QString desc, int category, QString longdesc, QString changehint);
                std::shared_ptr<sh::configuration::ConfigurationValue> register_ConfigurationValueBool(QString name, bool deflt, QString desc, int category, QString longdesc, QString changehint);
                std::shared_ptr<sh::configuration::ConfigurationValue> register_ConfigurationValueString(QString name, QString deflt, QString desc, int category, QString longdesc, QString changehint);

                void registerTransferrableDetailColumn(int i, std::shared_ptr<sh::base::DetailColumn> detailColumn);
                std::shared_ptr<sh::base::DetailColumn> findDetailColumnByName(QString name);

                QString shallotDataDir();
                QString shallotBuiltinPluginDir();
                QString shallotSystemPluginDir();
                QString shallotUserPluginDir();
                QString shallotLanguage();

                sh::tools::BookmarkManager* bookmarkManager();

                std::shared_ptr<sh::actions::Predicate> create_OnDirectoriesPredicate();
                std::shared_ptr<sh::actions::Predicate> create_OnFilesPredicate();
                std::shared_ptr<sh::actions::Predicate> create_OnLinksPredicate();
                std::shared_ptr<sh::actions::Predicate> create_OnSingleEntrySelectionPredicate();
                std::shared_ptr<sh::actions::Predicate> create_DontResolveLinksPredicate();
                std::shared_ptr<sh::actions::Predicate> create_HideOnCurrentDirectoryLevelPredicate();
                std::shared_ptr<sh::actions::Predicate> create_HideOnSelectionLevelPredicate();
                std::shared_ptr<sh::actions::Predicate> create_ByRegExpPredicate(QString pattern);
                std::shared_ptr<sh::actions::Predicate> create_KeyShortcutPredicate(QString shortcut, bool triggersOnCurrentDirectoryLevel);
                std::shared_ptr<sh::actions::Predicate> create_PositionIndexPredicate(int index);

                void deleteLater();

            private:
                ApiGlobalObject();
            };

        }
    }
}

#endif // APIGLOBALOBJECT_H
