/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef APIHELPERMETHODS_H
#define APIHELPERMETHODS_H

#include <memory>
#include <QByteArray>
#include <QDateTime>
#include <QListView>
#include <QIODevice>
#include "base/filesystemoperationtransfers.h"


namespace sh{
    namespace actions{ class ActionExecutionUserFeedback; }
    namespace base{ class Eurl; class FilesystemNode;class FilesystemNodeList;class DetailColumn; class FilesystemOperation; }
    namespace configuration { class ConfigurationValue; }
    namespace paneldetails { class PanelDetail; }
    namespace ui { class MainWindow;
        namespace filepropertydialog {
            class FilePropertyDialog;
        }
    }
    namespace scripting {
        namespace api {
            class ApiActionActionItem;
            class ApiSubmenuActionItem;
            class ApiReadDataDevice;
            class ApiFilesystemOperationProgressMonitor;
            class ApiFilePropertyDialogTab;

            class ApiHelperMethods
            {
            public:

                static QByteArray QIODevice_read(QIODevice* self);
                static QByteArray QIODevice_readall(QIODevice* self);

                static QByteArray ApiReadDataDevice_read(sh::scripting::api::ApiReadDataDevice* self);
                static QByteArray ApiReadDataDevice_readall(sh::scripting::api::ApiReadDataDevice* self);

                static void FilesystemNodeList_resetItems(sh::base::FilesystemNodeList* self, int type, QList<std::shared_ptr<sh::base::FilesystemNode>> list);

                static void FilesystemNode_addDetail(base::FilesystemNode *self, std::shared_ptr<sh::base::DetailColumn> col);
                static void FilesystemNode_setIcon(sh::base::FilesystemNode* self, QString icon);
                static void FilesystemNode_setDisplayName(base::FilesystemNode *self, QString displayname);
                static bool FilesystemNode_isHidden(base::FilesystemNode *self);
                static void FilesystemNode_setHidden(base::FilesystemNode *self, bool v);

                static void ConfigurationValue_setConfigValueInt(sh::configuration::ConfigurationValue* self, int v);
                static void ConfigurationValue_setConfigValueBool(sh::configuration::ConfigurationValue* self,bool v);
                static void ConfigurationValue_setConfigValueString(sh::configuration::ConfigurationValue* self,QString v);

                static QList<std::shared_ptr<sh::base::Eurl>> FilesystemOperation_itemlist(sh::base::FilesystemOperation *self, std::shared_ptr<const sh::base::Eurl> eurl);
                static QList<std::shared_ptr<sh::base::Eurl>> FilesystemOperation_itemlistByType(sh::base::FilesystemOperation *self, std::shared_ptr<const sh::base::Eurl> eurl, int ntype);
                static int FilesystemOperation_getType(sh::base::FilesystemOperation *self, std::shared_ptr<const sh::base::Eurl> eurl);
                static void FilesystemOperation_createFile(sh::base::FilesystemOperation *self, std::shared_ptr<const sh::base::Eurl> eurl, sh::scripting::api::ApiReadDataDevice* content, std::shared_ptr<ApiFilesystemOperationProgressMonitor> progressmon);
                static QList<std::shared_ptr<sh::base::FilesystemNode>> FilesystemOperation_resolveNodeLink(sh::base::FilesystemOperation *self, std::shared_ptr<sh::base::FilesystemNode> node);
                static QList<std::shared_ptr<sh::base::FilesystemNode>> FilesystemOperation_resolveNodeLink_NonRecursive(sh::base::FilesystemOperation *self, std::shared_ptr<sh::base::FilesystemNode> node);
                static std::shared_ptr<sh::base::Eurl> FilesystemOperation_resolveEurlLink(sh::base::FilesystemOperation *self, std::shared_ptr<const sh::base::Eurl> eurl);
                static std::shared_ptr<sh::base::Eurl> FilesystemOperation_resolveEurlLink_NonRecursive(sh::base::FilesystemOperation *self, std::shared_ptr<const sh::base::Eurl> eurl);

                static void PanelDetail_setRow(sh::paneldetails::PanelDetail* self, QString key, QList<QString> value);

                static void MainWindow_jumpToEurl(sh::ui::MainWindow* self, std::shared_ptr<const sh::base::Eurl> eurl);
                static std::shared_ptr<sh::base::FilesystemNode> MainWindow_getCurrentDirectoryNode(sh::ui::MainWindow* self);

                static void ApiActionActionItem_setEnabled(sh::scripting::api::ApiActionActionItem* self, bool enabled);
                static bool ApiActionActionItem_enabled(sh::scripting::api::ApiActionActionItem* self);
                static void ApiActionActionItem_setVisible(sh::scripting::api::ApiActionActionItem* self, bool visible);
                static bool ApiActionActionItem_visible(sh::scripting::api::ApiActionActionItem* self);
                static void ApiSubmenuActionItem_setEnabled(sh::scripting::api::ApiSubmenuActionItem* self, bool enabled);
                static bool ApiSubmenuActionItem_enabled(sh::scripting::api::ApiSubmenuActionItem* self);
                static void ApiSubmenuActionItem_setVisible(sh::scripting::api::ApiSubmenuActionItem* self, bool visible);
                static bool ApiSubmenuActionItem_visible(sh::scripting::api::ApiSubmenuActionItem* self);

                static int OperationStep_conflictResolution(base::FilesystemOperationTransfers::OperationStep *self);

                static bool FilesystemOperationProgressMonitor_hasItemInfo(sh::scripting::api::ApiFilesystemOperationProgressMonitor *self);
                static quint64 FilesystemOperationProgressMonitor_doneItems(sh::scripting::api::ApiFilesystemOperationProgressMonitor *self);
                static quint64 FilesystemOperationProgressMonitor_allItems(sh::scripting::api::ApiFilesystemOperationProgressMonitor *self);
                static bool FilesystemOperationProgressMonitor_hasBytesInfo(sh::scripting::api::ApiFilesystemOperationProgressMonitor *self);
                static quint64 FilesystemOperationProgressMonitor_doneBytes(sh::scripting::api::ApiFilesystemOperationProgressMonitor *self);
                static quint64 FilesystemOperationProgressMonitor_allBytes(sh::scripting::api::ApiFilesystemOperationProgressMonitor *self);
                static QString FilesystemOperationProgressMonitor_getItemInfoFrom(sh::scripting::api::ApiFilesystemOperationProgressMonitor *self);
                static QString FilesystemOperationProgressMonitor_getItemInfoTo(sh::scripting::api::ApiFilesystemOperationProgressMonitor *self);
                static QString FilesystemOperationProgressMonitor_estimation(sh::scripting::api::ApiFilesystemOperationProgressMonitor *self);

                static QList<std::shared_ptr<sh::base::FilesystemNode>> ApiFilePropertyDialogTab_nodes(sh::scripting::api::ApiFilePropertyDialogTab* self);

                static int ActionExecutionUserFeedback_messageBox(sh::actions::ActionExecutionUserFeedback* self, QString m, QList<QString> l, QString s , int i, int j);

                static int FilePropertyDialogTab_selectedIndexForProperty(ApiFilePropertyDialogTab* self, qintptr widget);
                static void FilePropertyDialogTab_refresh(ApiFilePropertyDialogTab* self);

            };

        }
    }
}

#endif // APIHELPERMETHODS_H
