/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "tools/threadpool.h"
#include "apipaneldetailfactory.h"


sh::scripting::api::ApiPanelDetailFactorySingle::ApiPanelDetailFactorySingle(int position, int valueWidthHint)
        : sh::paneldetails::PanelDetailFactoryForFunctionSingle(std::bind(&sh::scripting::api::ApiPanelDetailFactorySingle::setvalue, this, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3),
                                                                std::bind(&sh::scripting::api::ApiPanelDetailFactorySingle::linktriggered, this, std::placeholders::_1, std::placeholders::_2),
                                                                position, valueWidthHint)
{
}

void sh::scripting::api::ApiPanelDetailFactorySingle::setvalue(std::shared_ptr<sh::paneldetails::PanelDetail> detail, std::shared_ptr<sh::base::FilesystemNode> node, sh::base::Operation* op)
{
    sh::exceptions::Exception::executeGuarded_errorpanel([&](){
        _setvalue(detail.get(), node, op);
    });
}

void sh::scripting::api::ApiPanelDetailFactorySingle::linktriggered(std::shared_ptr<sh::base::FilesystemNode> node, QString linktarget)
{
    sh::tools::ThreadPool::enqueueForce([=](){
        sh::exceptions::Exception::executeGuarded([&](){
            _linktriggered(node, linktarget);
        }, sh::exceptions::ExecuteGuardFlag::RESUMEABLE_PROGRAM_ERROR_END_WITH_USER_FEEDBACK_HERE);
    });
}

sh::scripting::api::ApiPanelDetailFactoryMulti::ApiPanelDetailFactoryMulti(int position, int valueWidthHint)
        : sh::paneldetails::PanelDetailFactoryForFunctionMulti(std::bind(&sh::scripting::api::ApiPanelDetailFactoryMulti::setvalue, this, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3),
                                                               std::bind(&sh::scripting::api::ApiPanelDetailFactoryMulti::linktriggered, this, std::placeholders::_1, std::placeholders::_2),
                                                               position, valueWidthHint)
{
}

void sh::scripting::api::ApiPanelDetailFactoryMulti::setvalue(std::shared_ptr<sh::paneldetails::PanelDetail> detail, QList<std::shared_ptr<sh::base::FilesystemNode>> nodes, sh::base::Operation* op)
{
    sh::exceptions::Exception::executeGuarded_errorpanel([&](){
        _setvalue(detail.get(), nodes, op);
    });
}

void sh::scripting::api::ApiPanelDetailFactoryMulti::linktriggered(QList<std::shared_ptr<sh::base::FilesystemNode>> nodes, QString linktarget)
{
    sh::tools::ThreadPool::enqueueForce([=](){
        sh::exceptions::Exception::executeGuarded([&](){
            _linktriggered(nodes, linktarget);
        }, sh::exceptions::ExecuteGuardFlag::RESUMEABLE_PROGRAM_ERROR_END_WITH_USER_FEEDBACK_HERE);
    });
}
