/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "exceptions/exception.h"
#include "apisearchcriterion.h"


sh::scripting::api::ApiSearchCriterionFactory::ApiSearchCriterionFactory(QString key, QString description) :
                _key(key), _description(description)
{
}

QString sh::scripting::api::ApiSearchCriterionFactory::ApiSearchCriterionFactory::key()
{
    return _key;
}

QString sh::scripting::api::ApiSearchCriterionFactory::description()
{
    return _description;

}

QWidget* sh::scripting::api::ApiSearchCriterionFactory::editor(std::shared_ptr<sh::search::SearchCriterion> c, AuxiliaryButtonGenerator makeAuxButton)
{
    QWidget* result = sh::search::criteria::AbstractActionDrivenSearchCriterion::_createEditor(c, makeAuxButton);
    return result;
}

void sh::scripting::api::ApiSearchCriterionFactory::editorupdateconfig(QWidget* w, std::shared_ptr<sh::search::SearchCriterion> c)
{
    sh::search::criteria::AbstractActionDrivenSearchCriterion::_editorUpdateConfiguration(w, c);
}

std::shared_ptr<sh::search::SearchCriterion> sh::scripting::api::ApiSearchCriterionFactory::construct()
{
    return _construct();
}

bool sh::scripting::api::ApiSearchCriterionFactory::isVisibleFor(sh::base::Operation* op, std::shared_ptr<sh::base::FilesystemNode> node)
{
    bool result = false;
    sh::exceptions::Exception::executeGuarded_errorpanel([&](){
        result = _isVisibleFor(op, node);
    });
    return result;
}

sh::scripting::api::ApiSearchCriterion::ApiSearchCriterion(std::shared_ptr<search::SearchCriterionFactory> factory)
                : sh::search::criteria::AbstractActionDrivenSearchCriterion(factory)
{
}

void sh::scripting::api::ApiSearchCriterion::configure(sh::actions::ActionExecutionInfo* info)
{
    _configure(info);
}

bool sh::scripting::api::ApiSearchCriterion::match2(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl)
{
    bool result = false;
    sh::exceptions::Exception::executeGuarded_errorpanel([&](){
        result = _match(op, std::const_pointer_cast<sh::base::Eurl>(eurl));
    });
    return result;
}

QString sh::scripting::api::ApiSearchCriterion::getSearchSpecDescription()
{
    QString r = std::static_pointer_cast<sh::scripting::api::ApiSearchCriterionFactory>(factory())->_getsearchspecdesc(searchspec);
    if (r.isNull())
        r = sh::search::criteria::AbstractActionDrivenSearchCriterion::getSearchSpecDescription();
    return r;
}

QList<QString> sh::scripting::api::ApiSearchCriterion::getsearchspec()
{
    return searchspec;
}
