/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "exceptions/exception.h"
#include "ui/fileview/fileviewcontrol.h"
#include "apisetting.h"


sh::scripting::api::ApiSetting::ApiSetting(QString name, QString description, sh::settings::SettingGroup group, bool isAdvancedSetting, bool isGlobal, bool isPerFileview)
            : _name(name), _description(description), _group(group), _isAdvancedSetting(isAdvancedSetting),
                _isGlobal(isGlobal), _isPerFileview(isPerFileview)
{
}

QString sh::scripting::api::ApiSetting::name()
{
    return _name;
}

QString sh::scripting::api::ApiSetting::description()
{
    return _description;
}

sh::settings::SettingGroup sh::scripting::api::ApiSetting::group()
{
    return _group;
}

bool sh::scripting::api::ApiSetting::isAdvancedSetting()
{
    return _isAdvancedSetting;
}

void sh::scripting::api::ApiSetting::setValue(QString value)
{
    sh::exceptions::Exception::executeGuarded_errorpanel([&](){
        _setValue1(value);
    });
}

void sh::scripting::api::ApiSetting::setValue(sh::ui::fileview::FileViewControl *filelist, QString value )
{
    sh::exceptions::Exception::executeGuarded_errorpanel([&](){
        _setValue2(filelist->index(), value);
    });
}

QString sh::scripting::api::ApiSetting::getValue(sh::ui::fileview::FileViewControl *filelist )
{
    QString result;
    sh::exceptions::Exception::executeGuarded_errorpanel([&](){
        result = _getValue(filelist->index());
    });
    return result;
}

bool sh::scripting::api::ApiSetting::isGlobal()
{
    return _isGlobal;
}

bool sh::scripting::api::ApiSetting::isPerFileview()
{
    return _isPerFileview;
}

QString sh::scripting::api::ApiSetting::valueDescription(QString value)
{
    return _valueDescription(value);
}
