/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "tools/threadpool.h"
#include "tools/mainthread.h"
#include "apitimer.h"


sh::scripting::api::ApiTimer::ApiTimer()
{
    QObject::connect(&_timer, &QTimer::timeout, [=](){
        sh::tools::ThreadPool::enqueueForce([=](){
            {
                QMutexLocker _l1(&_mutex);
                if (_isexecuting)
                    return;
                _isexecuting = true;
            }
            sh::exceptions::Exception::executeGuarded_errorpanel([&](){
                _run();
            });
            {
                QMutexLocker _l1(&_mutex);
                _isexecuting = false;
            }
        }, shared_from_this());
    });
}

void sh::scripting::api::ApiTimer::start(int interval)
{
    sh::tools::MainThread::dispatcher()->invokeSync([&](){
        _runningTimers.append(shared_from_this());
        _timer.start(interval);
    });
}

void sh::scripting::api::ApiTimer::stop()
{
    sh::tools::MainThread::dispatcher()->invokeSync([&](){
        _runningTimers.removeOne(shared_from_this());
        _timer.stop();
    });
}
