import shallot
import time

# The class shallot.Actions.SubmenuAction is the subclass of
# submenu actions.

class MyActionMenu(shallot.Actions.SubmenuAction):

    def __init__(self, nodes):
        shallot.Actions.SubmenuAction.__init__(self, "Hello World!", icon="dialog-warning")

    def initialize(self):
        self.set_subitems([LogAction(),
                          WaitAction()])


class LogAction(shallot.Actions.ActionAction):

    def __init__(self):
        shallot.Actions.ActionAction.__init__(self, "Log action")

    def action(self, info):
        shallot.Logging.log_info("Hello world!")


class WaitAction(shallot.Actions.ActionAction):

    def __init__(self):
        shallot.Actions.ActionAction.__init__(self, "Wait action")

    def action(self, info):
        info.set_visualprocessfeedback_active(True)  # otherwise the progress dialog comes after some seconds
        for i in range(1000):
            info.respect_cancel()  # allows the user to cancel the execution
            info.set_head("Wrangling cattle")
            info.set_progress(i, 1000, "{} of {} unicorns".format(i, 1000))
            time.sleep(0.01)


# Register MyActionMenu so it is visible in the toolbar (in "Manage" per default).
# Note: This has some constructor requirements.
# Often we use HideOnSelectionLevelPredicate in order to prevent see the actions for the directory level
# and for the selection level. This typically makes sense for actions which don't act on your selection.
shallot.Actions.register(MyActionMenu, predicates=[shallot.Actions.HideOnSelectionLevelPredicate()])


