import shallot
import time
import datetime

# Inside actions, you can access a shallot.Operations.FilesystemOperation instance in `info.operation().filesystem()`.
# It provides for common operations on the filesystem. See the scripting help for reference.


class MyActionMenu(shallot.Actions.SubmenuAction):

    def __init__(self, nodes):
        shallot.Actions.SubmenuAction.__init__(self, "FS ops examples 1", icon="dialog-warning")
        self.nodes = nodes

    def initialize(self):
        self.set_subitems([ItemListAction(self.nodes),
                          CreateDirectoryAction(self.nodes),
                          GetSizeAction(self.nodes),
                          GetMimetypeAction(self.nodes)])


class ItemListAction(shallot.Actions.ActionAction):

    def __init__(self, nodes):
        shallot.Actions.ActionAction.__init__(self, "List current directory")
        self.nodes = nodes

    def action(self, info):
        currentdir = self.nodes[0].eurl()
        items = info.operation().filesystem().itemlist(currentdir)
        itemstrings = [x.basename() for x in items]
        info.userfeedback().simple_messagebox(", ".join(itemstrings), shallot.Actions.ExecutionUserFeedback.MessageBoxButton.OK, None, shallot.Actions.ExecutionUserFeedback.MessageBoxButton.OK, 0)


class CreateDirectoryAction(shallot.Actions.ActionAction):

    def __init__(self, nodes):
        shallot.Actions.ActionAction.__init__(self, "Create directory")
        self.nodes = nodes

    def action(self, info):
        currentdir = self.nodes[0].eurl()
        answer = info.userfeedback().simple_inputbox("Name for the new directory:", "New directory {}".format(datetime.datetime.now().strftime("%m-%d %H%M")))
        if answer is not None:
            info.operation().filesystem().create_directory(currentdir.with_appended_segment(answer), None)


class GetSizeAction(shallot.Actions.ActionAction):

    def __init__(self, nodes):
        shallot.Actions.ActionAction.__init__(self, "Get size of a file")
        self.nodes = nodes

    def action(self, info):
        currentdir = self.nodes[0].eurl()
        answer = info.userfeedback().simple_inputbox("Request size for which item?", "")
        if answer is not None:
            item = currentdir.with_appended_segment(answer)
            res = len(info.operation().filesystem().get_file_content(item).readall())
            info.userfeedback().simple_messagebox("Size of {0}: {1}".format(answer, res), shallot.Actions.ExecutionUserFeedback.MessageBoxButton.OK, None, shallot.Actions.ExecutionUserFeedback.MessageBoxButton.OK, 0)


class GetMimetypeAction(shallot.Actions.ActionAction):

    def __init__(self, nodes):
        shallot.Actions.ActionAction.__init__(self, "Get mime-type of a file")
        self.nodes = nodes

    def action(self, info):
        currentdir = self.nodes[0].eurl()
        answer = info.userfeedback().simple_inputbox("Request mime-type for which item?", "")
        if answer is not None:
            item = currentdir.with_appended_segment(answer)
            res = info.operation().filesystem().get_mimetype(item)
            info.userfeedback().simple_messagebox("Mime-type of {0}: {1}".format(answer, res), shallot.Actions.ExecutionUserFeedback.MessageBoxButton.OK, None, shallot.Actions.ExecutionUserFeedback.MessageBoxButton.OK, 0)


shallot.Actions.register(MyActionMenu, predicates=[shallot.Actions.HideOnSelectionLevelPredicate()])


