import os
import shallot
import sys

# There are some places in the Shallot programming interface, which work with data streams. The example
# 10.filesystem.py shows one of them, where a custom filesystem implementation answers a request for a file content.
# You may find others when you search the documentation for usages of the shallot.Streaming.ReadDataDevice class.
# One way (though not the simplest one) for a Shallot plugin to provide such a data stream is to subclass
# ThreadedReadDataDevice, like here:

class MyThreadedReadDataDevice(shallot.Streaming.ThreadedReadDataDevice):

    def __init__(self):
        shallot.Streaming.ThreadedReadDataDevice.__init__(self)

    def run(self):
        self.write(b"Some system data:\n\n")
        self.write(" Platform: {}\n".format(sys.platform).encode())
        self.write(" Python version: {}\n".format(sys.implementation.version).encode())
        self.write(" Your user name: {}\n".format(os.getlogin()).encode())
        self.write(" Your user id: {}\n".format(os.getuid()).encode())


# Although there are by far more exciting uses for a data stream, here we just implement an action which
# shows what our data stream contains.

class ShowStreamContentAction(shallot.Actions.ActionAction):

    def __init__(self, nodes):
        shallot.Actions.ActionAction.__init__(self, "Show my dynamic stream content")

    def action(self, info):
        stream = MyThreadedReadDataDevice()
        content = stream.readall().decode()
        info.set_head("The stream content")
        info.userfeedback().messagebox(content, ["OK"], None, 0, 0)


shallot.Actions.register(ShowStreamContentAction, predicates=[shallot.Actions.HideOnSelectionLevelPredicate()])

