/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ABSTRACTACTIONDRIVENSEARCHCRITERION_H
#define ABSTRACTACTIONDRIVENSEARCHCRITERION_H

#include <QSet>
#include <QWidget>
#include "actions/actionactionitem.h"
#include "search/searchcriterion.h"
#include "search/searchcriterionfactory.h"


namespace sh{
    namespace base{ class Eurl; class FilesystemNode; }

    namespace search{
        namespace criteria {

            class AbstractActionDrivenSearchCriterion : public SearchCriterion
            {
                friend class ActionAbstractActionDrivenSearchCriterionConfigure;
            public:
                QStringList searchspec;

                AbstractActionDrivenSearchCriterion(std::shared_ptr<sh::search::SearchCriterionFactory> factory);

                virtual bool match(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl);
                virtual bool match2(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl) =0;
                virtual QString valuedescription();

                static QWidget* _createEditor(std::shared_ptr<sh::search::SearchCriterion> cfg, sh::search::SearchCriterionFactory::AuxiliaryButtonGenerator makeAuxButton);
                static void _editorUpdateConfiguration(QWidget* w, std::shared_ptr<sh::search::SearchCriterion> c);

            protected:
                static void updatelabel(std::shared_ptr<sh::search::SearchCriterion> cfg, QWidget *w);
                virtual void _deserialize(QString s);
                virtual QString _serialize();
                virtual void configure(sh::actions::ActionExecutionInfo* info) =0;
                virtual QString getSearchSpecDescription();

                static QSet<QWidget*> _aliveWidgets;
            };

            class ActionAbstractActionDrivenSearchCriterionConfigure : public sh::actions::ActionActionItem
            {
            public:
                ActionAbstractActionDrivenSearchCriterionConfigure(std::shared_ptr<AbstractActionDrivenSearchCriterion> cfg, QMutex *mutex);
                virtual void action(sh::actions::ActionExecutionInfo* info);
            private:
                std::shared_ptr<AbstractActionDrivenSearchCriterion> _cfg;
                QMutex* _mutex;
            };

        }
    }
}

#endif // ABSTRACTACTIONDRIVENSEARCHCRITERION_H
