/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef EXTENDEDATTRIBUTESEARCHCRITERION_H
#define EXTENDEDATTRIBUTESEARCHCRITERION_H

#include "search/searchcriterion.h"
#include "search/searchcriterionfactory.h"


namespace sh{
    namespace base{ class Eurl; class FilesystemNode; }

    namespace search{
        namespace criteria {

            class ExtendedAttributeSearchCriterion : public sh::search::SearchCriterion
            {
            public:

                enum class Mode
                {
                    None,
                    Simple,
                    RegExp,
                    Fuzzy
                };

                static const QStringList ModeCodes;

                Mode keymode = Mode::None;
                Mode valuemode = Mode::Simple;
                QString key, value;

                ExtendedAttributeSearchCriterion(std::shared_ptr<sh::search::SearchCriterionFactory> factory);

                virtual bool match(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl);
                virtual QString valuedescription();

                static QWidget* createEditor(std::shared_ptr<sh::search::SearchCriterion> cfg, sh::search::SearchCriterionFactory::AuxiliaryButtonGenerator makeAuxButton);
                static void editorUpdateConfiguration(QWidget* w, std::shared_ptr<sh::search::SearchCriterion> c);

                /** \brief
                 * Only called by the Shallot infrastructure for initialization.
                 */
                static void _initialize();
                /** \brief
                 * Only called by the Shallot infrastructure for shutdown.
                 */
                static void _shutdown();

            protected:
                static void _checkEditFieldStates(QWidget* w, ExtendedAttributeSearchCriterion *cfg);
                virtual void _deserialize(QString s);
                virtual QString _serialize();

                bool matchString(Mode mode, QString needle, QString haystack, QRegularExpression* re, QString* restring);

                QRegularExpression reKey;
                QString reKeyKey;

                QRegularExpression reValue;
                QString reValueValue;

            };

        }
    }
}

#endif // EXTENDEDATTRIBUTESEARCHCRITERION_H
