/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QLineEdit>
#include <QRegularExpression>
#include "base/eurl.h"
#include "base/operation.h"
#include "base/filesystemoperation.h"
#include "search/searchcriterionfactoryfromfunction.h"
#include "search/searchmanager.h"
#include "tools/singletoninitializer.h"
#include "filecontentsearchcriterion.h"


namespace sh{namespace search{namespace criteria{INITIALIZE_STATIC_IN_MAINTHREAD(100000, FileContentSearchCriterion)}}}

void sh::search::criteria::FileContentSearchCriterion::_initialize()
{
    REGISTER_CRITERION_FACTORY(sh::search::SearchManager::REGISTER_SEARCHCRITERION_INDEX_CORE+14000, "content", QObject::tr("File content"), sh::search::criteria::FileContentSearchCriterion)
}

void sh::search::criteria::FileContentSearchCriterion::_shutdown()
{
}

sh::search::criteria::FileContentSearchCriterion::FileContentSearchCriterion(std::shared_ptr<SearchCriterionFactory> factory)
                : sh::search::criteria::AbstractStringSearchCriterion(factory)
{
}

QWidget* sh::search::criteria::FileContentSearchCriterion::createEditor(std::shared_ptr<sh::search::SearchCriterion> cfg, SearchCriterionFactory::AuxiliaryButtonGenerator makeAuxButton)
{
    return AbstractStringSearchCriterion::_createEditor(cfg, makeAuxButton);
}

void sh::search::criteria::FileContentSearchCriterion::editorUpdateConfiguration(QWidget* w, std::shared_ptr<sh::search::SearchCriterion> c)
{
    AbstractStringSearchCriterion::_editorUpdateConfiguration(w, c);
}

QString sh::search::criteria::FileContentSearchCriterion::itemstring(std::shared_ptr<const sh::base::Eurl> eurl, base::Operation *op)
{
    std::shared_ptr<QIODevice> econtent = op->filesystem()->getFileContent(eurl);
    QByteArray ba = econtent->readAll();
    return QString::fromUtf8(ba);
}

bool sh::search::criteria::FileContentSearchCriterion::forceCaseInsensitive()
{
    return false;
}

