/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/eurl.h"
#include "search/searchcriterionfactoryfromfunction.h"
#include "search/searchmanager.h"
#include "tools/singletoninitializer.h"
#include "ui/linkbutton.h"
#include "filenamesearchcriterion.h"


namespace sh{namespace search{namespace criteria{INITIALIZE_STATIC_IN_MAINTHREAD(100000, FilenameSearchCriterion)}}}

void sh::search::criteria::FilenameSearchCriterion::_initialize()
{
    REGISTER_CRITERION_FACTORY(sh::search::SearchManager::REGISTER_SEARCHCRITERION_INDEX_CORE+5000, "name", QObject::tr("File name"), sh::search::criteria::FilenameSearchCriterion)
}

void sh::search::criteria::FilenameSearchCriterion::_shutdown()
{
}

sh::search::criteria::FilenameSearchCriterion::FilenameSearchCriterion(std::shared_ptr<SearchCriterionFactory> factory)
                : sh::search::criteria::AbstractStringSearchCriterion(factory)
{
}

QWidget* sh::search::criteria::FilenameSearchCriterion::createEditor(std::shared_ptr<sh::search::SearchCriterion> cfg, SearchCriterionFactory::AuxiliaryButtonGenerator makeAuxButton)
{
    return AbstractStringSearchCriterion::_createEditor(cfg, makeAuxButton);
}

void sh::search::criteria::FilenameSearchCriterion::editorUpdateConfiguration(QWidget* w, std::shared_ptr<sh::search::SearchCriterion> c)
{
    AbstractStringSearchCriterion::_editorUpdateConfiguration(w, c);
}

QString sh::search::criteria::FilenameSearchCriterion::itemstring(std::shared_ptr<const sh::base::Eurl> eurl, base::Operation*)
{
    if (mode == Mode::RegExp)
        return eurl->path();
    else
        return eurl->basename();
}

bool sh::search::criteria::FilenameSearchCriterion::forceCaseInsensitive()
{
    return true;
}
