/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MTIMESEARCHCRITERION_H
#define MTIMESEARCHCRITERION_H

#include <QDateTime>
#include <QDateTimeEdit>
#include "search/searchcriterion.h"
#include "search/searchcriterionfactory.h"


namespace sh{
    namespace base{ class Eurl; class FilesystemNode; }

    namespace search{
        namespace criteria {

            class MtimeSearchCriterion : public sh::search::SearchCriterion
            {
            public:

                QDateTime from, to;

                MtimeSearchCriterion(std::shared_ptr<sh::search::SearchCriterionFactory> factory);

                virtual bool match(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl);
                virtual QString valuedescription();

                static QWidget* createEditor(std::shared_ptr<sh::search::SearchCriterion> cfg, sh::search::SearchCriterionFactory::AuxiliaryButtonGenerator makeAuxButton);
                static void editorUpdateConfiguration(QWidget* w, std::shared_ptr<sh::search::SearchCriterion> c);

                /** \brief
                 * Only called by the Shallot infrastructure for initialization.
                 */
                static void _initialize();
                /** \brief
                 * Only called by the Shallot infrastructure for shutdown.
                 */
                static void _shutdown();

            protected:
                virtual void _deserialize(QString s);
                virtual QString _serialize();

            };

            /**
             * @brief Fixes a size computation bug in QDateTimeEdit
             */
            class MtimeSearchCriterion_QDateTimeEdit : public QDateTimeEdit
            {
                Q_OBJECT
            public:
                explicit MtimeSearchCriterion_QDateTimeEdit(QWidget* parent = 0);
            };

        }
    }
}

#endif // MTIMESEARCHCRITERION_H
