/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SEARCH_H
#define SEARCH_H

#include <memory>
#include <QMap>
#include <QMutex>
#include <QObject>
#include "base/filesystemnode.h"
#include "searchconfiguration.h"


namespace sh{
    namespace search{

        /** \brief
         * A search object represents one search request the user made.
         */
        class Search : public QObject, public std::enable_shared_from_this<Search>
        {
            Q_OBJECT
        public:
            virtual ~Search();
            std::shared_ptr<const sh::base::Eurl> eurl();
            std::shared_ptr<sh::base::FilesystemNode> searchnode();
            Search(std::shared_ptr<const base::Eurl> eurl, std::shared_ptr<sh::base::FilesystemNode> searchnode);

            void search();
            bool isSearching();
        signals:
            void isSearchingChanged();
        private:
            std::shared_ptr<const sh::base::Eurl> _eurl;
            std::shared_ptr<sh::base::FilesystemNode> _searchnode;
            sh::search::SearchConfiguration* _config;
            void search(std::shared_ptr<const base::Eurl> eurl, QString relpath);
            void insertFileItem(std::shared_ptr<const sh::base::Eurl> item, QString relpath, sh::base::FilesystemNodeType ftype);
            std::shared_ptr<sh::base::FilesystemNode> getDirItem(std::shared_ptr<const sh::base::Eurl> item);
            QMap<std::shared_ptr<const sh::base::Eurl>, std::shared_ptr<sh::base::FilesystemNode>> _dirs;
            QMutex _isSearchingMutex;
            bool _isSearching = false;
            bool _isSearchAgain = false;
        };

    }
}

#endif // SEARCH_H
