/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SEARCHCRITERION_H
#define SEARCHCRITERION_H

#include <memory>
#include <QObject>


namespace sh{
    namespace base { class Eurl; class Operation; }

    namespace search{
        class SearchCriterionFactory;

        class SearchCriterion
        {
        public:
            SearchCriterion(std::shared_ptr<sh::search::SearchCriterionFactory> factory);
            virtual ~SearchCriterion();
            QString serialize();
            static std::shared_ptr<sh::search::SearchCriterion> deserialize(QString s);
            virtual bool match(sh::base::Operation* op, std::shared_ptr<const sh::base::Eurl> eurl) =0;
            virtual QString valuedescription() =0;
            std::shared_ptr<sh::search::SearchCriterionFactory> factory();
        protected:
            static QString _serializeStringList(QStringList l);
            static QStringList _deserializeStringList(QString s);
            virtual void _deserialize(QString s) =0;
            virtual QString _serialize() =0;
            std::shared_ptr<sh::search::SearchCriterionFactory> _factory;
        };

    }
}

#endif // SEARCHCRITERION_H
