/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SEARCHCRITERIONFACTORY_H
#define SEARCHCRITERIONFACTORY_H

#include <memory>
#include <QMap>
#include <QWidget>


namespace sh{
    namespace ui { class LinkButton; }
    namespace base { class Eurl; class Operation; class FilesystemNode; }

    namespace search {
        class SearchCriterion;

        class SearchCriterionFactory : public std::enable_shared_from_this<SearchCriterionFactory>
        {
        public:

            typedef std::function<sh::ui::LinkButton*()> AuxiliaryButtonGenerator;

            SearchCriterionFactory();
            virtual QString key() =0;
            virtual QString description() =0;
            virtual QWidget* editor(std::shared_ptr<sh::search::SearchCriterion> c, AuxiliaryButtonGenerator makeAuxButton) =0;
            virtual void editorupdateconfig(QWidget* w, std::shared_ptr<sh::search::SearchCriterion> c) =0;
            virtual std::shared_ptr<sh::search::SearchCriterion> construct() =0;
            virtual bool isVisibleFor(sh::base::Operation* op, std::shared_ptr<sh::base::FilesystemNode> node);
            virtual ~SearchCriterionFactory();

        };

    }
}

#endif // SEARCHCRITERIONFACTORY_H
