/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SEARCHCRITERIONFACTORYFROMFUNCTION_H
#define SEARCHCRITERIONFACTORYFROMFUNCTION_H

#include <memory>
#include "searchcriterionfactory.h"


namespace sh{
    namespace base { class Eurl; class Operation; }

    namespace search {

        class SearchCriterionFactoryFromFunction : public sh::search::SearchCriterionFactory
        {
        public:
            SearchCriterionFactoryFromFunction(QString key,
                                               std::function<QString()> descriptionfct,
                                               std::function<QWidget*(std::shared_ptr<sh::search::SearchCriterion>, sh::search::SearchCriterionFactory::AuxiliaryButtonGenerator)> editorfct,
                                               std::function<void(QWidget*, std::shared_ptr<sh::search::SearchCriterion>)> editorupdateconfigfct,
                                               std::function<std::shared_ptr<sh::search::SearchCriterion>(std::shared_ptr<sh::search::SearchCriterionFactory>)> constructfct);
            virtual QString key();
            virtual QString description();
            virtual QWidget* editor(std::shared_ptr<sh::search::SearchCriterion> c, sh::search::SearchCriterionFactory::AuxiliaryButtonGenerator makeAuxButton);
            virtual void editorupdateconfig(QWidget* w, std::shared_ptr<sh::search::SearchCriterion> c);
            virtual std::shared_ptr<SearchCriterion> construct();
        private:
            QString _key;
            std::function<QString()> _descriptionfct;
            std::function<QWidget*(std::shared_ptr<sh::search::SearchCriterion>, sh::search::SearchCriterionFactory::AuxiliaryButtonGenerator)> _editorfct;
            std::function<void(QWidget*, std::shared_ptr<sh::search::SearchCriterion>)> _editorupdateconfigfct;
            std::function<std::shared_ptr<sh::search::SearchCriterion>(std::shared_ptr<sh::search::SearchCriterionFactory>)> _constructfct;
        };

    }
}

#define REGISTER_CRITERION_FACTORY(I,K,D,T) sh::search::SearchManager::instance()->addFactory((I), std::make_shared<sh::search::SearchCriterionFactoryFromFunction>((K), [](){return (D);}, (T::createEditor), (T::editorUpdateConfiguration), [](std::shared_ptr<sh::search::SearchCriterionFactory> factory){return std::make_shared<T>(factory);}));

#endif // SEARCHCRITERIONFACTORYFROMFUNCTION_H
