/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "tools/threadpool.h"
#include "ui/mainwindow.h"
#include "ui/fileview/fileview.h"
#include "ui/fileview/fileviewcontrol.h"
#include "ui/filesystempanel.h"
#include "numberoffilepanels.h"

REGISTER_SETTING(sh::settings::common::NumberOfFilePanels)

sh::settings::common::NumberOfFilePanels::NumberOfFilePanels()
{
}

QString sh::settings::common::NumberOfFilePanels::name()
{
    return "number-of-file-panels";
}

QString sh::settings::common::NumberOfFilePanels::description()
{
    return QObject::tr("Number of file panels");
}

sh::settings::SettingGroup sh::settings::common::NumberOfFilePanels::group()
{
    return sh::settings::SettingGroup::GLOBAL;
}

bool sh::settings::common::NumberOfFilePanels::isAdvancedSetting()
{
    return false;
}

void sh::settings::common::NumberOfFilePanels::setValue( QString value )
{
    IGNORE_FORBIDDEN_ASYNC_CALLS
    int count = value.toInt();
    while(sh::ui::MainWindow::mainWindow()->filesystemPanel()->viewsCount() > count)
        sh::ui::MainWindow::mainWindow()->filesystemPanel()->removeView(
                    sh::ui::MainWindow::mainWindow()->filesystemPanel()->viewsCount() - 1);
    while(sh::ui::MainWindow::mainWindow()->filesystemPanel()->viewsCount() < count)
        sh::ui::MainWindow::mainWindow()->filesystemPanel()->addView();
}

QString sh::settings::common::NumberOfFilePanels::getValue(sh::ui::fileview::FileViewControl * )
{
    return QString("%1").arg(sh::ui::MainWindow::mainWindow()->filesystemPanel()->viewsCount());
}

bool sh::settings::common::NumberOfFilePanels::isGlobal()
{
    return true;
}

bool sh::settings::common::NumberOfFilePanels::isPerFileview()
{
    return false;
}

QString sh::settings::common::NumberOfFilePanels::valueDescription(QString value)
{
    int count = value.toInt();
    return (count==1 ? QObject::tr("Show 1 panel") : QObject::tr("Show %1 panels").arg(count));
}
