/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "ui/mainwindow.h"
#include "ui/fileview/fileview.h"
#include "ui/fileview/fileviewcontrol.h"
#include "showhiddenfiles.h"


REGISTER_SETTING(sh::settings::common::ShowHiddenFiles)

sh::settings::common::ShowHiddenFiles::ShowHiddenFiles()
{
}

QString sh::settings::common::ShowHiddenFiles::name()
{
    return "show-hidden-files";
}

QString sh::settings::common::ShowHiddenFiles::description()
{
    return QObject::tr("Visibility of hidden files");
}

sh::settings::SettingGroup sh::settings::common::ShowHiddenFiles::group()
{
    return sh::settings::SettingGroup::DATAVIEW;
}

bool sh::settings::common::ShowHiddenFiles::isAdvancedSetting()
{
    return false;
}

void sh::settings::common::ShowHiddenFiles::setValue(sh::ui::fileview::FileViewControl *filelist, QString value )
{
    filelist->setHiddenFilesVisible(value=="1");
}

QString sh::settings::common::ShowHiddenFiles::getValue(sh::ui::fileview::FileViewControl *filelist )
{
    return (filelist->hiddenFilesVisible() ? "1" : "0");
}

bool sh::settings::common::ShowHiddenFiles::isGlobal()
{
    return false;
}

bool sh::settings::common::ShowHiddenFiles::isPerFileview()
{
    return true;
}

QString sh::settings::common::ShowHiddenFiles::valueDescription(QString value)
{
    return ((value=="1") ? QObject::tr("Show hidden files") : QObject::tr("Leave hidden files hidden") );
}
