/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/filesystemmodelfileviewproxy.h"
#include "ui/fileview/fileview.h"
#include "ui/fileview/fileviewcontrol.h"
#include "sizeformattingmode.h"


REGISTER_SETTING(sh::settings::common::SizeFormattingMode)

sh::settings::common::SizeFormattingMode::SizeFormattingMode()
{
}

QString sh::settings::common::SizeFormattingMode::name()
{
    return "size-formatting-mode";
}

QString sh::settings::common::SizeFormattingMode::description()
{
    return QObject::tr("File size formatting");
}

sh::settings::SettingGroup sh::settings::common::SizeFormattingMode::group()
{
    return sh::settings::SettingGroup::GUI;
}

bool sh::settings::common::SizeFormattingMode::isAdvancedSetting()
{
    return false;
}

QString sh::settings::common::SizeFormattingMode::valueDescription(QString value)
{
    if (value == "prefix")
        return QObject::tr("formatted with prefix");
    if (value == "bytes")
        return QObject::tr("plain number of bytes");
    return value;
}

void sh::settings::common::SizeFormattingMode::setValue(sh::ui::fileview::FileViewControl *filelist, QString value )
{
    sh::base::SizeFormatting mode = sh::base::SizeFormatting::SizeFormattingModePrefixes;
    if (value=="bytes")
        mode = sh::base::SizeFormatting::SizeFormattingModePlainBytes;
    filelist->setSizeFormattingMode(mode);
}

QString sh::settings::common::SizeFormattingMode::getValue(sh::ui::fileview::FileViewControl *filelist )
{
    sh::base::SizeFormatting sf = filelist->getSizeFormattingMode();
    if (sf == sh::base::SizeFormattingModePlainBytes)
        return "bytes";
    else
        return "prefix";
}

bool sh::settings::common::SizeFormattingMode::isGlobal()
{
    return false;
}

bool sh::settings::common::SizeFormattingMode::isPerFileview()
{
    return true;
}
