/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QByteArray>
#include <QDomElement>
#include "base/eurl.h"
#include "profilenode.h"
#include "setting.h"


sh::settings::ProfileNode::ProfileNode()
{
}

QString sh::settings::ProfileNode::nodeId()
{
    return _nodeId;
}

void sh::settings::ProfileNode::setNodeId(QString nodeId)
{
    _nodeId = nodeId;
}

void sh::settings::ProfileNode::setSetting(QString name, QString value, int onlyInFileview)
{
    _values[name] = value;
    _valuesOnlyInFileviewIndex[name] = onlyInFileview;
}

int sh::settings::ProfileNode::getSettingOnlyInFileviewIndex(QString name)
{
    auto itr = _valuesOnlyInFileviewIndex.find(name);
    if (itr != _valuesOnlyInFileviewIndex.end())
        return *itr;
    else
        return -1;
}

void sh::settings::ProfileNode::setEurl(std::shared_ptr<const sh::base::Eurl> eurl)
{
    _eurl = eurl;
}

QString sh::settings::ProfileNode::profilename()
{
    return _profilename;
}

void sh::settings::ProfileNode::setProfilename(QString profilename)
{
    _profilename = profilename;
}

QStringList sh::settings::ProfileNode::inheritsFrom()
{
    return _inheritsFrom;
}

void sh::settings::ProfileNode::setInheritsFrom(QStringList profilenames)
{
    _inheritsFrom = profilenames;
}

bool sh::settings::ProfileNode::withSubfolders()
{
    return _withSubfolders;
}

void sh::settings::ProfileNode::setWithSubfolders(bool v)
{
    _withSubfolders = v;
}

QList<QString> sh::settings::ProfileNode::getSettingNames()
{
    return _values.keys();
}

QString sh::settings::ProfileNode::getSettingValue(QString name)
{
    return _values[name];
}

std::shared_ptr<const sh::base::Eurl> sh::settings::ProfileNode::eurl()
{
    return _eurl;
}
