/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ABSTRACTACCOUNTSPROVIDER_H
#define ABSTRACTACCOUNTSPROVIDER_H

#include <memory>
#include <QList>


namespace sh{
    namespace tools{
        namespace accounts{
            class Account;

            /** \brief
             * Abstract base class for accounts provider.
             *
             * Implement this class (and register an instance of it) in order to add
             * support for something like a password/account manager.
             */
            class AbstractAccountsProvider
            {
            public:
                AbstractAccountsProvider();
                virtual ~AbstractAccountsProvider();

                /** \brief
                 * Finds account info by a given pattern.
                 */
                virtual void findAccounts(std::shared_ptr<Account> pattern, QList<std::shared_ptr<Account>>& result) =0;
                /** \brief
                 * Stores account infos.
                 */
                virtual bool storeAccount(std::shared_ptr<Account> account) =0;
            };

        }
    }
}

#endif // ABSTRACTACCOUNTSPROVIDER_H

