/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "tools/singletoninitializer.h"
#include "accountsmanager.h"


namespace sh{namespace tools{namespace accounts{INITIALIZE_SINGLETON_IN_MAINTHREAD(20000, AccountsManager)}}}

sh::tools::accounts::AccountsManager::AccountsManager()
{
}

sh::tools::accounts::AccountsManager* sh::tools::accounts::AccountsManager::instance()
{
    static sh::tools::accounts::AccountsManager* _instance = new sh::tools::accounts::AccountsManager();
    return _instance;
}

void sh::tools::accounts::AccountsManager::_initialize()
{
}

void sh::tools::accounts::AccountsManager::_shutdown()
{
}

void sh::tools::accounts::AccountsManager::registerProvider(std::shared_ptr<AbstractAccountsProvider> provider)
{
    QMutexLocker l1(&mutex);
    accountsproviders.append(provider);
}

QList<std::shared_ptr<sh::tools::accounts::Account>> sh::tools::accounts::AccountsManager::findAccounts(std::shared_ptr<Account> pattern)
{
    QMutexLocker l1(&mutex);
    QList<std::shared_ptr<sh::tools::accounts::Account>> result;
    foreach(std::shared_ptr<AbstractAccountsProvider> provider, accountsproviders)
        provider->findAccounts(pattern, result);
    return result;
}

void sh::tools::accounts::AccountsManager::storeAccount(std::shared_ptr<Account> account)
{
    QMutexLocker l1(&mutex);
    foreach(std::shared_ptr<AbstractAccountsProvider> provider, accountsproviders) {
        if (provider->storeAccount(account))
            break;
    }
}
