/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef FALLBACKACCOUNTSPROVIDER_H
#define FALLBACKACCOUNTSPROVIDER_H

#include <QObject>
#include "abstractaccountsprovider.h"


namespace sh{
    namespace tools{
        namespace accounts{
            class Account;

            /** \brief
             * A fallback accounts provider which at least stores user names (i.e. no passwords)
             * locally on disk.
             */
            class FallbackAccountsProvider : public AbstractAccountsProvider
            {
            public:
                FallbackAccountsProvider();
                void findAccounts(std::shared_ptr<Account> pattern, QList<std::shared_ptr<Account>>& result);
                bool storeAccount(std::shared_ptr<Account> account);
                /** \brief
                 * Only called by the Shallot infrastructure for initialization.
                 */
                static void _initialize();
                /** \brief
                 * Only called by the Shallot infrastructure for shutdown.
                 */
                static void _shutdown();
            private:
                QList<std::shared_ptr<Account> > _find_rem_account_helper(std::shared_ptr<Account> pattern, bool remove);
                QString accountsdir;
            };

        }
    }
}

#endif // FALLBACKACCOUNTSPROVIDER_H
