/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef LIBSECRETACCOUNTSPROVIDER_H
#define LIBSECRETACCOUNTSPROVIDER_H

#include <QObject>
#include "abstractaccountsprovider.h"


#ifdef LIBSECRET

namespace sh{
    namespace tools{
        namespace accounts{
            class Account;

            /** \brief
             * An accounts provider based on gnome-keyring.
             */
            class LibsecretAccountsProvider : public AbstractAccountsProvider
            {
            public:
                LibsecretAccountsProvider();
                void findAccounts(std::shared_ptr<Account> pattern, QList<std::shared_ptr<Account>>& result);
                bool storeAccount(std::shared_ptr<Account> account);
                /** \brief
                 * Only called by the Shallot infrastructure for initialization.
                 */
                static void _initialize();
                /** \brief
                 * Only called by the Shallot infrastructure for shutdown.
                 */
                static void _shutdown();
            private:
                QByteArray sauthtype = QString("authtype").toUtf8();
                QByteArray sdomain = QString("domain").toUtf8();
                QByteArray spath = QString("object").toUtf8();
                QByteArray sprotocol = QString("protocol").toUtf8();
                QByteArray sserver = QString("server").toUtf8();
                QByteArray sserverport = QString("port").toUtf8();
                QByteArray susername = QString("user").toUtf8();
            };

        }
    }
}

#endif

#endif // LIBSECRETACCOUNTSPROVIDER_H
