/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "benchmarking.h"
#include "logger.h"
#include "mainthread.h"
#include "singletoninitializer.h"
#include "threadpool.h"


namespace sh{namespace tools{INITIALIZE_SINGLETON_IN_MAINTHREAD(10000, Benchmarking)}}

sh::tools::Benchmarking::Benchmarking(QObject *parent) :
    QObject(parent)
{
}

void sh::tools::Benchmarking::_initialize()
{
#ifdef BENCHMARKING
    sh::tools::MainThread::dispatcher()->invokeAsync([&](){
        elapsedTimer.start();
        benchmarkReporter = new QTimer();
        connect(benchmarkReporter, &QTimer::timeout, this, [&](){
            LOG_DEBUG("\n" + report())
        });
        benchmarkReporter->start(60000);
    });
    addReporter([=](){
        QMap<QString, QString> _report;
        foreach(QString name, benchResults.keys())
            _report.insert(QString("%1 time").arg(name) , QString("%2 msec").arg(benchResults[name]/(1000*1000)));
        return _report;
    });
#endif
}

void sh::tools::Benchmarking::_shutdown()
{
#ifdef BENCHMARKING
    sh::tools::MainThread::dispatcher()->invokeAsync([&](){
        delete benchmarkReporter;
    });
#endif
}

#ifdef BENCHMARKING
QString sh::tools::Benchmarking::report()
{
    QMutexLocker l(&mutex);
    QStringList r;
    QMap<QString, QString> _report;
    for(auto reporter=reporters.begin(); reporter!=reporters.end(); reporter++)
    {
        QMap<QString, QString> lines = (*reporter)();
        foreach(QString name, lines.keys())
            _report.insert(name, lines[name]);
    }
    foreach(QString name, _report.keys())
        r << QString("%1: %2\n").arg(name).arg(_report[name]);
    return r.join("");
}

void sh::tools::Benchmarking::addReporter(std::function<QMap<QString,QString>()> reporter)
{
    reporters.append(reporter);
}
#endif
