/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef DATAEXCHANGE_H
#define DATAEXCHANGE_H

#include <memory>
#include <QObject>
#include <QMimeData>
#include <QMutex>


namespace sh{
    namespace base{ class Eurl; class FilesystemNode; }
    namespace actions{ class AbstractActionItem; }

    namespace tools{

        /** \brief
         * Clipboard and DnD related tools.
         */
        class DataExchange : public QObject
        {
            Q_OBJECT
        public:

            /** \brief A kind of data exchange movement.
             */
            enum class DataExchangeType{
                COPY,
                MOVE
            };

            /** \brief
             * Returns the instance of this singleton.
             */
            static DataExchange* instance();
            /** \brief
             * Only called by the Shallot infrastructure for initialization.
             */
            void _initialize();
            /** \brief
             * Only called by the Shallot infrastructure for shutdown.
             */
            void _shutdown();

            static QString FilelistTypeShallot;
            static QString FilelistTypeGnome;
            static QString FilelistTypeUrilist;
            static QString FilelistTypePlaintext;

            static QString linebreak;

            bool containsFileEntries(const QMimeData *);
            QMimeData *getMimeDataFromFilesystemNodes(QList<std::shared_ptr<sh::base::FilesystemNode>>, DataExchangeType type);

            std::shared_ptr<sh::actions::AbstractActionItem> createCopyAction(const QMimeData *src, std::shared_ptr<const sh::base::Eurl>dest);
            std::shared_ptr<sh::actions::AbstractActionItem> createMoveAction(const QMimeData *src, std::shared_ptr<const sh::base::Eurl>dest);

            std::shared_ptr<sh::actions::AbstractActionItem> createPasteAction(const QMimeData *src, std::shared_ptr<const sh::base::Eurl>dest);

            QList<std::shared_ptr<const sh::base::Eurl>> getSources(const QMimeData *src, bool *isCut = 0);

        private:
            explicit DataExchange();
        };

    }
}

#endif // DATAEXCHANGE_H
