/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QDir>
#include "tools/threadpool.h"
#include "freedesktoporgmimetypeinformationretrievalstrategy.h"


sh::tools::filetypes::FreeDesktopOrgMimetypeInformationRetrievalStrategy::FreeDesktopOrgMimetypeInformationRetrievalStrategy()
                : sh::tools::filetypes::FileTypeManager::MimetypeInformationRetrievalStrategy()
{
    sh::tools::ThreadPool::enqueueForce([=](){
        QQueue<QString> mimeLocations;
        mimeLocations << "/usr/share/mime";
        mimeLocations << QDir::homePath() + "/.local/share/mime";
        while(!mimeLocations.isEmpty())
        {
            QString mimeLocation = mimeLocations.dequeue();
            foreach(QString sentry, QDir(mimeLocation).entryList())
            {
                if (sentry != "." && sentry != "..")
                {
                    sh::exceptions::Exception::executeGuarded([&](){
                        QString msentry = mimeLocation + QDir::separator() + sentry;
                        if (QDir(msentry).exists()) {
                            foreach(QString ssentry, QDir(msentry).entryList())
                            {
                                if (ssentry.endsWith(".xml"))
                                {
                                    QString mssentry = msentry + QDir::separator() + ssentry;
                                    QString mimetype = sentry + "/" + ssentry.left(ssentry.length()-4);
                                    QXmlSimpleReader xmlReader;
                                    QFile fmssentry(mssentry);
                                    QXmlInputSource source(&fmssentry);
                                    MimetypeInfo mimeinfo;
                                    xmlReader.setContentHandler(&mimeinfo);
                                    xmlReader.setErrorHandler(&mimeinfo);
                                    xmlReader.parse(&source);
                                    mimeSubclassOf[mimetype] << mimeinfo.subClassOf;
                                }
                            }
                        }
                    }, sh::exceptions::ExecuteGuardFlag::IGNORE_ALL_RESUMEABLE_ERRORS_SILENTLY);
                }
            }
        }
    });
}

QStringList sh::tools::filetypes::FreeDesktopOrgMimetypeInformationRetrievalStrategy::getMimetypeSubclasses(QString mimetype)
{
    return mimeSubclassOf[mimetype];
}

bool sh::tools::filetypes::FreeDesktopOrgMimetypeInformationRetrievalStrategy::MimetypeInfo::startElement(const QString&, const QString& localName, const QString&, const QXmlAttributes &atts)
{
    if (localName == "sub-class-of")
        subClassOf.append(atts.value("type"));
    return true;
}

bool sh::tools::filetypes::FreeDesktopOrgMimetypeInformationRetrievalStrategy::MimetypeInfo::endElement(const QString&, const QString&, const QString&)
{
    return true;
}
