/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef FREEDESKTOPORGMIMETYPEINFORMATIONRETRIEVALSTRATEGY_H
#define FREEDESKTOPORGMIMETYPEINFORMATIONRETRIEVALSTRATEGY_H

#include <QXmlDefaultHandler>
#include <QXmlSimpleReader>
#include "filetypemanager.h"


namespace sh {
    namespace tools {
        namespace filetypes {

            /** \brief Tries to determine some mimetype information with the freedesktop.org specs.
             */
            class FreeDesktopOrgMimetypeInformationRetrievalStrategy : public sh::tools::filetypes::FileTypeManager::MimetypeInformationRetrievalStrategy
            {
            public:
                FreeDesktopOrgMimetypeInformationRetrievalStrategy();
                QStringList getMimetypeSubclasses(QString mimetype);
            private:
                QMap<QString, QStringList> mimeSubclassOf;

                class MimetypeInfo : public QXmlDefaultHandler
                {
                public:
                    MimetypeInfo() = default;
                    bool startElement(const QString &namespaceURI, const QString &localName, const QString &qName, const QXmlAttributes &atts);
                    bool endElement(const QString &namespaceURI, const QString &localName, const QString &qName);
                    QStringList subClassOf;
                };

            };

        }
    }
}

#endif // FREEDESKTOPORGMIMETYPEINFORMATIONRETRIEVALSTRATEGY_H
