/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QProcess>
#include <QRegularExpression>
#include "base/eurl.h"
#include "base/operation.h"
#include "base/filesystemhandlerregister.h"
#include "configuration/configurationmanager.h"
#include "filesystemhandlers/localfilesystemhandler.h"
#include "unixfiletoolmimetypedeterminationmethod.h"


std::shared_ptr<sh::configuration::ConfigurationValue> sh::tools::filetypes::UnixFileToolMimetypeDeterminationStrategy::cfgvalFilePath =
        sh::configuration::ConfigurationManager::instance()->registerConfigValue(
            "sh::tools::filetypes::UnixFileToolMimetypeDeterminationStrategy.FilePath", "/usr/bin/file",
            sh::configuration::ConfigurationValue::ValueTypeLocalFilePath(),
            QObject::tr("Path to the 'file' tool"),
            sh::configuration::ConfigurationCategory::CategoryExternalTools,
            QObject::tr("This tool is possibly used for file type detection."));

sh::tools::filetypes::UnixFileToolMimetypeDeterminationStrategy::UnixFileToolMimetypeDeterminationStrategy(sh::tools::filetypes::FileTypeManager* manager)
    :reMimetype( QRegularExpression("^[^ /]+/[^ /]+$") )
{
    cfgvalFilePath->consumeValue(manager, [=](){
        pathToFileTool = cfgvalFilePath->getConfigValueString();
    });
}

QMap<std::shared_ptr<const sh::base::Eurl>, QString> sh::tools::filetypes::UnixFileToolMimetypeDeterminationStrategy::determineMimetype(sh::base::Operation*, QList<std::shared_ptr<const sh::base::Eurl>> items)
{
    QMap<std::shared_ptr<const sh::base::Eurl>, QString> result;
    auto _reMimetype = reMimetype;  // thread-local copy needed
    if (!pathToFileTool.isEmpty())
    {
        foreach(std::shared_ptr<const sh::base::Eurl> eurl, items){
            sh::filesystemhandlers::LocalFilesystemHandler* ehandler = qobject_cast<sh::filesystemhandlers::LocalFilesystemHandler*>(sh::base::FilesystemHandlerRegister::instance()->findHandler(eurl->scheme()).get());
            if (ehandler)
            {
                QString localpath = ehandler->eurlToLocal(eurl);
                QProcess fileprocess;
                fileprocess.setProgram(pathToFileTool);
                QStringList args;
                args << "-b" << "--mime-type" << localpath;
                fileprocess.setArguments(args);
                fileprocess.start();
                if (fileprocess.waitForFinished())
                {
                    QByteArray ba = fileprocess.readAll();
                    QString t = QString::fromUtf8(ba).trimmed();
                    if (t != "inode/x-empty" && t != "application/octet-stream" && _reMimetype.match(t).hasMatch())
                        result[eurl] = t;
                }
            }
        }
    }
    return result;
}
