/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef USERDEFINEDOPENMETHODDETERMINATIONSTRATEGY_H
#define USERDEFINEDOPENMETHODDETERMINATIONSTRATEGY_H

#include "filetypemanager.h"


namespace sh {
    namespace base {
        class FilesystemNode;
    }

    namespace tools {
        namespace filetypes {

            /** \brief Tries to determine a 'open method' for a file by information the user stored before.
             */
            class UserDefinedOpenMethodDeterminationStrategy : public sh::tools::filetypes::FileTypeManager::OpenMethodDeterminationStrategy
            {
            public:
                /** \brief
                 * Returns the instance of this singleton.
                 */
                static std::shared_ptr<UserDefinedOpenMethodDeterminationStrategy> instance();
                /** \brief
                 * Only called by the Shallot infrastructure for initialization.
                 */
                void _initialize();
                /** \brief
                 * Only called by the Shallot infrastructure for shutdown.
                 */
                void _shutdown();
                UserDefinedOpenMethodDeterminationStrategy();
                virtual QList<std::shared_ptr<sh::tools::filetypes::OpenMethod>> getOpenMethods(QString mimetype, QList<std::shared_ptr<sh::base::FilesystemNode>> nodes);

                void storeCustomOpenMethod(QList<std::shared_ptr<sh::base::FilesystemNode>> nodes, QString mimetype, sh::tools::filetypes::OpenMethod* m, bool rememberForMimetype, std::shared_ptr<const sh::base::Eurl> rememberForDirectory, bool rememberForFile);
            };

        }
    }
}

#endif // USERDEFINEDOPENMETHODDETERMINATIONSTRATEGY_H
