/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/operation.h"
#include "base/eurl.h"
#include "ui/mainwindow.h"
#include "mainthread.h"
#include "operationscache.h"
#include "singletoninitializer.h"


namespace sh{namespace tools{INITIALIZE_SINGLETON_IN_MAINTHREAD(150000, OperationsCache)}}

sh::tools::OperationsCache::OperationsCache() : QObject()
{
}

sh::tools::OperationsCache* sh::tools::OperationsCache::instance()
{
    static sh::tools::OperationsCache* _instance = new sh::tools::OperationsCache();
    return _instance;
}

void sh::tools::OperationsCache::_initialize()
{
}

void sh::tools::OperationsCache::_shutdown()
{
}

std::shared_ptr<sh::base::Operation> sh::tools::OperationsCache::getOperationForContainer(std::shared_ptr<const sh::base::Eurl> container)
{
    {
        QMutexLocker _l1(&cachemutex);
        auto itr = cache.find(container);
        if (itr != cache.end())
            return *itr;
        else
        {
            std::shared_ptr<sh::base::Operation> result(new sh::base::Operation());
            result->setMaxAllowedSizeRatioPerPart(0.0001);
            cache.insert(container, result);
            sh::tools::MainThread::dispatcher()->invokeAsync([=](){
                QTimer* timer = new QTimer(sh::ui::MainWindow::mainWindow());
                QObject::connect(timer, &QTimer::timeout, this, [=](){
                    QMutexLocker _l1(&cachemutex);
                    cache.remove(container);
                    timer->deleteLater();
                });
                timer->setSingleShot(true);
                timer->start(10000);
            });
            return result;
        }
    }
}

