/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <cmath>
#include <cstdlib>
#include "logger.h"
#include "mainthread.h"
#include "threadpool.h"


namespace sh{namespace tools{INITIALIZE_STATIC_IN_MAINTHREAD(1000, ThreadPool)}}

void sh::tools::ThreadPool::_initialize()
{
    for(int i=0; i<THREAD_COUNT; i++)
    {
        ThreadPoolThread* t = new ThreadPoolThread();
        t->start();
        _threads.append(t);
    }
}

sh::tools::ThreadPoolThread::ThreadPoolThread()
{
}

void sh::tools::ThreadPoolThread::run()
{
    ThreadPool::poolmutex.lock();
    ThreadPool::_threadsalive++;
    while (!ThreadPool::_shutsdown)
    {
        ThreadPool::pooladdedcondition.wait(&ThreadPool::poolmutex);
        if (ThreadPool::_shutsdown)
            break;
        bool foundstuff = true;
        while (foundstuff) {
            if (ThreadPool::tasks.isEmpty())
                foundstuff = false;
            else
            {
                std::function<void ()> f = ThreadPool::tasks.takeFirst();
                ThreadPool::poolmutex.unlock();
                try
                {
                    sh::exceptions::Exception::executeGuarded([&](){
                        f();
                    }, sh::exceptions::ExecuteGuardFlag::ALL_ERRORS_KILL_SHALLOT);
                }
                catch(sh::exceptions::ThreadAbortException&)
                {
                }
                ThreadPool::poolmutex.lock();
            }
        }
    }
    ThreadPool::_threadsalive--;
    ThreadPool::poolmutex.unlock();
}

void sh::tools::ThreadPool::_shutdown()
{
    {
        QMutexLocker l1(&poolmutex);
        _shutsdown = true;
        pooladdedcondition.wakeAll();
    }
    while(!isShutdown())
        QThread::msleep(10);
    foreach(ThreadPoolThread* t, _threads)
        delete t;
}

QMutex sh::tools::ThreadPool::poolmutex;

QWaitCondition sh::tools::ThreadPool::pooladdedcondition;

QList<std::function<void ()>> sh::tools::ThreadPool::tasks;

bool sh::tools::ThreadPool::_shutsdown = false;

int sh::tools::ThreadPool::_threadsalive = 0;

QList<sh::tools::ThreadPoolThread*> sh::tools::ThreadPool::_threads;

#ifdef QT_DEBUG
QList<QThread*> sh::tools::AsyncForbid::restrictedThreads;
QMutex sh::tools::AsyncForbid::mutex;
QList<QThread*> sh::tools::AsyncForbidIgnore::restrictionIgnoredThreads;
QMutex sh::tools::AsyncForbidIgnore::mutex;
#endif
