/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "base/filesystemnode.h"
#include "base/operation.h"
#include "tools/singletoninitializer.h"
#include "defaultimagethumbnailprovider.h"


namespace sh{namespace tools{namespace thumbnailproviders{INITIALIZE_STATIC_IN_MAINTHREAD(130100, DefaultImageThumbnailProvider)}}}

void sh::tools::thumbnailproviders::DefaultImageThumbnailProvider::_initialize()
{
    ThumbnailManager::instance()->addThumbnailProvider(ThumbnailManager::REGISTER_THUMBNAILPROVIDER_INDEX_CORE+1000, std::make_shared<DefaultImageThumbnailProvider>());
}

void sh::tools::thumbnailproviders::DefaultImageThumbnailProvider::_shutdown()
{
}

void sh::tools::thumbnailproviders::DefaultImageThumbnailProvider::getThumbnail(sh::base::Operation *operation, std::shared_ptr<sh::base::FilesystemNode> node, QString contentType, int width, int height, QIcon* icon)
{
    if (contentType.startsWith("image/"))
    {
        QString imgfile;
        sh::exceptions::Exception::executeGuarded([&](){
            imgfile = operation->fetchFile(node->eurl());
        }, sh::exceptions::ExecuteGuardFlag::AUTOMATIC_RETRY_ENABLED|sh::exceptions::ExecuteGuardFlag::IGNORE_ALL_RESUMEABLE_ERRORS_SILENTLY);
        if (!imgfile.isNull())
        {
            QImage img(imgfile);
            *icon = QIcon(QPixmap::fromImage(img.scaled(width, height, Qt::KeepAspectRatio, Qt::SmoothTransformation)));
        }
    }
}
