/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef FFMPEGVIDEOTHUMBNAILPROVIDER_H
#define FFMPEGVIDEOTHUMBNAILPROVIDER_H

#include <QRegularExpression>
#include "tools/thumbnailmanager.h"


namespace sh{
    namespace configuration {class ConfigurationValue;}

    namespace tools{
        namespace thumbnailproviders {

            /** \brief
             * Thumbnail provider for videos using the ffmpeg tool.
             */
            class FfmpegVideoThumbnailProvider : public QObject, public ThumbnailProvider
            {/* QObject needed for working with ConfigurationValue */
                Q_OBJECT
            public:
                /** \brief
                 * Only called by the Shallot infrastructure for initialization.
                 */
                static void _initialize();
                /** \brief
                 * Only called by the Shallot infrastructure for shutdown.
                 */
                static void _shutdown();

                FfmpegVideoThumbnailProvider();
                virtual void getThumbnail(sh::base::Operation* operation, std::shared_ptr<sh::base::FilesystemNode> node, QString contentType, int width, int height, QIcon* icon);
            private:
                static std::shared_ptr<sh::configuration::ConfigurationValue> cfgvalFfmpegPath;
                QString pathToFfmpegTool;
                QString pathToFfprobeTool;
                const QRegularExpression reDuration;
                QMutex mutexReDuration;
            };

        }
    }
}

#endif // FFMPEGVIDEOTHUMBNAILPROVIDER_H
