/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QDateTime>
#include <QDesktopServices>
#include <QUrl>
#include "configuration/configurationmanager.h"
#include "tools/iconmanager.h"
#include "aboutdialog.h"
#include "ui_aboutdialog.h"


sh::ui::AboutDialog::AboutDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::AboutDialog)
{
    ui->setupUi(this);

    ui->btnClose->setSizeByFactor(120);
    ui->btnClose->setIcon(sh::tools::IconManager::instance()->getIcon("dialog-close"));

    sh::configuration::ConfigurationManager::instance()->style()->createSheet()
                .background_liketitlebar()
            .applyTo(ui->panelHead);
    sh::configuration::ConfigurationManager::instance()->style()->createSheet()
                .fontsize_byFactor(140)
            .applyTo(ui->label);
    sh::configuration::ConfigurationManager::instance()->style()->createSheet()
                .fontsize_byFactor(120)
            .applyTo(ui->labelVersion);
    sh::configuration::ConfigurationManager::instance()->style()->createSheet()
            .applyTo(ui->labelBuildtime);

    ui->labelVersion->setText(sh::configuration::ConfigurationManager::instance()->revisionString());
    ui->labelBuildtime->setText(tr("built %1").arg(
                    sh::configuration::ConfigurationManager::instance()->buildtime().toString(Qt::DefaultLocaleShortDate)
                ));

}

sh::ui::AboutDialog::~AboutDialog()
{
    delete ui;
}

void sh::ui::AboutDialog::on_btnClose_clicked()
{
    close();
}

void sh::ui::AboutDialog::on_btnLicense_clicked()
{
    QDesktopServices::openUrl(QUrl("https://gnu.org/licenses/gpl.html"));
}

void sh::ui::AboutDialog::on_btnHomepage_clicked()
{
    QDesktopServices::openUrl(QUrl(sh::configuration::ConfigurationManager::instance()->homepage()));
}
