/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ACTIONEXECUTIONINFODIALOG_H
#define ACTIONEXECUTIONINFODIALOG_H

#include <functional>
#include <QDialog>
#include <QKeyEvent>
#include <QStack>
#include "actions/actionexecutionuserfeedback.h"


namespace Ui {
class ActionExecutionInfoDialog;
}

namespace sh {
    namespace actions {class ActionActionItem;class ActionExecutionInfo;}
    namespace ui {
        namespace feedbackpanels{class FeedbackPanel;}

        /** \brief
         * Progress dialog for action executions.
         */
        class ActionExecutionInfoDialog : public QDialog, public sh::actions::ActionExecutionUserFeedback
        {
            Q_OBJECT

        public:

            ActionExecutionInfoDialog(sh::actions::ActionExecutionInfo *info, QWidget *parent = 0);
            ~ActionExecutionInfoDialog();

            void setDetails(QString fv, QString fob, QString tv, QString tob);
            void setHead(QString txt);
            void setProgress(bool isDeterminate, quint64 done, quint64 all, QString text);

            virtual int messageBox(QString text, QList<QString> answers, QString icon=QString(), int defaultanswer=-1, int cancelanswer=-1, QList<QString> answericons=QList<QString>());
            virtual int inputBox(QString text, QList<QString> answers, QString *value, QString icon=QString(), int defaultanswer=-1, int cancelanswer=-1);
            virtual int multilineInputBox(QString text, QList<QString> answers, QString *value, QString icon=QString(), int defaultanswer=-1, int cancelanswer=-1);
            virtual int gridform(QString text, QList<QPair<QString, QWidget *> > form, QStringList answers, int defaultanswer=-1, int cancelanswer=-1);
            virtual int simpleChooserGridform(QString text, GridformEntries *entries, QStringList answers, int defaultanswer=-1, int cancelanswer=-1);

            bool isLogicallyVisible();
            void setLogicallyVisible(bool v);

            bool isBackground();
            void setBackground(bool v);

            void setForceForeground(bool v);

            virtual bool credentialsDialog(QString text, bool showDomain, bool showUsername, bool showPassword, bool showAnonymous, bool showRemember, QString* domain, QString* username, QString* password, bool* isAnonymous, bool* isRemember);

            void askFeedbackViaWidget(sh::ui::feedbackpanels::FeedbackPanel* widget);

        protected:
            void keyPressEvent(QKeyEvent *e);
            void reject();

        private slots:
            void on_btnCancel_clicked();
            void on_btnBackground_clicked();

        private:
            sh::actions::ActionExecutionInfo* info;
            Ui::ActionExecutionInfoDialog *ui;
            bool _logicallyVisible = false;
            bool _isbackground = false;
            bool _isforeground_forced = false;
            sh::ui::feedbackpanels::FeedbackPanel* currentFeedbackPanel = 0;
            void _computevisibility();
        };

    }
}

#endif // ACTIONEXECUTIONINFODIALOG_H
