/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QFile>
#include "configuration/configurationmanager.h"
#include "columndimensions.h"


sh::ui::ColumnDimensions::ColumnDimensions(int index) :
    index(index)
{
}

QSettings* sh::ui::ColumnDimensions::db()
{
    sh::configuration::ConfigurationManager::ConfigFilesLock sfl; //this locks at least the initial reading
    return new QSettings(sh::configuration::ConfigurationManager::instance()->mypath()+QString("columns%1").arg(index), QSettings::IniFormat);
}

void sh::ui::ColumnDimensions::store()
{
    QSettings* settings = db();
    foreach(QString col, dims.keys())
        settings->setValue(col, dims[col]);
    delete settings;
}

bool sh::ui::ColumnDimensions::remove()
{
    sh::configuration::ConfigurationManager::ConfigFilesLock sfl;
    return QFile(sh::configuration::ConfigurationManager::instance()->mypath()+QString("columns%1").arg(index)).remove();
}

void sh::ui::ColumnDimensions::setWidth(QString col, int width)
{
    QSettings* settings = db();
    dims[col] = width;
    settings->setValue(col, width);
    delete settings;
}

int sh::ui::ColumnDimensions::getWidth(QString col, int defaultval)
{
    auto it1 = dims.constFind(col);
    if (it1 != dims.constEnd())
        return (*it1);
    QSettings* settings = db();
    int v = settings->value(col, defaultval).toInt();
    dims[col] = v;
    delete settings;
    return v;
}
