/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef COLUMNDIMENSIONS_H
#define COLUMNDIMENSIONS_H

#include <QMap>
#include <QObject>
#include <QSettings>


namespace sh{
    namespace ui {

        /** \brief
         * Auxiliary data structure for persistent column dimensions.
         *
         * It stores the width of a column whenever it changes and
         * restores it when needed (mostly when Shallot starts).
         */
        class ColumnDimensions
        {
        public:
            /** \brief Constructed only by the infrastructure and made available otherwise.
             */
            ColumnDimensions(int index);
            void store();
            bool remove();
            void setWidth(QString col, int width);
            int getWidth(QString col, int defaultval);
            int index;

        private:
            QSettings *db();
            QMap<QString, int> dims;

        };

    }
}

#endif // COLUMNDIMENSIONS_H
