/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef EXCEPTIONDIALOG_H
#define EXCEPTIONDIALOG_H

#include <functional>
#include <QDialog>
#include <QMap>
#include <QVariant>
#include <QWaitCondition>
#include "exceptions/exception.h"


namespace Ui {
class ExceptionDialog;
}

namespace sh{
    namespace ui {

        /** \brief
         * The exception dialog.
         */
        class ExceptionDialog : public QDialog
        {
            Q_OBJECT

        public:
            static int exceptionDialog(sh::exceptions::Exception& e);

        private slots:
            void on_btnExit_clicked();
            void on_btnClose_clicked();
            void on_btnCancel_clicked();
            void on_btnRetry_clicked();
            void on_btnShowLog_clicked();
            void on_btnExitAndSaveLog_clicked();
            void on_btnShowDetails_toggled(bool checked);

        private:
            explicit ExceptionDialog(bool mayClose, QWidget *parent = 0);
            ~ExceptionDialog();
            static int exceptionDialog(QString error1, QString error2, QString details, QString icon, bool mayRetry, bool mayClose, bool mayCancel, bool showLoglabelAndDetails);
            bool mayClose;
            bool cancelled;
            Ui::ExceptionDialog *ui;

            static bool _inShutdown;
            static QMutex _inShutdownMutex;

        };

    }
}

#endif // EXCEPTIONDIALOG_H
