/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "tools/mainthread.h"
#include "feedbackpanel.h"


sh::ui::feedbackpanels::FeedbackPanel::FeedbackPanel(QWidget *parent) :
    QWidget(parent)
{
}

bool sh::ui::feedbackpanels::FeedbackPanel::isClosed()
{
    QMutexLocker _l(&_mutex);
    return _closed;
}

void sh::ui::feedbackpanels::FeedbackPanel::waitUntilClosed()
{
    if (QThread::currentThread()==QApplication::instance()->thread())
    {
        QEventLoop loop;
        QObject::connect(this, &sh::ui::feedbackpanels::FeedbackPanel::wasClosed, &loop, [&](){
            loop.exit();
        });
        loop.exec();
    }
    else
    {
        QMutexLocker _l(&_mutex);
        while (!_closed)
            _closedcondition.wait(&_mutex, 5000);
    }
}

void sh::ui::feedbackpanels::FeedbackPanel::setClosed()
{
    QMutexLocker _l(&_mutex);
    _closed = true;
    emit wasClosed();
    _closedcondition.wakeAll();
}

int sh::ui::feedbackpanels::FeedbackPanel::preferredHeight()
{
    return -1;
}
