/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QKeyEvent>
#include <QStackedLayout>
#include <QSpacerItem>
#include "gridform.h"
#include "ui_gridform.h"


sh::ui::feedbackpanels::GridForm::GridForm(QString text, QList<sh::ui::feedbackpanels::GridFormRow> form, QStringList answers, int defaultanswer, int cancelanswer, QWidget *parent) :
    sh::ui::feedbackpanels::FeedbackPanel(parent),
    ui(new Ui::GridForm)
{
    ui->setupUi(this);
    int irow = 0;
    ui->label->setText(text);
    foreach(sh::ui::feedbackpanels::GridFormRow row, form)
    {
        QLabel* lbl = new QLabel(this);
        lbl->setWordWrap(true);
        lbl->setText(row.first);
        QStackedLayout* l1 = new QStackedLayout();
        QStackedLayout* l2 = new QStackedLayout();
        l1->addWidget(lbl);
        l2->addWidget(row.second);
        ui->gridLayout->addLayout(l1, irow, 0);
        ui->gridLayout->addLayout(l2, irow, 1);
        irow++;
    }
    QSpacerItem *space = new QSpacerItem(1,1, QSizePolicy::Preferred, QSizePolicy::Expanding);
    ui->gridLayout->addItem(space, irow, 0);
    int i = 0;
    foreach(QString s, answers)
    {
        QPushButton* btn = new QPushButton(this);
        if (i==defaultanswer)
            _defaultBtn = btn;
        if (i==cancelanswer)
            _cancelBtn = btn;
        btn2index[btn] = i;
        btn->setText(s);
        connect(btn, SIGNAL(clicked()), this, SLOT(slot_btnclicked()));
        i++;
        ui->verticalLayout->addWidget(btn);
    }
}

sh::ui::feedbackpanels::GridForm::~GridForm()
{
    delete ui;
}

void sh::ui::feedbackpanels::GridForm::cancelRequested()
{
    if (_cancelBtn)
        _cancelBtn->click();
}

void sh::ui::feedbackpanels::GridForm::showEvent(QShowEvent* event)
{
    if (event->spontaneous())
        return;
    if (_defaultBtn)
        _defaultBtn->setFocus();
}

int sh::ui::feedbackpanels::GridForm::preferredHeight()
{
    int _width=width();
    return heightForWidth(_width)-ui->scrollArea->heightForWidth(_width)+ui->scrollAreaWidgetContents->heightForWidth(_width);
}

void sh::ui::feedbackpanels::GridForm::slot_btnclicked()
{
    QPushButton* me = dynamic_cast<QPushButton*>(sender());
    answer = btn2index[me];
    setClosed();
}
