/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef FILEPROPERTYDIALOGTABACTIONSVIEW_H
#define FILEPROPERTYDIALOGTABACTIONSVIEW_H

#include <functional>
#include <memory>
#include <QVBoxLayout>
#include <QWidget>


namespace sh {
    namespace ui {
        namespace filepropertydialog {

            /** @brief A tab view which shows a main widget in the main part and some buttons below.
             */
            class FilePropertyDialogTabActionsView : public QWidget
            {
                Q_OBJECT
            public:
                /** \brief Is intended to be directly constructed from everywhere.
                 */
                explicit FilePropertyDialogTabActionsView(QWidget* parent = 0);
                /** @brief The main widget.
                 */
                QWidget* content();
                /** @brief Sets the main widget.
                 */
                void setContent(QWidget* w);
                /** @brief The list of buttons.
                 */
                QList<QString> buttons();
                /** @brief Sets the list of buttons.
                 */
                void setButtons(QList<QString> buttons);

            signals:
                /** @brief Triggered when the user clicks on a button.
                 */
                void buttonTriggered(int i);

            protected:
                QVBoxLayout* layout;
                QVBoxLayout* inlayout;
                QVBoxLayout* btnslayout;
                QList<QString> _buttons;

            };

        }
    }
}

#endif // FILEPROPERTYDIALOGTABACTIONSVIEW_H
