/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QLabel>
#include "actions/common/actionabstracttransfertree.h"
#include "base/filesystemnode.h"
#include "base/filesystemoperation.h"
#include "base/eurl.h"
#include "base/operation.h"
#include "detailcolumns/detailcolumnresolvedsymlink.h"
#include "detailcolumns/detailcolumnfilesize.h"
#include "detailcolumns/detailcolumnmimetype.h"
#include "detailcolumns/detailcolumnmtime.h"
#include "tools/singletoninitializer.h"
#include "tools/threadpool.h"
#include "tools/mainthread.h"
#include "filepropertydialogtabgeneral.h"


namespace sh{namespace ui{namespace filepropertydialog{INITIALIZE_STATIC_IN_MAINTHREAD(56000, FilePropertyDialogTabGeneral)}}}

sh::ui::filepropertydialog::FilePropertyDialogTabGeneral::FilePropertyDialogTabGeneral()
{
}

QString sh::ui::filepropertydialog::FilePropertyDialogTabGeneral::title()
{
    return QObject::tr("&General");
}

QList<QString> sh::ui::filepropertydialog::FilePropertyDialogTabGeneral::properties()
{
    QStringList result;
    result << QObject::tr("Location");
    result << QObject::tr("Resolved location");
    result << QObject::tr("Type");
    result << QObject::tr("Size");
    result << QObject::tr("Last modified");
    return result;
}

QWidget* sh::ui::filepropertydialog::FilePropertyDialogTabGeneral::createWidget(int)
{
    QLabel* lbl = new QLabel();
    lbl->setWordWrap(true);
    return lbl;
}

void sh::ui::filepropertydialog::FilePropertyDialogTabGeneral::updateWidget(int i, QWidget* widget, sh::base::Operation* op)
{
    QString result;
    bool visible = true;
    if (nodes().length() == 1)
        switch(i)
        {
        case 0: // Location
        {
            result = nodes().first()->eurl()->asString();
            break;
        }
        case 1: // Resolved location
        {
            QList<std::shared_ptr<sh::base::FilesystemNode>> resnodes = op->filesystem()->resolveNodeLink(nodes().first(), true, true);
            if (resnodes.isEmpty())
                visible = false;
            else
                result = resnodes.first()->eurl()->asString();
            break;
        }
        case 2: // Type
        {
            result = sh::detailcolumns::DetailColumnMimetype::instance()->requestValue(nodes().first(), op).toString();
            break;
        }
        case 3: // Size
        {
            qint64 size = (qint64) sh::detailcolumns::DetailColumnFilesize::instance()->requestValue(nodes().first(), op).toLongLong();
            double dsize = size;
            QString unit;
            sh::actions::common::ActionAbstractTransferTree::makereadableunits(&dsize, &unit);
            result = QObject::tr("%1 %2").arg(dsize, 0, 'f', 2).arg(unit);
            if (dsize != size)
                result = QObject::tr("%1 (%2 byte)").arg(result).arg(size);
            break;
        }
        case 4: // Last modified
        {
            result = sh::detailcolumns::DetailColumnMtime::instance()->requestValue(nodes().first(), op).toDateTime().toString(Qt::DefaultLocaleLongDate);
            break;
        }
        }
    sh::tools::MainThread::dispatcher()->invokeSync([&](){
        ((QLabel*)widget)->setText(result);
        widget->setVisible(visible);
    });
}

void sh::ui::filepropertydialog::FilePropertyDialogTabGeneral::_initialize()
{
    REGISTER_FILEPROPERTYDIALOGTAB(sh::ui::filepropertydialog::FilePropertyDialogTabFactory::REGISTER_FILEPROPERTYDIALOGTAB_INDEX_CORE, sh::ui::filepropertydialog::FilePropertyDialogTabGeneral);
}

void sh::ui::filepropertydialog::FilePropertyDialogTabGeneral::_shutdown()
{
}
