/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QHeaderView>
#include <QStandardItemModel>
#include "filepropertydialogtabtableview.h"


sh::ui::filepropertydialog::FilePropertyDialogTabTableView::FilePropertyDialogTabTableView(QWidget* parent)
            : QTableView(parent)
{
    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Fixed);
    horizontalHeader()->setVisible(false);
    verticalHeader()->setSectionResizeMode(QHeaderView::Fixed);
    this->installEventFilter(this);
}

void sh::ui::filepropertydialog::FilePropertyDialogTabTableView::setContent(QList<QPair<QString, QString>> content)
{
    this->content = content;
    QStandardItemModel* imodel = new QStandardItemModel(content.count(), 1, this);
    int i=0;
    foreach(StringPair line, content)
    {
        imodel->setHeaderData(i, Qt::Vertical, line.first);
        QStandardItem* itm = new QStandardItem(line.second);
        itm->setFlags(itm->flags() & ~Qt::ItemIsEditable);
        imodel->setItem(i, 0, itm);
        i++;
    }
    setModel(imodel);
    resizeRowsToContents();
    updateGeometry();
}

bool sh::ui::filepropertydialog::FilePropertyDialogTabTableView::eventFilter(QObject *obj, QEvent *event)
{
    if (event->type()==QEvent::Wheel)
        event->ignore();
    return QTableView::eventFilter(obj, event);
}

QSize sh::ui::filepropertydialog::FilePropertyDialogTabTableView::sizeHint() const
{
    return QSize(0, rowViewportPosition(content.count()-1) + rowHeight(content.count()-1) + 2*frameWidth());
}

QSize sh::ui::filepropertydialog::FilePropertyDialogTabTableView::minimumSizeHint() const
{
    return QSize(20,20);
}

void sh::ui::filepropertydialog::FilePropertyDialogTabTableView::resizeEvent(QResizeEvent* event)
{
    if (event->oldSize().width() != event->size().width())
    {
        horizontalHeader()->resizeSection(0, this->width()-verticalHeader()->width()-2*frameWidth());
        resizeRowsToContents();
        updateGeometry();
    }
}
