/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef FILEPROPERTYDIALOGTABTABLEVIEW_H
#define FILEPROPERTYDIALOGTABTABLEVIEW_H

#include <functional>
#include <memory>
#include <QMap>
#include <QResizeEvent>
#include <QTableWidget>


namespace sh {
    namespace ui {
        namespace filepropertydialog {

            /** @brief A special QTableView tuned for showing key/value pairs.
             */
            class FilePropertyDialogTabTableView : public QTableView
            {
                Q_OBJECT
            public:
                /** \brief Is intended to be directly constructed from everywhere.
                 */
                explicit FilePropertyDialogTabTableView(QWidget* parent = 0);
                /** @brief Sets the content.
                 */
                void setContent(QList<QPair<QString, QString>> content);
            protected:
                typedef QPair<QString, QString> StringPair;
                virtual bool eventFilter(QObject *obj, QEvent *event);
                virtual QSize sizeHint() const;
                virtual QSize minimumSizeHint() const;
                virtual void resizeEvent(QResizeEvent* event);
                QList<QPair<QString, QString>> content;
            };

        }
    }
}

#endif // FILEPROPERTYDIALOGTABTABLEVIEW_H
