/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef FILEPROPERTYDIALOGTABUNIXPERMISSIONS_H
#define FILEPROPERTYDIALOGTABUNIXPERMISSIONS_H

#include "actions/actionactionitem.h"
#include "filepropertydialog.h"


namespace sh{
    namespace ui {
        namespace filepropertydialog {

            /** @brief Properties dialog tab for unix permissions.
             */
            class FilePropertyDialogTabUnixPermissions : public FilePropertyDialogTab
            {
            public:
                FilePropertyDialogTabUnixPermissions();
                virtual QString title();
                virtual QList<QString> properties();
                virtual QWidget* createWidget(int i);
                virtual void updateWidget(int i, QWidget *widget, sh::base::Operation *op);

                static QString getUserName(int uid);
                static QString getGroupName(int gid);

                static QMap<int, QString> getAllUsers();
                static QMap<int, QString> getAllGroups();

                /** \brief
                 * Only called by the Shallot infrastructure for initialization.
                 */
                static void _initialize();
                /** \brief
                 * Only called by the Shallot infrastructure for shutdown.
                 */
                static void _shutdown();

            private:
                void slot_buttontriggered(int i);
                QString userPermissionsDescription(int flags, int rflag, int wflag, int xflag);

                class ActionChange : public sh::actions::ActionActionItem
                {
                public:
                    ActionChange(std::shared_ptr<sh::ui::filepropertydialog::FilePropertyDialogTabUnixPermissions> tab);
                protected:
                    virtual void action(sh::actions::ActionExecutionInfo* info);
                    std::shared_ptr<sh::ui::filepropertydialog::FilePropertyDialogTabUnixPermissions> tab;
                };

            };

        }
    }
}

#endif // FILEPROPERTYDIALOGTABUNIXPERMISSIONS_H
