/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef FILESYSTEMPANEL_H
#define FILESYSTEMPANEL_H

#include <memory>
#include <QTreeView>
#include "settings/setting.h"
#include "tools/historytracker.h"
#include "columndimensions.h"


namespace Ui {
class FilesystemPanel;
}

namespace sh{
    namespace base{class FilesystemNode;}
    namespace settings{class SettingsRegistration;}

    namespace ui {

        namespace fileview {class FileView;class FileViewControl;enum class FileViewMode;}

        class MainWindow;

        /** \brief
         * A splitted horizontal panel of file views.
         */
        class FilesystemPanel : public QWidget
        {
            friend class MainWindow;
            friend class ui::fileview::FileViewControl;
            Q_OBJECT

        public:
            explicit FilesystemPanel(QWidget *parent = 0);
            ~FilesystemPanel();
            void addView(bool reinitialize=false);
            void setViewMode(int i, sh::ui::fileview::FileViewMode mode);
            sh::ui::fileview::FileViewMode viewMode(int i);
            void removeView(int i);
            sh::ui::fileview::FileViewControl *currentView();
            int currentViewIndex();
            int viewsCount();
            sh::ui::fileview::FileViewControl* view(int i);
            void reloadAll(bool requestFetch);
            void selectFolder(std::shared_ptr<sh::base::FilesystemNode> folder, int index = -1);
            std::shared_ptr<sh::ui::fileview::FileView> viewwidget(int i);
            QList<std::shared_ptr<sh::base::FilesystemNode>> selectedNodes();

        signals:
            void folderActivated(std::shared_ptr<sh::base::FilesystemNode> folder, bool realSwitch);
            void currentViewChanged();
            void viewmodeChanged(int i);
            void viewOptionChanged(int i);
            void viewCountChanged();
            void selectionChanged();

        protected:
            bool eventFilter(QObject *object, QEvent *event);
            void buildView(std::shared_ptr<sh::ui::fileview::FileView>* list, sh::ui::fileview::FileViewControl* viewctrl, bool isNew);
            void setCustomWidget(int i, std::shared_ptr<QWidget> w);

        private:
            Ui::FilesystemPanel *ui;
            std::shared_ptr<sh::ui::fileview::FileView> activeView;
            QString activeColor;
            QString inactiveColor;
            QList<std::shared_ptr<sh::ui::fileview::FileView>> _views;
            QList<QWidget*> _mainviews;
            QHash<QWidget*, std::shared_ptr<QWidget>> _customwidgets;
            QList<sh::ui::fileview::FileViewControl*> _viewcontrols;
            bool _skip_eventfilter = false;
        };

    }
}

#endif // FILESYSTEMPANEL_H
