/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef FILELIST_H
#define FILELIST_H

#include <memory>
#include <QTreeView>
#include "fileview.h"


namespace sh{
    namespace ui {
        namespace fileview {

            /** \brief
             * List view for the contents of one directory.
             */
            class FileList : public QTreeView, public sh::ui::fileview::FileView
            {
                Q_OBJECT
            public:
                /** \brief Constructed only by the infrastructure and made available otherwise.
                 */
                explicit FileList(QWidget *parent = 0);
                virtual void setSort(int column, Qt::SortOrder order);

            private slots:
                void slot_sectionResized(int index, int oldsize, int newsize);

            protected:
                void dragEnterEvent(QDragEnterEvent *event);
                void dragMoveEvent(QDragMoveEvent *event);
                void dragLeaveEvent(QDragLeaveEvent *event);
                void dropEvent(QDropEvent *event);
                void mousePressEvent(QMouseEvent *event);
                void mouseMoveEvent(QMouseEvent *event);
                void keyPressEvent(QKeyEvent *event);
                virtual void _reload_goto_modelloaded();
                virtual void onBackgroundColorSet();

            private:
                static std::shared_ptr<sh::configuration::ConfigurationValue> cfgvalFileListIconSize;
                QString getColumnNameForIndex(int index);
                void _adaptColumnWidth(int index);
                bool _skip_slot_sectionResized = false;
            };

        }
    }
}

#endif // FILELIST_H
