/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef FILEVIEWCONTROL_H
#define FILEVIEWCONTROL_H

#include <memory>
#include <QObject>
#include "base/filesystemmodelfileviewproxy.h"
#include "tools/historytracker.h"


namespace sh{
    namespace base{ class Eurl; class FilesystemNode; }

    namespace ui{
        class ColumnDimensions;
        class FilesystemPanel;
        namespace fileview {

            class FileView;
            class FileViewEvents;
            class FileViewEventsImpl;

            enum class FileViewMode
            {
                IconView,
                ListView
            };

            /** \brief
             * Control interface for a file view.
             *
             * It is used for getting and setting stuff from one of the
             * visible fileviews (e.g. browse to another directory,
             * show hidden files, ...).
             */
            class FileViewControl : public QObject
            {
                friend class sh::ui::FilesystemPanel;
                friend class sh::ui::fileview::FileView;
                Q_OBJECT
            public:
                /** \brief Constructed only by the infrastructure and made available otherwise.
                 */
                FileViewControl(sh::ui::FilesystemPanel* panel, int i);
                ~FileViewControl();
                sh::ui::ColumnDimensions* columndimensions();
                sh::tools::HistoryTracker<std::shared_ptr<const sh::base::Eurl>>* historyTracker();
                FileViewMode viewmode();
                int index();
                sh::ui::fileview::FileViewEvents* events();

                std::shared_ptr<sh::base::FilesystemNode> node();
                void gotoDir(std::shared_ptr<sh::base::FilesystemNode> n);
                sh::base::SizeFormatting getSizeFormattingMode();
                void setSizeFormattingMode(sh::base::SizeFormatting mode);
                bool hiddenFilesVisible();
                void setHiddenFilesVisible(bool v);
                int iconDimension();
                void setIconDimension(int v);

                void setSort(int column, Qt::SortOrder order);
                int sortColumn();
                Qt::SortOrder sortOrder();

            private:
                sh::ui::FilesystemPanel* _panel;
                int i;

                sh::ui::fileview::FileViewEventsImpl* _events;
                sh::ui::ColumnDimensions* _columndimensions;
                sh::tools::HistoryTracker<std::shared_ptr<const sh::base::Eurl>>* _historyTracker;
                FileViewMode _viewmode = FileViewMode::ListView;
                bool _hiddenfilesvisible = false;
                int _icondimension = 0;
                std::shared_ptr<sh::base::FilesystemNode> _node;
                int _sortColumn;
                Qt::SortOrder _sortOrder;
                sh::base::SizeFormatting _sizeformattingmode = sh::base::SizeFormatting::SizeFormattingModePrefixes;
                void setIndex(int i);
            };

        }
    }
}

#endif // FILEVIEWCONTROL_H
