/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef FILEVIEWEVENTS_H
#define FILEVIEWEVENTS_H

#include <QObject>
#include "base/filesystemnode.h"


namespace sh{
    namespace ui {
        namespace fileview {

            class FileView;

            /** \brief
             * Events from the file views.
             *
             * Listen here if you are interested in changes of a fileview.
             * This includes e.g. behavior and the current directory,
             * but is not the right place for watching for changes in the filesystem
             * itsef.
             */
            class FileViewEvents : public QObject
            {
                Q_OBJECT
            public:
                int index();
            protected:
                explicit FileViewEvents(QObject *parent, int index);
                int _index;

            signals:
                void nodesActivated(QList<std::shared_ptr<sh::base::FilesystemNode>> nodes);
                void selectionChanged();
                void viewOptionChanged();
            };

        }
    }
}

#endif // FILEVIEWEVENTS_H
