/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "tools/mainthread.h"
#include "icontextbanner.h"


sh::ui::IconTextBanner::IconTextBanner(QWidget *parent) : QWidget(parent)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    _layout = new QHBoxLayout(this);
    clear();
}

void sh::ui::IconTextBanner::clear()
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    QLayoutItem *child;
    while((child = _layout->takeAt(0)) != 0) {
        if (child->widget())
            delete child->widget();
        delete child;
    }
    _layout->addWidget(new QLabel(this), 1); // a real stretch doesn't seem to work here
}

void sh::ui::IconTextBanner::insertIcon(QIcon icon, int size)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    QLabel* l = new QLabel(this);
    l->setPixmap(icon.pixmap(size, size));
    _layout->insertWidget(_layout->count()-1, l);
}

void sh::ui::IconTextBanner::insertText(QString text)
{
    ONLY_ALLOWED_TO_CALL_FROM_MAIN_THREAD
    QLabel* l = new QLabel(this);
    l->setText(text);
    l->setWordWrap(true);
    _layout->insertWidget(_layout->count()-1, l);
}
