/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef JUMPBAR_H
#define JUMPBAR_H

#include <memory>
#include <QKeyEvent>
#include <QLineEdit>
#include <QPushButton>


namespace Ui {
class JumpBar;
}

namespace sh{
    namespace base{ class Eurl; class FilesystemNode; }

    namespace ui {

        /** \brief Base class of JumpBar.
         */
        class LineEditWithKeyboardShortcuts : public QLineEdit
        {
            Q_OBJECT
        public:
            explicit LineEditWithKeyboardShortcuts(QWidget *parent = 0);
        signals:
            void enterPressed();
            void escapePressed();
        protected:
            void keyPressEvent(QKeyEvent *event);
        };

        /** \brief
         * Button bar for navigating to places with parent-buttons and an text
         * entry.
         */
        class JumpBar : public QWidget
        {
            Q_OBJECT

        public:
            explicit JumpBar(QWidget *parent = 0);
            ~JumpBar();
            void setButtonMode();
            void setTextMode();
            std::shared_ptr<sh::base::FilesystemNode> node();
            void setNode(std::shared_ptr<sh::base::FilesystemNode> node);
            QSize sizeHint() const;

        signals:
            void nodeChanged();
            void eurlRequested(std::shared_ptr<const sh::base::Eurl> eurl);

        protected:
            void resizeEvent(QResizeEvent * event);

        private slots:
            void on_btnOk_clicked();
            void on_btnAbort_clicked();

        private:
            std::shared_ptr<sh::base::FilesystemNode> _node=0;
            Ui::JumpBar *ui;
            bool isTextMode;
            void _buildButtons();
        };

    }
}

#endif // JUMPBAR_H
