/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "configuration/configurationmanager.h"
#include "linkbutton.h"


sh::ui::LinkButton::LinkButton(QWidget *parent) :
    QToolButton(parent)
{
    _linkcolor = sh::configuration::ConfigurationManager::instance()->style()->linkColor().name();
    this->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
    this->setAutoRaise(true);
    setSizeByFactor(90);
}

void sh::ui::LinkButton::setSizeByFactor(int f)
{
    int s = sh::configuration::ConfigurationManager::instance()->style()->fontSizeInPt(f);
    setStyleSheet(QString("QToolButton:hover{text-decoration:underline;} QToolButton:enabled{font-weight:normal; color:%1;font-size:%2pt;}")
                  .arg(_linkcolor).arg(s));
}

void sh::ui::LinkButton::setMenu(QStringList items, QString menuchangetxt)
{
    _menuitems = items;
    _menuchangetxt = menuchangetxt;
    disconnect(this, &LinkButton::clicked, 0, 0);
    connect(this, &LinkButton::clicked, this, [=](){
        QMenu menu;
        foreach(QString s, items)
            menu.addAction(s);
        QAction* s = menu.exec(mapToGlobal(QPoint(0, height())));
        if (s)
        {
            int p = menu.actions().indexOf(s);
            setSelectedMenuItem(p);
            emit menuItemSelected(p);
        }
    });
    setSelectedMenuItem(0);
}

void sh::ui::LinkButton::setSelectedMenuItem(int idx)
{
    setText(QString("%1 %2").arg(_menuitems[idx], _menuchangetxt));
}
