/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef OPENWITHDIALOG_H
#define OPENWITHDIALOG_H

#include <memory>
#include <QAbstractItemModel>
#include <QDialog>
#include <QFileDialog>
#include <QKeyEvent>
#include <QStringListModel>
#include "actions/actionactionitem.h"


namespace Ui {
class OpenWithDialog;
}

namespace sh{
    namespace base { class Eurl; }
    namespace tools { namespace filetypes { class OpenMethod; }}
    namespace ui {

        /** \brief
         * The 'Open with' dialog.
         */
        class OpenWithDialog : public QDialog
        {
            Q_OBJECT

        public:
            explicit OpenWithDialog(std::shared_ptr<const sh::base::Eurl> forDirectory, QWidget *parent = 0);

            void setProgramList(QList<sh::tools::filetypes::OpenMethod*> openMethods);

            ~OpenWithDialog();

            sh::tools::filetypes::OpenMethod* chosenMethod();

            bool rememberForMimetype();
            std::shared_ptr<const sh::base::Eurl> rememberForDirectory();
            bool rememberForFile();

        private slots:
            void on_btnCancel_clicked();
            void on_btnOk_clicked();
            void on_btnSelectOther_clicked();
            void on_listView_activated(const QModelIndex &index);
            void on_btnAnotherAncestor_clicked();
            void on_chkRememberDir_toggled(bool checked);

        private:

            QList<sh::tools::filetypes::OpenMethod*> _openMethods;
            sh::tools::filetypes::OpenMethod* _manualMethod = 0;
            sh::tools::filetypes::OpenMethod* _chosenMethod = 0;
            std::shared_ptr<const sh::base::Eurl> _rememberfordirectory;
            QAbstractItemModel* _listmodel;
            Ui::OpenWithDialog *ui;
        };

        /**
         * @brief Used internally in OpenWithDialogModel.
         */
        class OpenWithDialogModel : public QStringListModel
        {
            Q_OBJECT
        public:
            OpenWithDialogModel(QList<sh::tools::filetypes::OpenMethod*> methods, QObject* parent = 0);
            QVariant data(const QModelIndex &index, int role) const;
        private:
            QList<sh::tools::filetypes::OpenMethod*> _methods;
        };

    }
}

#endif // OPENWITHDIALOG_H
