/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef STOREPROFILEDIALOG_H
#define STOREPROFILEDIALOG_H

#include <memory>
#include <QAbstractButton>
#include <QCompleter>
#include <QDialog>
#include <QInputDialog>
#include <QLabel>
#include <QMap>
#include <QSpacerItem>


namespace Ui {
class StoreProfileDialog;
}

namespace sh {
    namespace configuration {class ConfigurationValue;}
    namespace settings {class Setting;}

    namespace ui {

        class SettingUIFrame;

        /** \brief
         * The 'Save settings' dialog.
         */
        class StoreProfileDialog : public QDialog
        {
            Q_OBJECT

        public:
            class CheckedSetting
            {
            public:
                CheckedSetting(sh::settings::Setting* setting, int onlyInFileview=-1);
                sh::settings::Setting* setting;
                int onlyInFileview;
            };

            explicit StoreProfileDialog(QString eurl, QWidget *parent = 0);
            ~StoreProfileDialog();
            QList<CheckedSetting> getCheckedSettings();
            QString profilename();
            bool withSubdirectories();
            QStringList inheritsFrom();
            bool hasGlobal();

        private slots:
            void on_btnAddProf_clicked();
            void on_btnGlobal_clicked();
            void on_btn22_clicked();
            void on_toolButton_2_clicked();
            void on_toolButton_clicked();

        private:
            void _settingsLevel(int level, bool persist = true);
            void _setglobal(bool v);
            static std::shared_ptr<sh::configuration::ConfigurationValue> cfgvalLevel;
            QMap<sh::ui::SettingUIFrame*, std::shared_ptr<sh::settings::Setting>> _widgets;
            QMap<int, QLabel*> _headerlabels;
            QStringList _inheritProfileNameList;
            bool _global = false;
            Ui::StoreProfileDialog *ui;
            QString _eurl;
            int _level;
        };

    }
}

#endif // STOREPROFILEDIALOG_H
