/*
Shallot is a Qt based file manager.
Copyright (C) 2014-2018 Josef Hahn

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "configuration/configurationmanager.h"
#include "configuration/configurationvaluetype.h"
#include "tools/iconmanager.h"
#include "tuningdialog.h"
#include "ui_tuningdialog.h"


sh::ui::TuningDialog::TuningDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::TuningDialog)
{
    ui->setupUi(this);
    ui->btnCancel->setSizeByFactor(120);
    sh::configuration::ConfigurationManager* sm = sh::configuration::ConfigurationManager::instance();

    ui->btnCancel->setIcon(sh::tools::IconManager::instance()->getIcon("dialog-close"));

    sm->style()->createSheet()
                .background_highlighted()
            .applyTo(ui->layoutAppear)
            .applyTo(ui->layoutBehav)
            .applyTo(ui->layoutExt)
            .applyTo(ui->layoutInt);
    sm->style()->createSheet()
                .background_liketitlebar()
            .applyTo(ui->panelHead);
    sm->style()->createSheet()
                .fontsize_header()
            .applyTo(ui->label);
    sm->style()->createSheet()
                .fontsize_smaller()
            .applyTo(ui->label_2);

    foreach(std::shared_ptr<sh::configuration::ConfigurationValue> cv, sh::configuration::ConfigurationManager::instance()->getAllConfigurationValues())
    {
        QWidget* par = ui->layoutInt;
        if (cv->getCategory()==sh::configuration::ConfigurationCategory::CategoryGUI)
            par = ui->layoutAppear;
        if (cv->getCategory()==sh::configuration::ConfigurationCategory::CategoryBehavior)
            par = ui->layoutBehav;
        if (cv->getCategory()==sh::configuration::ConfigurationCategory::CategoryExternalTools)
            par = ui->layoutExt;
        if (par) {
            QBoxLayout* layout = static_cast<QBoxLayout*>(par->layout());
            QFrame* box = new QFrame(par);
            box->setStyleSheet(QString(".QFrame{border-radius:10px; background:%1;} .QWidget{background:%1;}")
                          .arg(sh::configuration::ConfigurationManager::instance()->style()->windowColorHalfHighlighted().name()));
            QVBoxLayout* blayout = new QVBoxLayout(box);
            QLabel* t = new QLabel(box);
            t->setText(cv->getDescription());
            QWidget* w = cv->getValueType()->gui(cv.get(),box);
            blayout->addWidget(t);
            blayout->addWidget(w);
            blayout->setMargin(10);
            blayout->setSpacing(0);
            box->setLayout(blayout);
            layout->addWidget(box);
        }
    }
    static_cast<QBoxLayout*>(ui->layoutAppear->layout())->addStretch(1);
    static_cast<QBoxLayout*>(ui->layoutBehav->layout())->addStretch(1);
    static_cast<QBoxLayout*>(ui->layoutExt->layout())->addStretch(1);
    static_cast<QBoxLayout*>(ui->layoutInt->layout())->addStretch(1);
    ui->tabWidget->setCurrentIndex(0);
}

sh::ui::TuningDialog::~TuningDialog()
{
    delete ui;
}

void sh::ui::TuningDialog::on_btnCancel_clicked()
{
    close();
}
